
/*
 *  Ump - Unnamed Math Program
 *  Copyright (c) 2004-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */

#ifndef NO_GTK
	#include <gtk/gtk.h>
	#include "ump_picturewin.h"
#endif
#include "addon_functions.h"
#include "main.h"
#include "keyfile.h"
#include <string.h>


extern void (*TEXTVIEW_FUNCTION)(const utf8_string &string);
extern void (*QUIT_FUNCTION)(void);

void term_quit(void);
void term_showtext(const utf8_string &string);



void string_replace_escapes( utf8_string &str ) throw(error_obj)
{
	char tkn[] = " ";

	try
	{
		for( uint32 i=0; i<str.get_length(); i++ )
		{
			if( str.test_character( i, "\0" ) )
				break;
			else if( str.test_character( i, "\\" ) )
			{
				if( str.test_character( i+1, "n" ) )
					tkn[0] = '\n';
				else if( str.test_character( i+1, "\\" ) )
					tkn[0] = '\\';
				else if( str.test_character( i+1, "t" ) )
					tkn[0] = '\t';
				else
					tkn[0] = '\0';
				if( tkn[0] != '\0' )
					str.replace( i, 2, tkn );
			}
		}
	}
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
}


void addon_load_picture( Variable *res, Variable *left, const Variable *right, VariableList *private_varlist ) throw(error_obj)
{
	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: load_picture wasn't called the right way" );

	if( res == 0 )
		return;

	if( right->get_type() != math::VariableType_String )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "load_picture" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	math::Picture pic;

	pic.pic.load_pcx( right->get_string()->c_str() );

	res->set_picture( &pic );
}

void addon_save_picture( Variable *res, Variable *left, const Variable *right, VariableList *private_varlist ) throw(error_obj)
{
	const math::Picture *pic;
	const math::Array *array;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: save_picture wasn't called the right way" );

	if( right->get_type() != math::VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "save_picture" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();

	if(array->get_size() != 2)
		THROW_ERROR( ErrorType_General, _("Wrong number of arguments for 'save_picture'") );

	if( ( array->get_variable(0)->get_type() != math::VariableType_Picture) |
	    ( array->get_variable(1)->get_type() != math::VariableType_String) )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "save_picture" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	pic = array->get_variable(0)->get_picture();

	pic->pic.save_pcx( array->get_variable(1)->get_string()->c_str() );

	if( res != 0 )
		res->set_picture( pic );
}

void addon_load( Variable *res, Variable *left, const Variable *right, VariableList *private_varlist ) throw(error_obj)
{
	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: load wasn't called the right way" );

	if(right->get_type() != math::VariableType_String)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "load" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	math::CodeBlock code;

	code.set_code_from_file( *right->get_string() );

	if( res != 0 )
		res->set_string( "File loaded" );
}

void addon_print(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	utf8_string str;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: print wasn't called the right way" );

	if(right->get_type() != math::VariableType_String)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "print" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	str = *right->get_string();

	string_replace_escapes( str );

	(*TEXTVIEW_FUNCTION)(str);

	if( res != 0 )
		res->set_string( str );
}

void addon_text_message(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const math::Array *array;
	char s[10];
	utf8_string title, msg;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: message wasn't called the right way" );

	if(right->get_type() != math::VariableType_Array)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();
	if( array->get_size() != 2 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	for( int i=0; i<2; i++ )
	{
		if( array->get_variable(i)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	title = *( array->get_variable(0)->get_string() );
	msg = *( array->get_variable(1)->get_string() );

	string_replace_escapes( msg );

	term_showtext( title );
	term_showtext( "\n" );
	term_showtext( msg );

	fgets( s, 2, stdin );

	if( res != 0 )
		res->set_void();
}

void addon_text_menu(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const math::Array *array;
	uint32 index;
	utf8_string ostr;
	char str[25];

	if( left != 0  ||  right == 0  ||  res == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: menu wasn't called the right way" );

	if(right->get_type() != math::VariableType_Array)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();
	if( array->get_size() < 3 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	for( index=0; index<array->get_size(); index++ )
	{
		if( array->get_variable(index)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	term_showtext( *( array->get_variable(0)->get_string() ) );
	term_showtext( "\n\n" );
	term_showtext( *( array->get_variable(1)->get_string() ) );
	term_showtext( "\n\n" );

	for( index=2; index<array->get_size(); index++ )
	{
		snprintf( str, 20, "%lu - ", index-1 );
		ostr = str;
		ostr.append( *( array->get_variable(index)->get_string() ) );
		ostr.append( "\n" );
		term_showtext( ostr.c_str() );
	}
	term_showtext( "? " );
	fgets( str, 20, stdin );
	if( sscanf( str, "%lu", &index ) == 0 )
	{
		res->set_complex( Complex( -1 ) );
		return;
	}

	if( index == 0  ||  index >= (array->get_size() +1) )
	{
		res->set_complex( Complex( -1 ) );
		return;
	}

	res->set_string( *( array->get_variable(index+1)->get_string() ) );
}

void addon_text_input(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const math::Array *array;
	uint32 index;
	char str[500];
	math::Variable *tmp;
	const utf8_string *varname;
	utf8_string input, type_string;
	math::CodeLine code;
	math::Variable var_tmp;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: input wasn't called the right way" );

	if(right->get_type() != math::VariableType_Array)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();
	if( (array->get_size() % 4) != 1 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( array->get_variable(0)->get_type() != math::VariableType_String )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	term_showtext( *( array->get_variable(0)->get_string() ) );
	term_showtext( "\n\n" );

	for( index=1; index<array->get_size(); index += 4 )
	{
		if( array->get_variable(index)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+1)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+2)->has_name() == false )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+3)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		addon_text_input_rerun_input:
		term_showtext( *( array->get_variable(index)->get_string() ) );

		if( array->get_variable(index+3)->get_string()->get_length() > 0 )
		{
			term_showtext( " (" );
			term_showtext( *( array->get_variable(index+3)->get_string() ) );
			term_showtext( "): " );
		}
		else
			term_showtext( ": " );

		fgets( str, 500, stdin );
		for( int i=0; str[i] != '\0'; i++ )
		{
			if( str[i] == '\n'  ||  str[i] == '\r' )
			{
				str[i] = '\0';
				break;
			}
		}

		varname = array->get_variable(index+2)->get_name();
		if( private_varlist )
		{
			try{
				tmp = private_varlist->get_pointer_rw( private_varlist->get_id( (*varname).c_str() ) );
			}
			catch(...) {
				tmp = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( (*varname).c_str() ) );
			}
		}
		else
			tmp = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( (*varname).c_str() ) );

		if( str[0] == '\0' )
		{
			if( array->get_variable(index+3)->get_string()->get_length() == 0 )
				goto addon_text_input_rerun_input;
			input = *( array->get_variable(index+3)->get_string() );
		}
		else
			input = str;

		type_string = *( array->get_variable(index+1)->get_string() );
		if( type_string == "String" )
			tmp->set_string( input );
		else if( type_string == "Integer" )
		{
			Integer int_tmp;

			code.set_code_line( input.c_str() );
			code.calc( &var_tmp, private_varlist );

			math::argument_check( "input", &var_tmp, math::type_integer, &int_tmp, math::type_end );
			tmp->set_complex( Complex( int_tmp ) );
		}
		else if( type_string == "Complex" )
		{
			Complex complex_tmp;

			code.set_code_line( input.c_str() );
			code.calc( &var_tmp, private_varlist );

			math::argument_check( "input", &var_tmp, math::type_complex, &complex_tmp,
			                      math::type_end );
			tmp->set_complex( complex_tmp );
		}
		else
			THROW_ERROR( ErrorType_General, _("Unknown conversion type for function input") );
	}

	if( res != 0 )
		res->set_boolean( true );
}

void addon_viewall(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	if( left != 0  ||  right != 0 )
		THROW_ERROR( ErrorType_General, "Internal error: viewall wasn't called the right way" );

	uint32 i,high;
	const Variable *var;

	if(private_varlist == 0)
		private_varlist = &math::global_varlist;

	high = private_varlist->get_highest_idnr();

	for(i=1;i<=high;i++)
	{
		try{  var = private_varlist->get_pointer_rw(i);  }
		catch(...)
		{
			continue; // try next idnr...
		}

		{
			utf8_string str;
			var->append_to_string( str, format, true );
			(*TEXTVIEW_FUNCTION)( str );
			str = "\n";
			(*TEXTVIEW_FUNCTION)( str );
		}
	}
	(*TEXTVIEW_FUNCTION)("\n");

	if( private_varlist != &math::global_varlist)
		addon_viewall( 0, left, right, 0 );

	if( res != 0 )
		res->set_void();
}

void addon_view(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	utf8_string outstr, trans_str;
	const utf8_string *str;
	const Variable *var;
	uint32 idnr;
	bool deletable;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: view wasn't called the right way" );

	if(right->has_name())
		str = right->get_name();
	else if(right->get_type() == math::VariableType_String)
		str = right->get_string();
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "view" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	idnr = math::global_varlist.get_id(str->c_str());
	var = math::global_varlist.get_pointer(idnr);
	deletable = math::global_varlist.get_deletable(idnr);

	trans_str = "\n";
	trans_str.append( _("Type: ") );
	if( var->get_isconstant() )
		trans_str.append( _("Constant") );
	else
		trans_str.append( _("Variable") );
	(*TEXTVIEW_FUNCTION)( trans_str );

	if( deletable == false )
	{
		trans_str = "\n  ";
		trans_str.append( _("Undeletable") );

		if( var->get_can_change_type() == false )
		{
			trans_str.append( "  " );
			trans_str.append( _("Type-locked") );
		}
		(*TEXTVIEW_FUNCTION)( trans_str );
	}
	else if( var->get_can_change_type() == false )
	{
		trans_str = "\n  ";
		trans_str.append( _("Type-locked") );
		(*TEXTVIEW_FUNCTION)( trans_str );
	}

	trans_str = "\n";
	trans_str.append( _("Sub-type: ") );
	(*TEXTVIEW_FUNCTION)( trans_str );
	(*TEXTVIEW_FUNCTION)(math::VARIABLE_TYPE_NAMES[var->get_type()]);

	outstr = "\n\n";
	(*TEXTVIEW_FUNCTION)( outstr );
	var->append_to_string( outstr, format, false );
	(*TEXTVIEW_FUNCTION)( outstr );
	outstr = "\n\n";
	(*TEXTVIEW_FUNCTION)( outstr );

	if( res != 0 )
		res->set_void();
}

void addon_quit(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	(*QUIT_FUNCTION)();
}

#ifndef NO_GTK
void addon_show(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const math::Picture *pic;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: show wasn't called the right way" );

	if( right->get_type() != math::VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "show" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	pic = right->get_picture();

	if( right->has_name() )
	{
		utf8_string str;

		str = *right->get_name();
		str.append( " - " );
		str.append( PRG_NAME );

		picturewin_show( str.c_str(), *pic );
	}
	else
		picturewin_show( PRG_NAME, *pic );

	if( res != 0 )
		*res = *right;
}

void addon_message(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const math::Array *array;
	GtkWidget *dialog;
	utf8_string title, msg;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: message wasn't called the right way" );

	if( right->get_type() != math::VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();
	if( array->get_size() != 2 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	for( int i=0; i<2; i++ )
	{
		if( array->get_variable(i)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "message" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	title = *( array->get_variable(0)->get_string() );
	msg = *( array->get_variable(1)->get_string() );

	string_replace_escapes( msg );

	dialog = gtk_message_dialog_new( NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
	                                 "%s", msg.c_str() );

	gtk_window_set_title( GTK_WINDOW(dialog), title.c_str() );

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if( res != 0 )
		res->set_void();
}

void addon_menu(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	GtkWidget *dialog, *label, *align, *vbox, **radio;
	uint32 index;
	const math::Array *array;

	if( left != 0  ||  right == 0  ||  res == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: menu wasn't called the right way" );

	if( right->get_type() != math::VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();
	if( array->get_size() < 3 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	for( index=0; index<array->get_size(); index++ )
	{
		if( array->get_variable(index)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "menu" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	try
	{
		radio = new GtkWidget* [array->get_size()-2];
	}
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }

	dialog = gtk_dialog_new_with_buttons( NULL, NULL, GTK_DIALOG_MODAL,
	                                      GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

	align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
	gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0);

	vbox = gtk_vbox_new( FALSE, 3 );
	gtk_container_add( GTK_CONTAINER(align), vbox );

	gtk_window_set_title( GTK_WINDOW(dialog), array->get_variable(0)->get_string()->c_str() );

	label = gtk_label_new( array->get_variable(1)->get_string()->c_str() );
	gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5 );
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);


	for( index=2; index<array->get_size(); index++ )
	{
		if( index == 2 )
		{
			radio[0] = gtk_radio_button_new_with_label( NULL,
			  array->get_variable(index)->get_string()->c_str() );
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radio[0]), TRUE );
		}
		else
			radio[index-2] = gtk_radio_button_new_with_label_from_widget( 
			  GTK_RADIO_BUTTON(radio[0]), array->get_variable(index)->get_string()->c_str() );

		gtk_box_pack_start(GTK_BOX(vbox), radio[index-2], FALSE, FALSE, 0);
	}

	gtk_widget_show_all(dialog);

	switch(gtk_dialog_run( GTK_DIALOG(dialog) ))
	{
	case GTK_RESPONSE_OK:

		for( index=2; index<array->get_size(); index++ )
		{
			if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radio[index-2]) ) == TRUE)
				break;
		}
		break;

	default:
		gtk_widget_destroy(dialog);
		delete [] radio;

		res->set_complex( Complex(-1) );
		return;
	}
	gtk_widget_destroy(dialog);
	delete [] radio;

	res->set_string( *( array->get_variable(index)->get_string() ) );
}

void addon_input(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	GtkWidget *dialog, *label, *align, *table, **entry;
	uint32 index;
	const math::Array *array;
	utf8_string input, type_string;
	const utf8_string *varname;
	math::Variable *tmp, var_tmp;
	math::CodeLine code;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: input wasn't called the right way" );

	if(right->get_type() != math::VariableType_Array)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	array = right->get_array();
	if( (array->get_size() % 4 ) != 1 )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( array->get_variable(0)->get_type() != math::VariableType_String )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	for( index=1; index<array->get_size(); index += 4 )
	{
		if( array->get_variable(index)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+1)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+2)->has_name() == false )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		if( array->get_variable(index+3)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "input" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	try
	{
		entry = new GtkWidget* [array->get_size() / 4];
	}
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }

	try
	{    
		dialog = gtk_dialog_new_with_buttons( NULL, NULL, GTK_DIALOG_MODAL,
		                                      GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

		gtk_window_set_title( GTK_WINDOW(dialog), array->get_variable(0)->get_string()->c_str() );

		align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
		gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0);

		table = gtk_table_new ( array->get_size() / 4, 2, FALSE );
		gtk_table_set_row_spacings( GTK_TABLE(table), 3 );
		gtk_table_set_col_spacings( GTK_TABLE(table), 3 );

		gtk_container_add( GTK_CONTAINER(align), table );

		for( index=0; index<(array->get_size()-1); index += 4 )
		{
			label = gtk_label_new( array->get_variable(index+1)->get_string()->c_str() );
			gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5 );
			gtk_table_attach( GTK_TABLE(table), label, 0, 1, index/4, index/4+1, GTK_FILL,
			                  GTK_FILL, 0, 0 );

			entry[index/4] = gtk_entry_new();
			gtk_entry_set_max_length( GTK_ENTRY(entry[index/4]), 500 ); 
			gtk_entry_set_activates_default( GTK_ENTRY(entry[index/4]), FALSE );
			if( array->get_variable(index+4)->get_string()->get_length() > 0 )
				gtk_entry_set_text( GTK_ENTRY(entry[index/4]), 
				                    array->get_variable(index+4)->get_string()->c_str() );
			else
				gtk_entry_set_text( GTK_ENTRY(entry[index/4]), "" );
			gtk_table_attach( GTK_TABLE(table), entry[index/4], 1, 2, index/4, index/4+1, GTK_FILL,
			                  GTK_FILL, 0, 0 );
		}
		gtk_widget_show_all(dialog);

		switch(gtk_dialog_run( GTK_DIALOG(dialog) ))
		{
		case GTK_RESPONSE_OK:

			for( index=0; index<(array->get_size()-1); index += 4 )
			{
				varname = array->get_variable(index+3)->get_name();
				if(private_varlist)
				{
					try{
						tmp = private_varlist->get_pointer_rw( private_varlist->get_id(
						               (*varname).c_str() ) );
					}
					catch(...) {
						tmp = math::global_varlist.get_pointer_rw( math::global_varlist.get_id(
						               (*varname).c_str() ) );
					}
				}
				else
					tmp = math::global_varlist.get_pointer_rw( math::global_varlist.get_id(
					               (*varname).c_str() ) );

				input = gtk_entry_get_text( GTK_ENTRY(entry[index/4]) );

				type_string = *( array->get_variable(index+2)->get_string() );
				if( type_string == "String" )
					tmp->set_string( input );
				else if( type_string == "Integer" )
				{
					Integer int_tmp;

					code.set_code_line( input.c_str() );
					code.calc( &var_tmp, private_varlist );

					math::argument_check( "input", &var_tmp, math::type_integer, &int_tmp,
					                      math::type_end );
					tmp->set_complex( Complex( int_tmp ) );
				}
				else if( type_string == "Complex" )
				{
					Complex complex_tmp;

					code.set_code_line( input.c_str() );
					code.calc( &var_tmp, private_varlist );

					math::argument_check( "input", &var_tmp, math::type_complex, &complex_tmp,
					                      math::type_end );
					tmp->set_complex( complex_tmp );
				}
				else
					THROW_ERROR( ErrorType_General, _("Unknown conversion type in function input") );
			}
			break;

		default:
			gtk_widget_destroy(dialog);
			delete [] entry;
			entry = 0;

			if( res != 0 )
				res->set_boolean( false );
			return;
		}
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		gtk_widget_destroy(dialog);
		delete [] entry;
		throw;
	}
	delete [] entry;

	if( res != 0 )
		res->set_boolean( true );
}

#endif
