
/*
 *  inliner - This is a small program to make files inline C++ files
 *  Copyright (c) 2005 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*
	This is a command line program.
	usage:  inliner  input_file  variable_name  output_file



*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>



int main(int argc, char **argv)
{
	char *in_file, *variable_name, *out_file, number[]="\\000\0";
	unsigned char *in_buf;
	FILE *fp;
	long filesize, tmp, i, length;
	
	if( argc != 4 )
	{
		printf("  usage:  inliner  input_file  variable_name  output_file\n");
		return -1;
	}
	
	in_file = argv[1];
	variable_name = argv[2];
	out_file = argv[3];

	fp = fopen( in_file, "rb" );
	if(fp == 0)
	{
failed_input:
		if(fp != 0)
			fclose(fp);

		printf("inliner: failed to read file\n");
		return -1;
	}

	if( fseek( fp, 0L, SEEK_END ) != 0 )
		goto failed_input;
	filesize = ftell(fp);
	if( filesize == -1 )
		goto failed_input;
	if( fseek( fp, 0L, SEEK_SET ) != 0 )
		goto failed_input;

	if( (in_buf = (unsigned char *) malloc( filesize )) == 0 )
		goto failed_input;
	
	if( ((long) fread( in_buf, 1, filesize, fp )) != filesize )
	{
		free(in_buf);
		goto failed_input;
	}
	if(fclose(fp) != 0)
	{
		free(in_buf);
		goto failed_input;
	}
// in_file is now loaded

	fp = fopen( out_file, "wb" );
	if(fp == 0)
	{
failed_output:
		if(fp != 0)
			fclose(fp);

		free(in_buf);
		printf("inliner: failed to write file\n");
		return -1;
	}

	if( fwrite( "/* This file is generated from ", 1, 31, fp ) != 31 )
		goto failed_output;

	if( fwrite( in_file, 1, strlen(in_file), fp ) != strlen(in_file) )
		goto failed_output;
	
	if( fwrite( " */\n\nchar *", 1, 11, fp ) != 11 )
		goto failed_output;
	
	if( fwrite( variable_name, 1, strlen(variable_name), fp ) != strlen(variable_name) )
		goto failed_output;
	
	if( fwrite( " =\n\"", 1, 4, fp ) != 4 )
		goto failed_output;

	for( i=0; i<filesize; i++ )
	{
		if( i != 0  &&  (i % 16) == 0 )
			if( fwrite( "\"\n\"", 1, 3, fp ) != 3 )
				goto failed_output;
		tmp = (long) in_buf[i];
		
		if( tmp >= 64 )
		{
			number[1] = ((char) ( ((long) '0') + tmp / 64 ) );
			tmp %= 64;
			number[2] = ((char) ( ((long) '0') + tmp / 8 ) );
			tmp %= 8;
			number[3] = ((char) ( ((long) '0') + tmp ) );
			length = 4;
		}
		else if( tmp >= 8 )
		{
			number[1] = ((char) ( ((long) '0') + tmp / 8 ) );
			tmp %= 8;
			number[2] = ((char) ( ((long) '0') + tmp ) );
			length = 3;
		}
		else
		{
			number[2] = ((char) ( ((long) '0') + tmp ) );
			length = 2;
		}
		
		if( fwrite( number, 1, length, fp ) != length )
			goto failed_output;
	}
	if( fwrite( "\";\n", 1, 3, fp ) != 3 )
		goto failed_output;

	if(fclose(fp) != 0)
	{
		free(in_buf);
		goto failed_output;
	}
	return 0;
}
