
/*
 *  Integer - Implements an integer of arbitrary size
 *  Copyright (c) 2005-2007 by Mattias Hultgren <tilda_o_tize@hotmail.com>
 *
 *  See integer.h
 */


#include "integer.h"
#include "keyfile.h"
#include <math.h>
#include <stdio.h>
#include <string.h>



void Integer::append_to_string( utf8_string &str, uint32 nrofdigits ) const throw(error_obj)
{
	utf8_string str1(""), str2(""), *ptr, *ptr2, *tmp_ptr;
	Integer nr, rest, divider, tmp;
	char char_str[25];

	divider = int64(1000000000)*int64(1000000000);

	ptr = &str1;
	ptr2 = &str2;
	if( used == 0 )
		str1 = "0";
	else
	{
		if( used >= 5 )
		{
			uint32 rest_digits;

			rest = int32( float((used-1)/4 +1)*1.07 );
			rest_digits = rest.data[0]*18;

			divider.power( rest, &tmp );
			this->divide( tmp, &nr, &rest );
			rest.negative = false;

			if( nrofdigits < rest_digits )
				nr.append_to_string( str );
			else
				nr.append_to_string( str, nrofdigits - rest_digits );
			rest.append_to_string( str, rest_digits );
			return;
		}
		else
		{
			nr = *this;

			do
			{
				nr.divide( divider, &tmp, &rest );
				nr.swap( tmp );

				if( nr == 0 )
				{
					if( rest.used == 1 )
						uint64_to_char( uint64(rest.data[0]), char_str );
					else
						uint64_to_char( uint64(rest.data[0])+(uint64(rest.data[1])<<32), char_str );
				}
				else
				{
					if( rest.used == 0 )
						uint64_to_char( uint64(0), char_str, 18 );
					else if( rest.used == 1 )
						uint64_to_char( uint64(rest.data[0]), char_str, 18 );
					else
						uint64_to_char( uint64(rest.data[0]) + (uint64(rest.data[1])<<32), char_str, 18 );
				}
				tmp_ptr = ptr;
				ptr = ptr2;
				ptr2 = tmp_ptr;

				*ptr = char_str;
				ptr->append( *ptr2 );
			}while( nr != 0 );

			if( negative )
			{
				tmp_ptr = ptr;
				ptr = ptr2;
				ptr2 = tmp_ptr;

				*ptr = "-";
				ptr->append( *ptr2 );
			}
		}
	}

	if( ptr->test_character( 0, "-" ) == false )
	{
		if( ptr->get_length() < nrofdigits ) // the number of digit is not met,
		{                                    // lets add some starting zero's
			tmp_ptr = ptr;
			ptr = ptr2;
			ptr2 = tmp_ptr;

			*ptr = "";
			while( ptr2->get_length() < nrofdigits )
			{
				nrofdigits--;
				ptr->append( "0" );
			}
			ptr->append( *ptr2 );
		}
	}

	str.append( *ptr );
}

// gcd - greatest common divisior (using Euklides algorithm)
// if any of nr1 & nr2 is zero the return value is zero also
Integer gcd( Integer nr1, Integer nr2 ) throw(error_obj)
{
	if( nr1.used == 0 )
		return nr2;
	if( nr2.used == 0 )
		return nr1;

	if( nr1.negative )
		nr1.negate();
	if( nr2.negative )
		nr2.negate();

	{
		Integer tmp;

		if(nr2 > nr1) // this makes nr1 to be the largest number
			goto gcd_nr2_is_bigger;
		for(;;)
		{
			nr1.divide( nr2, 0, &tmp );
			if(tmp.used == 0)
				return nr2;
			nr1.swap( tmp );

gcd_nr2_is_bigger:
			nr2.divide( nr1, 0, &tmp );
			if(tmp.used == 0)
				return nr1;
			nr2.swap( tmp );
		}
	}
}
// lcm - least common multiple ( (nr1*nr2)/gcd(nr1,nr2) )
// if(gcd(nr1,nr2) == 0) the return value is zero also
Integer lcm( const Integer &nr1, const Integer &nr2 ) throw(error_obj)
{
	Integer tmp;

	tmp = gcd(nr1,nr2);
	if(tmp == 0)
		return 0;

	return (nr1 / tmp) * nr2;
}

Integer::Integer(floatx src) throw(error_obj)
{
	uint32 i;
	floatx tmp,tmp2;

	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src <= -1.0 )
		size = uint32( (log2x( -src ) + 1.0) / 32.0 ) + 1;
	else if( src < 1.0  &&  src > -1.0 )
	{
		size = 0;
		return;
	}
	else
		size = uint32( (log2x( src ) + 1.0) / 32.0 ) + 1;

	if( size > 2 )
	{
		try{  data = new uint32[size];  }
		catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
	}
	else
		data = buf;

	used = size;
	if( src < 0.0 )
	{
		negative = true;
		src = -src;
	}
	src = truncx( src );

	for( i=1; i<=size; i++ )
	{
		if( size == i )
			tmp2 = truncx(src);
		else
		{
			tmp = powx( (floatx(uint32_max)+1.0), floatx(size-i) );
	
			tmp2 = truncx(src/tmp);
		}
		data[size-i] = uint32( tmp2 );
	
		src = truncx( src - tmp2*tmp );
	}

	if( data[used-1] == 0 )
		used--;
}
Integer::operator floatx(void)
{
	floatx res;

	switch( used )
	{
	case 0:
		return 0.0;

	case 1:
		res = floatx( *data );
		break;

	default:
		res = 0.0;
		for( uint32 i=0; i < used  &&  i < 3 ; i++ )
			res += floatx(data[used-i-1]) * powx( float(uint32_max)+1.0f, floatx(used-i-1) );
	}

	if( negative )
		return -res;
	 return res;
}
Integer & Integer::operator=(floatx val) throw(error_obj)
{
	uint32 i, new_size;
	floatx tmp,tmp2;

	if( val <= -1.0 )
		new_size = uint32( (log2x( -val ) + 1.0) / 32.0 ) + 1;
	else if( val < 1.0  &&  val > -1.0 )
	{
		used = 0;
		return *this;
	}
	else
		new_size = uint32( (log2x( val ) + 1.0) / 32.0 ) + 1;

	if( new_size > size )
	{
		if( size > 2 )
			delete [] data;
		data = 0;
		used = 0;
		size = 0;

		if( new_size > 2 )
		{
			try{  data = new uint32[new_size];  }
			catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
		}
		else
			data = buf;

		size = new_size;
	}

	used = new_size;
	if( val < 0.0 )
	{
		negative = true;
		val = -val;
	}
	val = truncx( val );

	for( i=1; i<=new_size; i++ )
	{
		if( new_size == i )
			tmp2 = truncx(val);
		else
		{
			tmp = powx( (floatx(uint32_max)+1.0), floatx(new_size-i) );

			tmp2 = truncx(val/tmp);
		}
		data[new_size-i] = uint32( tmp2 );

		val = truncx( val - tmp2*tmp );
	}

	if( data[used-1] == 0 )
		used--;
	return *this;
}




Integer::Integer()
{
	negative = false;
	size = 0;
	data = 0;
	used = 0;
}

Integer::Integer( const Integer &src ) throw(error_obj)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	this->operator=( src );
}

Integer::Integer(int32 src)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src == 0 )
		return;

	data = buf;

	size = 1;
	used = 1;
	if( src < 0 )
	{
		negative = true;
		data[0] = uint32( -src );
	}
	else
	{
		negative = false;
		data[0] = uint32( src );
	}
}

Integer::Integer(int64 src)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src == 0 )
		return;

	data = buf;

	size = 2;
	used = 2;

	if( src < 0 )
	{
		negative = true;
		data[1] = uint32( uint64(-src) >> 32 );
		data[0] = uint32( uint64(-src) );
	}
	else
	{
		negative = false;
		data[1] = uint32( uint64(src) >> 32 );
		data[0] = uint32( uint64(src) );
	}
	if( data[1] == 0 )
		used = 1;
}

Integer::~Integer()
{
	if( size > 2 )
		delete [] data;
}

Integer::operator int64(void)
{
	int64 res;

	if( used == 0 )
		return int64(0);

	if( used >= 3 )
		res = int64_max;
	else
	{
		res = data[0];

		if( used == 2 )
		{
			if( data[1] >= (uint32(1)<<31) )
				res = int64_max;
			else
				res += int64(data[1])*(int64(1)<<32);
		}
	}

	if( negative )
		return -res;
	return res;  
}

Integer & Integer::operator=(int32 src)
{
	if( src == 0 )
	{
		used = 0;
		return *this;
	}

	enlarge_to( 1 );

	used = 1;
	if( src < 0 )
	{
		negative = true;
		data[0] = uint32( -src );
	}
	else
	{
		negative = false;
		data[0] = uint32( src );
	}
	return *this;
}

Integer & Integer::operator=(int64 src)
{
	if( src == 0 )
	{
		used = 0;
		return *this;
	}

	enlarge_to( 2 );

	used = 2;

	if( src < 0 )
	{
		negative = true;
		data[1] = uint32( uint64(-src) >> 32 );
		data[0] = uint32( uint64(-src) );
	}
	else
	{
		negative = false;
		data[1] = uint32( uint64(src) >> 32 );
		data[0] = uint32( uint64(src) );
	}
	if( data[1] == 0 )
		used = 1;

	return *this;
}

Integer & Integer::operator=(const Integer &val) throw(error_obj)
{
	if( size < val.used )
	{
		if( size > 2 )
			delete [] data;
		size = 0;
		used = 0;
		data = 0;

		if( val.used > 2 )
		{
			try{  data = new uint32[val.used];  }
			catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
		}
		else
			data = buf;

		size = val.used;
	}
	used = val.used;
	negative = val.negative;

	for( uint32 i=0; i<used; i++ )
		data[i] = val.data[i];

	return *this;
}

bool Integer::operator==(const Integer &val) const
{
	if( used != val.used )
		return false;
	if( used == 0 )
		return true;

	if( negative != val.negative )
		return false;

	for( uint32 i=0; i<used; i++ )
	{
		if( data[i] != val.data[i] )
			return false;
	}
	return true;
}

bool Integer::operator==(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
		return (val == 0);

	if( used > 1 )
		return false;

	if( negative )
	{
		if( val < 0 )
			tmp = uint32(-int64(val));
		else
			return false;
	}
	else
	{
		if( val < 0 )
			return false;
		tmp = uint32(val);
	}

	return ( data[0] == tmp );
}

// -1 this is smaller
//  0 they are equal
//  1 this is bigger
int Integer::compare_abs_value( const Integer &val ) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return 0;
		return -1;
	}

	if( used < val.used )
		return -1;
	if( used > val.used )
		return 1;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
			return ( data[i-1] < val.data[i-1] ) ? -1 : 1;
	}
	return 0;
}

int Integer::compare_abs_value_shifted( const Integer &val, uint32 shift ) const
{
	uint32 i;
	if( used == 0 )
	{
		if( val.used == 0 )
			return 0;
		return -1;
	}
	if( val.used == 0 )
		return 1;

	if( used < (val.used+shift) )
		return -1;
	if( used > (val.used+shift) )
		return 1;

	for( i=used; i>shift; i-- )
	{
		if( data[i-1] != val.data[i-shift-1] )
			return ( data[i-1] < val.data[i-shift-1] ) ? -1 : 1;
	}
	for( ; i>0; i-- )
	{
		if( data[i-1] != 0 )
			return 1;
	}
	return 0;
}

bool Integer::operator<(const Integer &val) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return false;

		return !val.negative;
	}

	if( negative != val.negative )
		return negative;

	if( used < val.used )
		return !negative;
	if( used > val.used )
		return negative;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
		{
			if( negative )
				return ( data[i-1] > val.data[i-1] );
			return ( data[i-1] < val.data[i-1] );
		}
	}

	return false;
}

bool Integer::operator<(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
	{
		if( val == 0 )
			return false;

		return (val > 0);
	}

	if( negative != (val < 0) )
		return negative;

	if( used > 1 )
		return false;

	if( val < 0 )
		tmp = uint32( (-int64(val))+int64(1) );
	else
		tmp = uint32(val);

	if( negative )
		return ( data[0] > tmp );
	return ( data[0] < tmp );
}

bool Integer::operator>(const Integer &val) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return false;

		return val.negative;
	}

	if( negative != val.negative )
		return !negative;

	if( used < val.used )
		return negative;
	if( used > val.used )
		return !negative;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
		{
			if( negative )
				return ( data[i-1] < val.data[i-1] );
			return ( data[i-1] > val.data[i-1] );
		}
	}

	return false;
}

bool Integer::operator>(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
	{
		if( val == 0 )
			return false;

		return (val < 0);
	}

	if( negative != (val < 0) )
		return !negative;

	if( used > 1 )
		return true;

	if( val < 0 )
		tmp = uint32( (-int64(val))+int64(1) );
	else
		tmp = uint32(val);

	if( negative )
		return ( data[0] < tmp );
	return ( data[0] > tmp );
}

Integer operator-(const Integer &val) throw(error_obj)
{
	Integer tmp;

	tmp = val;
	tmp.negative = !tmp.negative;

	return tmp;
}

void Integer::enlarge_to(uint32 new_size) throw(error_obj)
{
	if( size < new_size )
	{
		if( new_size > 2 )
		{
			uint32 *new_data;
			bool use_delete = ( size > 2 );

			try{  new_data = new uint32[new_size];  }
			catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }

			size = new_size;

			for( uint32 i=0; i<used; i++ )
				new_data[i] = data[i];

			if( use_delete )
				delete [] data;
			data = new_data;
		}
		else
		{
			size = new_size;
			data = buf;
		}
	}
}

Integer & Integer::operator++(void) throw(error_obj)
{
	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		negative = false;
		return *this;
	}

	if( negative )
	{
		for( uint32 i=0; ; i++ )
		{
			data[i]--;
			if( data[i] != uint32_max )
				break;
		}
		if( data[used-1] == 0 )
			used--;
	}
	else
	{
		if( data[used-1] == uint32_max )
			enlarge_to( used+1 );

		for( uint32 i=0; ; i++ )
		{
			if( i == used )
			{
				data[i] = 1;
				used++;
				break;
			}
			data[i]++;
			if( data[i] != 0 )
				break;
		}
	}

	return *this;
}

Integer & Integer::operator--(void) throw(error_obj)
{
	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		negative = true;
		return *this;
	}
	else if( data[used-1] == uint32_max )
		enlarge_to( used+1 );

	if( !negative )
	{
		for( uint32 i=0; ; i++ )
		{
			data[i]--;
			if( data[i] != uint32_max )
				break;
		}
		if( data[used-1] == 0 )
			used--;
	}
	else
	{
		for( uint32 i=0; ; i++ )
		{
			if( i == used )
			{
				data[i] = 1;
				used = i+1;
				break;
			}
			data[i]++;
			if( data[i] != 0 )
				break;
		}
	}

	return *this;
}

void Integer::add_data( const Integer &nr ) throw(error_obj)
{
	uint32 i;

	if( nr.used == 0 ) // nothing to add
		return;

	if( nr.used >= size )
	{
		enlarge_to( nr.used + 1 );
		for( i=used; i<(nr.used+1); i++ )
			data[i] = 0;
	}
	else if( size == used )
	{
		enlarge_to( size + 1 );
		data[used] = 0;
		used++;
	}

	if( used <= nr.used )
	{
		for( i=used; i<=nr.used; i++ )
			data[i] = 0;
		used = nr.used + 1;
	}

	{
		uint32 overflow=0, low_used, *ptr, *ptr2;

		ptr = data;
		ptr2 = nr.data;
		low_used = ( used < nr.used ) ? used : nr.used;
		for( i=0; i<low_used; i++, ptr++, ptr2++ )
		{
			if( (*ptr += *ptr2 + overflow) <= *ptr2 )
			{
				if( *ptr < *ptr2 )
					overflow = 1;
				//else // *ptr == *ptr2
				//	overflow = overflow;
			}
			else
				overflow = 0;
		}
		if( overflow )
		{
			i--;
			do
			{
				i++;
				data[i]++;
			}while( data[i] == 0 );
		}
	}

	if( used != 0 )
		if( data[used-1] == 0 )
			used--;
}

void Integer::add_data_uint32_shifted( const Integer &nr, uint32 shift) throw(error_obj)
{
	uint32 i;

	if( nr.used == 0 ) // nothing to add
		return;

	if( (nr.used+shift) >= size )
	{
		enlarge_to( nr.used + shift + 1 );
		for( i=used; i<(nr.used+shift+1); i++ )
			data[i] = 0;
	}
	else if( size == used )
	{
		enlarge_to( size + 1 );
		data[used] = 0;
		used++;
	}

	if( used <= (nr.used+shift) )
	{
		for( i=used; i<=(nr.used+shift); i++ )
			data[i] = 0;
		used = nr.used + shift + 1;
	}

	{
		uint32 overflow=0, low_used, *ptr, *ptr2;

		ptr = &data[shift];
		ptr2 = nr.data;
		low_used = ( used < nr.used ) ? used : nr.used;
		for( i=0; i<low_used; i++, ptr++, ptr2++ )
		{
			if( (*ptr += *ptr2 + overflow) <= *ptr2 )
			{
				if( *ptr < *ptr2 )
					overflow = 1;
				//else // *ptr == *ptr2
				//	overflow = overflow;
			}
			else
				overflow = 0;
		}
		if( overflow )
		{
			do
			{
				(*ptr)++;
			}while( *ptr++ == 0 );
		}
	}

	if( used != 0 )
		if( data[used-1] == 0 )
			used--;
}

void Integer::sub_data_reverse( const Integer &nr )
{
	Integer res;
	res = nr;
	res.sub_data( *this );
	*this = res;
}

//
// this must be larger than nr....no checking for that will be done
//
void Integer::sub_data( const Integer &nr )
{
	uint32 i;

	{
		uint64 tmp, mask, borrowed=0;
		uint32 *ptr, *ptr2;
		mask = uint64( uint32_max );

		ptr = data;
		ptr2 = nr.data;
		for( i=0; i<nr.used; i++, ptr++, ptr2++ )
		{
			tmp = uint64(*ptr) - uint64(*ptr2) - borrowed;
			if( tmp > mask )
			{
				borrowed = 1;
				tmp &= mask;
			}
			else
				borrowed = 0;

			*ptr = uint32(tmp);
		}
		if( borrowed )
		{
			for( ; ; i++ )
			{
				data[i]--;
				if( data[i] != uint32_max )
					break;
			}
		}
	}

	for( i=used-1; i != uint32_max; i-- )
	{
		if( data[i] != 0 )
			break;
	}
	used = i+1;
}

Integer Integer::operator+(const Integer &val) const throw(error_obj)
{
	Integer res;

	if( negative == val.negative )
	{
		res = *this;
		res.add_data( val );
		res.negative = negative;
	}
	else
	{
		if( val.negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = false;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = true;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = false;
			}
		}
	}
	return res;
}

Integer Integer::operator-(const Integer &val) const throw(error_obj)
{
	Integer res;

	if( negative != val.negative )
	{
		res = *this;
		res.add_data( val );
		res.negative = negative;
	}
	else
	{
		if( !negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = false;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = true;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = false;
			}
		}
	}
	return res;
}

void Integer::mul_by_2(void) throw(error_obj)
{
	if( used == 0 )
		return;

	if( data[used-1] & (uint32(1)<<31) )
	{
		enlarge_to( used + 1 );
		data[used] = 0;
		used++;
	}

	for( uint32 i=used-1; i>0; i-- )
	{
		data[i] <<= 1;
		if( data[i-1] & (uint32(1)<<31) )
			data[i]++;
	}
	data[0] <<= 1;
}

void Integer::mul_by_2_pow_32(uint32 n) throw(error_obj)
{
	if( used == 0  ||  n == 0 )
		return;

	if( size < (used+n) )
	{
		uint32 *new_data;
		bool use_delete = ( size > 2 );

		try{  new_data = new uint32[used+n+1];  }
		catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }

		size = used+n+1;

		for( uint32 i=used+n-1; i>(n-1); i-- )
			new_data[i] = data[i-n];

		for( uint32 i=0; i<n; i++ )
			new_data[i] = 0;

		used += n;
		if( use_delete )
			delete [] data;
		data = new_data;
	}
	else
	{
		for( uint32 i=used+n-1; i>(n-1); i-- )
			data[i] = data[i-n];

		for( uint32 i=0; i<n; i++ )
			data[i] = 0;
		used += n;
	}
}

void Integer::div_by_2(void) throw(error_obj)
{
	if( used == 0 )
		return;

	data[0] >>= 1;
	for( uint32 i=1; i<used; i++ )
	{
		data[i-1] += ( (data[i] & uint32(1)) << 31);
		data[i] >>= 1;
	}

	if( data[used-1] == 0 )
		used--;
}

void Integer::div_by_2_pow_32(void) throw(error_obj)
{
	if( used == 0 )
		return;

	used--;
	for( uint32 i=0; i<used; i++ )
		data[i] = data[i+1];
}

// special function, used by divide      *this -= nr * 2^(32*shift) * mult
// this must be larger than   nr * 2^(32*shift) * mult
void Integer::sub_uint32_mult_shifted( uint32 mult, const Integer &nr, uint32 shift )
{
	uint64 tmp, mult64, mask, borrowed=0, tmp_hi;
	uint32 i, *ptr=&data[shift], *ptr2=nr.data;

	mult64 = uint64(mult);
	mask = uint64( uint32_max );

	for( i=nr.used; i; i-- )
	{
		tmp = mult64 * uint64(*ptr2++);
		if( tmp > mask )
		{
			tmp_hi = tmp >> 32;
			tmp &= mask;
		}
		else
			tmp_hi = 0;

		tmp = uint64(*ptr) - tmp - borrowed;
		if( tmp > mask )
		{
			borrowed = 1;
			tmp &= mask;
		}
		else
			borrowed = 0;

		*ptr++ = uint32(tmp);

		if( tmp_hi )
		{
			tmp = uint64(*ptr) - tmp_hi;
			if( tmp > mask )
			{
				*ptr = uint32(tmp & mask);
				for( uint32 *ptr3=ptr+1; ;  )
				{
					(*ptr3)--;
					if( *ptr3++ != uint32_max )
						break;
				}
			}
			else
				*ptr = uint32(tmp);
		}
	}
	if( borrowed )
	{
		for(;;)
		{
			(*ptr)--;
			if( *ptr++ != uint32_max )
				break;
		}
	}
	while( used != 0 )
	{
		if( data[used-1] == 0 )
			used--;
		else
			break;
	}
}

// special function, used by divide      *this += nr * 2^(32*shift)
// this must be large enough and the data should be zero filled
void Integer::add_uint32_shifted( uint32 nr, uint32 shift ) throw(error_obj)
{
	data[shift] += nr;
	if( data[shift] < nr ) // overflowed
	{
		do{
			shift++;
			data[shift]++;
		}while( data[shift] == 0 );
	}
}

void Integer::divide( const Integer &divider, Integer *ans, Integer *rest ) const throw(error_obj)
{
	if( divider.used == 0 )
		THROW_ERROR( ErrorType_Divide_by_zero, _("Divide by zero") );

	if( ans == 0  &&  rest == 0 )
		return;

	if( ans )
		*ans = 0;

	if( this->compare_abs_value( divider ) < 0 )
	{
		if( rest )
			*rest = *this;
		return;
	}

	if( ans )
	{
		ans->enlarge_to( used - divider.used + 1 );

		for( uint32 i=0; i<ans->size; i++ )
			ans->data[i] = 0;
	}

	if( used <= 2  &&  divider.used <= 2 )
	{
		uint64 tmp_nr, tmp_div, res;

		tmp_nr = uint64( data[0] );
		if( used == 2 )
			tmp_nr += uint64( data[1] ) << 32;

		tmp_div = uint64( divider.data[0] );
		if( divider.used == 2 )
			tmp_div += uint64( divider.data[1] ) << 32;

		res = tmp_nr / tmp_div;

		if( ans )
		{
			ans->data[0] = uint32( res & uint64(uint32_max) );
			if( res > uint64(uint32_max) )
			{
				ans->data[1] = uint32( res >> 32 );
				ans->used = 2;
			}
			else
				ans->used = 1;
				ans->negative = negative ^ divider.negative;
		}

		if( rest )
		{
			rest->enlarge_to( 2 );

			tmp_nr -= res*tmp_div;
			rest->data[0] = uint32( tmp_nr & uint64(uint32_max) );
			if( tmp_nr > uint64(uint32_max) )
			{
				rest->data[1] = uint32( tmp_nr >> 32 );
				rest->used = 2;
			}
			else
				rest->used = ( rest->data[0] != 0 );
			rest->negative = negative;
		}
	}
	else
	{
		Integer tnr( *this ), tdiv( divider );
		uint32 appr_mult, tdiv_shift;
		uint64 div;

		if( ans )
		{
			ans->enlarge_to( tnr.used - tdiv.used +1 );
			ans->used = tnr.used - tdiv.used +1;
			for( uint32 i=0; i<ans->used; i++ )
				ans->data[i] = 0;
		}

		div = uint64(tdiv.data[tdiv.used-1]);
		div++;

		tdiv_shift = tnr.used - tdiv.used;
		while( tnr.compare_abs_value( tdiv ) >= 0 )
		{
			while( tnr.compare_abs_value_shifted( tdiv, tdiv_shift ) < 0 )
				tdiv_shift--;

			if( tnr.used > (tdiv.used + tdiv_shift) )
				appr_mult = uint32( ( (uint64(tnr.data[tnr.used-1])<<32) + uint64(tnr.data[tnr.used-2]) ) / div );
			else
				appr_mult = uint32( uint64(tnr.data[tnr.used-1]) / div );

			if( appr_mult == 0 )
				appr_mult = 1;

			// tnr -= tdiv * 2^(32*tdiv_shift) * appr_mult;
			tnr.sub_uint32_mult_shifted( appr_mult, tdiv, tdiv_shift );

			if( ans ) // *ans += appr_mult * 2^(32*tdiv_shift)
				ans->add_uint32_shifted( appr_mult, tdiv_shift );
		}

		if( ans )
		{
			while( ans->used != 0 )
			{
				if( ans->data[ans->used-1] == 0 )
					ans->used--;
				else
					break;
			}
		}

		if( ans )
			ans->negative = negative ^ divider.negative;

		if( rest )
		{
			rest->swap( tnr );
			rest->negative = negative;
		}
	}
}

void Integer::add(const Integer &val) throw(error_obj)
{
	if( negative == val.negative )
		add_data( val );
	else
	{
		if( val.negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				sub_data( val );
				negative = false;
			}
			else
			{
				sub_data_reverse( val );
				negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				sub_data( val );
				negative = true;
			}
			else
			{
				sub_data_reverse( val );
				negative = false;
			}
		}
	}
}

void Integer::sub(const Integer &val) throw(error_obj)
{
	if( negative != val.negative )
		add_data( val );
	else
	{
		if( !negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				sub_data( val );
				negative = false;
			}
			else
			{
				sub_data_reverse( val );
				negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				sub_data( val );
				negative = true;
			}
			else
			{
				sub_data_reverse( val );
				negative = false;
			}
		}
	}
}

uint32 Integer::count_used_bits(void) const
{
	uint32 tmp, i;

	if( used == 0 )
		return 0;

	tmp = data[used-1];
	for( i=0; tmp; i++ )
		tmp >>= 1;

	return (used-1)*32 + i;
}

void Integer::set_bit( uint32 bit ) throw(error_obj)
{
	enlarge_to( bit/32 +1 );

	if( (bit/32 +1) > used )
	{
		for( uint32 i=used; i<(bit/32 +1); i++ )
			data[i] = 0;

		used = bit/32 +1;
	}

	data[bit/32] |= uint32(1) << (bit%32);
}

void Integer::clear_bit( uint32 bit ) throw(error_obj)
{
	if( (bit/32 +1) > used )
		return;

	data[bit/32] &= ~(uint32(1) << (bit%32));
	if( used == (bit/32 +1) )
	{
		for( ; used != 0  &&  data[used-1] == 0; used-- )
			;
	}
}

bool Integer::check_bit( uint32 bit ) const throw(error_obj)
{
	if( (bit/32 +1) > used )
		return false;

	if( data[bit/32] & (uint32(1) << (bit%32)) )
		return true;
	return false;
}

Integer iSqrt( const Integer &var ) throw(error_obj)
{
	Integer x, tmp;

	if( var.negative )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Number must be positive in iSqrt.") );

	if( var.used == 0 )
	{
		x.used = 0;
		return x;
	}

	x.enlarge_to( (var.used+1)/2  ); // makes room for the start guess
	{
		uint32 ans_bits = (var.count_used_bits()+1)/2;
		x.set_bit( ans_bits-1 ); // make the start guess
		if( ans_bits >= 3 )
		{
			x.set_bit( ans_bits-3 );
			if( !(var.count_used_bits() & 1) )
				x.set_bit( ans_bits-2 );
		}
	}

	do
	{
// nexton-raphson's method for solving sqrt p = x
		var.divide( x, &tmp, 0 ); // X[n+1] = ( X[n] + p/X[n] ) / 2
		x.add( tmp );
		x.div_by_2();

		x.mul( x, &tmp );  // this code checks to see if the loop is finished
		tmp.add( x );      // n^2 + 2n = (n+1)^2 -1
		tmp.add( x ); // tmp2 = x^2 + 2x
	}while( tmp.compare_abs_value( var ) < 0 );

	return x;
}

void Integer::power( Integer nr, Integer *dest ) const throw(error_obj)
{
	Integer tmp;
	uint32 power;

	if( dest == 0 )
		return;

	if( used == 0 )
	{
		if( nr.used == 0 )
		{
			*dest = 1;
			return;
		}
		else
		{
			dest->used = 0;
			return;
		}
	}

	if( nr.used > 1 )
	{
		if( used == 1  &&  data[0] == 1 )
		{
			dest->enlarge_to( 1 );
			dest->used = 1;
			dest->negative = false;
			dest->data[0] = 1;
			return;
		}
		else
			THROW_ERROR( ErrorType_Overflow, _("Overflow") );
	}

	if( nr.used == 0 )
	{
		dest->enlarge_to( 1 );
		dest->used = 1;
		dest->negative = false;
		dest->data[0] = 1;
		return;
	}

	power = nr.data[0];
	nr = *this;

	if( power & 1 )
		*dest = *this;
	else
		*dest = 1;

	while( power >>= 1 )
	{
		nr.mul( nr, &tmp );
		nr = tmp;

		if( power & 1 )
		{
			dest->mul( nr, &tmp );
			*dest = tmp;
		}
	}
}

void Integer::mul_data( const Integer &factor, Integer *dest ) const throw(error_obj)
{
	uint32 i, i2, *ptr, *ptr_dest;
	const uint32 *ptr2;
	uint64 overflow=0, tmp;

	if( dest == 0 )
		return;

	dest->enlarge_to( used + factor.used );
	dest->used = dest->size;

	for( i=0; i<dest->used; i++ )
		dest->data[i] = 0;

	ptr = data;
	for( i=0; i<used; i++, ptr++ )
	{
		tmp = *ptr;
		ptr2 = factor.data;
		ptr_dest = &(dest->data[i]);
		for( i2=0; i2<factor.used; i2++ )
		{
			overflow += uint64(*ptr_dest) + tmp * uint64(*ptr2++);
			*ptr_dest++ = uint32( overflow );
			overflow >>= 32;
		}
		do{
			overflow += uint64(*ptr_dest);
			*ptr_dest++ = uint32( overflow );
			overflow >>= 32;
		}while( overflow );
	}

	for( i=dest->used-1; i != uint32_max; i-- )
	{
		if( dest->data[i] != 0 )
			break;
	}
	dest->used = i+1;
}

void Integer::mul( const Integer &factor, Integer *dest ) const throw(error_obj)
{
	if( dest == 0 )
		return;

	if( used < 100  ||  factor.used < 100 ) // this limit is probobly not the best for all cpus but it's
		mul_data( factor, dest );       // likely to be good enough
	else
	{  // here is a Recursive Divide & Conquer multiply implementation
		Integer ah, al, bh, bl, X;
		uint32 base;

		base = ( (used>=factor.used) ? used : factor.used ) /2;

		dest->enlarge_to( used + factor.used );

		extract( &ah, base, used-1 );
		extract( &al, 0, base-1 );
		factor.extract( &bh, base, factor.used-1 );
		factor.extract( &bl, 0, base-1 );

		al.mul( bl, dest );        // dest = al*bl
		ah.mul( bh, &X );          // X = ah*bh
		bh.add_data( bl );         // bh += bl
		ah.add_data( al );         // ah += al
		ah.mul( bh, &al );         // al = ah*bh = (ah+al)*(bh+bl)
		al.sub_data( X );          // al -= X
		al.sub_data( *dest );      // al -= dest

		dest->add_data_uint32_shifted( al, base );
		dest->add_data_uint32_shifted( X, base*2 );
	}
	dest->negative = negative ^ factor.negative;
}

void Integer::faculty(void) throw(error_obj)
{
	if( used >= 2 )
		THROW_ERROR( ErrorType_Overflow, _("Overflow") );

	if( negative )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate faculty of a negative number.") );

	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		return;
	}

	Integer tmp, tmp2, tmp3(1);
	uint32 twos = 0, this_many, take_many = 1, taken_this = 0;
	static const uint32 limit[] = { 0,      92681,    2048,    430,   128,   64,  35,  29, 1 };

	tmp = *this;
	while( *tmp.data <= limit[take_many] )
		take_many++;
	
// this loop calculates the faculty without all 2 factors, they are fixed after the loop
// it also takes upto to five factors at a time (as long as their product is less than 2^32-1
// because that multiplication is done in O(1) instead of O(n) as is the case when
// multipling it with the total product
	for( (*tmp.data)--; *tmp.data > uint32(1); (*tmp.data)-- )
	{
		this_many = 0;
		while( !(*tmp.data & 1) )
		{
			this_many++;
			*tmp.data >>= 1;
		}
		*tmp3.data *= *tmp.data;
		taken_this++;
		twos += this_many;
		*tmp.data <<= this_many;
		if( taken_this == take_many )
		{
			this->mul( tmp3, &tmp2 );
			*this = tmp2;

			taken_this = 0;
			*tmp3.data = 1;
			if( *tmp.data <= limit[take_many] )
				take_many++;
		}
	}
	if( *tmp3.data != 1 )
	{
		this->mul( tmp3, &tmp2 );
		*this = tmp2;
	}
	this->mul_by_2_pow_32( twos/32 );
	twos %= 32;
	while( twos-- != 0 )
		this->mul_by_2();
}

void Integer::perm( Integer n, Integer k ) throw(error_obj)
{
	if( n.negative  ||  k.negative )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate perm(n,k) if n or k is negative.") );

	if( n.compare_abs_value( k ) < 0 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate perm(n,k) when k > n.") );

	Integer tmp;
	tmp = n;
	tmp.sub( k );
	k = tmp;

	*this = 1;

	for( ; n>k; n-- )
	{
		this->mul( n, &tmp );
		*this = tmp;
	}
}

void Integer::comb( const Integer &n, Integer k ) throw(error_obj)
{
	if( n.negative  ||  k.negative )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate comb(n,k) if n or k is negative.") );

	if( n.compare_abs_value( k ) < 0 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate comb(n,k) when k > n.") );

	Integer tmp;

	tmp = n;
	tmp.sub( k );
	if( tmp < k )
		k = tmp;

	tmp.perm( n, k );
	k.faculty();

	tmp.divide( k, this, 0 );
}

void Integer::extract( Integer *dest, uint32 start, uint32 stop ) const throw(error_obj)
{
	if( dest == 0 )
		return;

	if( used == 0 )
	{
		*dest = 0;
		return;
	}
	if( stop >= used )
		stop = used -1;

	if( start > stop )
	{
		*dest = 0;
		return;
	}

	dest->enlarge_to( stop-start+1 );
	dest->used = stop-start+1;

	for( uint32 i=0; start <= stop; i++, start++ )
		dest->data[i] = data[start];

	if( dest->data[dest->used-1] == 0 )
	{
		for( ; dest->used != 0  &&  dest->data[dest->used-1] == 0 ; dest->used-- )
			;
	}
}

// Calculates dest = [ (n^p) mod m ]
void powMod( Integer *dest, Integer n, Integer p, const Integer &m ) throw(error_obj)
{
	Integer tmp;

	if( dest == 0 )
		return;

	if( p.isNegative() )
		THROW_ERROR( ErrorType_Domain, _("Domain error: result not an integer when raising to negative numbers!") );

	if( m.isNegative()  ||  m.isZero() )
		THROW_ERROR( ErrorType_Domain, _("Domain error: powMod: m must be a positive integer!") );

	if( p.isZero() )
	{
		Integer(1).divide( m, 0, dest );
		return;
	}

	if( n.isZero() )
	{
		dest = 0;
		return;
	}

	if( n.get_used() == 1  &&  n.get_data()[0] == 1 )
	{
		Integer(1).divide( m, 0, dest );
		return;
	}

	n.divide( m, 0, dest );
	n = *dest;
	if( !(p.get_data()[0] & 1) )
		*dest = 1;

	p.div_by_2();
	while( !p.isZero() )
	{
		n.mul( n, &tmp );
		tmp.divide( m, 0, &n );

		if( p.get_data()[0] & 1 )
		{
			dest->mul( n, &tmp );
			tmp.divide( m, 0, dest );
		}
		p.div_by_2();
	}
}
