
/*
*  keyfile - an easy way to read and write keyfiles
*  Copyright (c) 2004-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
*
*  See keyfile.h
*/

/*
News
----

v1  2006-07-27
--

	Extracted this file from vartypes.h
	Renamed keyfile_io -> keyfile
	Replaced all throw_... calls with THROW_ERROR
	Replaced all std::string usage with utf8_string

*/


#include <stdio.h>
#include <string.h>
#include "utf8_string.h"
#include "vartypes.h"
#include "keyfile.h"

keyfile translation;



keyfile::keysDB::keysDB(keysDB &src)
{
	*this = src;
}
void keyfile::keysDB::operator=(keysDB &src)
{
	name = src.name;
	value = src.value;
}


keyfile::keyfile()
{
	keys = 0;
	nrofkeys = 0;
	replace_escape_sequences = false;
}
keyfile::keyfile(const char *filename) throw(error_obj)
{
	keys = 0;
	nrofkeys = 0;
	replace_escape_sequences = false;

	read( filename );
}
keyfile::keyfile(const keyfile &src) throw(error_obj)
{
	keys = 0;
	nrofkeys = 0;
	replace_escape_sequences = false;

	*this = src;
}

keyfile::~keyfile()
{
	reset();
}

void keyfile::operator=(const keyfile &src)
{
	reset();
	for( uint32 i=0; i<src.nrofkeys; i++ )
		set_key( src.keys[i].name, src.keys[i].value );
}

void keyfile::reset(void)
{
	delete [] keys;
	keys = 0;
	nrofkeys = 0;
}

void keyfile::read_intern(const char *intern_file) throw(error_obj)
{
	FILE *fp;

	fp = tmpfile();
	if(fp == 0)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/write to file (%s)"), "<tmp_file>" );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}

	if(fwrite( intern_file, 1, strlen(intern_file), fp ) != strlen(intern_file))
	{
		fclose(fp);
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/write to file (%s)"), "<tmp_file>" );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}

	rewind( fp );

	try
	{
		read( fp, "<tmp_file>" );
	}
	catch(...)
	{
		fclose( fp );
		throw;
	}
	if(fclose(fp) != 0)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), "<tmp_file>" );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}
}

void keyfile::read(const char *filename) throw(error_obj)
{
	FILE *fp;

	fp = fopen(filename,"r");
	if(fp == 0)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}

	try
	{
		read( fp, filename );
	}
	catch(...)
	{
		fclose( fp );
		throw;
	}
	if(fclose(fp) != 0)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), filename );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}
}

void keyfile::read(FILE *open_file, const char *filename) throw(error_obj)
{
	char str[5010];
	int i,keyname,value,equal,i2;

	for(;;)
	{
		if(fgets(str,5005,open_file) == 0)
		{
			if(feof(open_file)) // this is not an error, the hole file is read
				break;

			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
			THROW_ERROR( ErrorType_File_IO, tmp_err );
		}
		str[5007] = ' ';
		str[5008] = '\"';
		str[5009] = '\0'; // this is to prevent that the loops overruns the buffer limit

		for(i=0; str[i] == ' ' || str[i] == '\t' ;i++); // skipping all spaces and tabs at the beginning of the row

		if(str[i] == '#') // this line is a comment
			continue;

		if( str[i] == '\n'  ||  str[i] == '\r'  ||  str[i] == '\0' ) // empty line
			continue;

		keyname = i;

		if( str[i] == '\"' )
		{
			keyname++;
			for(i++;;i++)
			{
				if( str[i] == '\"' )
				{
					// let's check if this " is escaped out
					for( i2=0; str[i-i2-1] == '\\'; i2++ ) ;
					
					if( i2 > 0  &&  (i2%2) == 1 ) // check if the \ is escaped out
						continue;

					str[i] = 0;
					break;
				}
			}
		}
		else
		{
			for(;;i++)
			{
				if( !( ((str[i] >= 'a') & (str[i] <= 'z')) | ((str[i] >= 'A') & (str[i] <= 'Z')) | (str[i] == '_') |
				       ((str[i] >= '0') & (str[i] <= '9')) )  )
				{
					if(str[i] != ' ' || str[i] == '\t')
					{
						char tmp_err[ERROR_OBJ_MSG_LEN];
						snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Invalid format in file %s"), filename );
						THROW_ERROR( ErrorType_File_IO, tmp_err );
					}
					str[i] = 0;
					break;
				}
			}
		}

		for( equal=0,i++; ; i++ ) // getting start of value
		{
			if( str[i] == ' ' || str[i] == '\t' )
				{  }
			else if( str[i] == '=' )
			{
				if( equal == 1 ) // more than one '=' sign
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Invalid format in file %s"), filename );
					THROW_ERROR( ErrorType_File_IO, tmp_err );
				}
				equal = 1;
			}
			else if(str[i] == '\"')
			{
				if(equal == 0)
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Invalid format in file %s"), filename );
					THROW_ERROR( ErrorType_File_IO, tmp_err );
				}
				i++;
				break;
			}
			else
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Invalid format in file %s"), filename );
				THROW_ERROR( ErrorType_File_IO, tmp_err );
			}
		}
		value = i;

		for(;;i++) // getting the end of value
		{
			if(str[i] == '\"')
			{
				// let's check if this " is escaped out
				for( i2=0; str[i-i2-1] == '\\'; i2++ ) ;
				
				if( i2 > 0  &&  (i2%2) == 1 ) // check if the \ is escaped out
					continue;

				break;
			}
			else if(str[i] == 0)
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Invalid format in file %s"), filename );
				THROW_ERROR( ErrorType_File_IO, tmp_err );
			}
			
		}
		str[i] = 0;

		set_key( &str[keyname], &str[value] );
	}
}

void keyfile_replace_escape_sequences( utf8_string &str ) throw(error_obj)
{
	for( uint32 ind=0; ind < str.get_length(); ind++ )
	{
		if( str.test_character( ind, "\\" ) )
		{
			if( str.test_character( ind+1, "\\" ) )
				str.replace( ind, 2, "\\" );
			else if( str.test_character( ind+1, "n" ) )
				str.replace( ind, 2, "\n" );
			else if( str.test_character( ind+1, "r" ) )
				str.replace( ind, 2, "\r" );
			else if( str.test_character( ind+1, "t" ) )
				str.replace( ind, 2, "\t" );
			else if( str.test_character( ind+1, "\"" ) )
				str.replace( ind, 2, "\"" );
			else if( str.test_character( ind+1, "\'" ) )
				str.replace( ind, 2, "\'" );
			else
				THROW_ERROR( ErrorType_General, _("Unknown escape sequence") );
		}
	}
}

void keyfile::set_key(const utf8_string &keyname,const utf8_string &value) throw(error_obj)
{
	utf8_string tmp_keyname( keyname );
	keysDB *newkeys = 0;
	
	if( replace_escape_sequences )
		keyfile_replace_escape_sequences( tmp_keyname );
	

	for( uint32 i=0; i<nrofkeys; i++ )
	{
		if( keys[i].name == tmp_keyname ) // key already exists, replace it
		{
			try
			{
				keys[i].value = value;

				if( replace_escape_sequences )
					keyfile_replace_escape_sequences( keys[i].value );

				return;
			}
			catch(error_obj) { throw; }
			catch(...) { THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		}
	}

	try
	{
		newkeys = new keysDB [ nrofkeys + 1 ];

		newkeys[nrofkeys].name = tmp_keyname;
		newkeys[nrofkeys].value = value;
	}
	catch(...)
	{
		if(newkeys != 0)
			delete [] newkeys;

		THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );
	}

	if( replace_escape_sequences )
		keyfile_replace_escape_sequences( newkeys[nrofkeys].value );

	for( uint32 i=0; i<nrofkeys; i++ )
		newkeys[i] = keys[i];

	delete [] keys;
	keys = newkeys;
	nrofkeys++;
}

void keyfile::remove_key( const utf8_string &keyname ) throw(error_obj)
{
	keysDB *newkeys = 0;

	for( uint32 i=0; i<nrofkeys; i++ )
	{
		if( keys[i].name == keyname ) // key found
		{
			if( nrofkeys == 1 )
			{
				reset();
				return;
			}
			else
			{
				try
				{
					newkeys = new keysDB [ nrofkeys - 1 ];
				}
				catch(error_obj) { throw; }
				catch(...) { THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }

				uint32 step = 0;
				for( uint32 o=0; o<nrofkeys; o++ )
				{
					if(o == i)
						step = 1;
					else
						newkeys[o - step] = keys[o];
				}
				delete [] keys;
				keys = newkeys;
				nrofkeys--;
			}
		}
	}
}

const utf8_string * keyfile::get_key( const utf8_string &keyname, const utf8_string *fail_pointer )
{
	for( uint32 i=0; i<nrofkeys; i++ )
	{
		if(keys[i].name == keyname)
			return &keys[i].value;
	}
	return fail_pointer;
}
const char * keyfile::get_key_c_str( const utf8_string &keyname, const char *fail_pointer )
{
	for( uint32 i=0; i<nrofkeys; i++ )
	{
		if( keys[i].name == keyname )
			return keys[i].value.c_str();
	}
	return fail_pointer;
}

const char* keyfile::get_text( const char *keyname )
{
	for( uint32 i=0; i<nrofkeys; i++ )
	{
		if( keys[i].name == keyname )
			return keys[i].value.c_str();
	}
	return keyname;
}

void keyfile_expand_to_escape_sequences( const utf8_string &src, utf8_string &dest )
       throw(error_obj)
{
	dest = src;
	
	try
	{
		for( uint32 i=0; i<dest.get_length(); i++ )
		{
			if( dest.test_character( i, "\\" ) )
			{
				dest.replace( i, 1, "\\\\" );
				i++;
			}
			else if( dest.test_character( i, "\n" ) )
			{
				dest.replace( i, 1, "\\n" );
				i++;
			}
			else if( dest.test_character( i, "\r" ) )
			{
				dest.replace( i, 1, "\\r" );
				i++;
			}
			else if( dest.test_character( i, "\t" ) )
			{
				dest.replace( i, 1, "\\t" );
				i++;
			}
			else if( dest.test_character( i, "\"" ) )
			{
				dest.replace( i, 1, "\\\"" );
				i++;
			}
		}
	}
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
}

void keyfile::write(const char *filename) throw(error_obj)
{
	FILE *fp;

	fp = fopen( filename, "wb" );

	if( fp == NULL )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/write to file (%s)"), filename );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}

	if( replace_escape_sequences )
	{
		utf8_string name, value;
		
		for( uint32 i=0; i<nrofkeys ;i++ )
		{
			keyfile_expand_to_escape_sequences( keys[i].name, name );
			keyfile_expand_to_escape_sequences( keys[i].value, value );
			fprintf( fp, "%s = \"%s\"\n", name.c_str(), value.c_str() );
		}
	}
	else
	{
		for( uint32 i=0; i<nrofkeys ;i++ )
			fprintf( fp, "%s = \"%s\"\n", keys[i].name.c_str(), keys[i].value.c_str() );
	}

	if( fclose( fp ) )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), filename );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
	}
}
