
/*
 *  keyfile - an easy way to read and write keyfiles
 *  Copyright (c) 2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef KEYFILE_H_
#define KEYFILE_H_

#define KEYFILE_H_VERSION "v1"
#define KEYFILE_H_DATE    "2006-07"

#include <stdio.h>
#include <string.h>
#include "vartypes.h"
#include "utf8_string.h"


// Note that the maximum fileline-width is 5000 bytes.

// comment lines must start with a '#', no blank steps before are allowed

// signs allowed in keyname's are a-z A-Z 0-9 _

// eg.  KEYNAME = "key value inside"
class keyfile
{
private:
	class keysDB
	{
	public:
		utf8_string name, value;

		keysDB() {}
		keysDB(keysDB &src);

		void operator=(keysDB &src);
	}*keys;
	uint32 nrofkeys;

	void read(FILE *open_file, const char *filename) throw(error_obj);

public:
	// false is default
	bool replace_escape_sequences; // known escape sequences are  \\ \n \t \r \"

	keyfile();
	keyfile(const char *filename) throw(error_obj);
	keyfile(const keyfile &src) throw(error_obj);
	~keyfile();

	// empties the object
	void reset(void);

	// sets a key to a special value, if the key is found it will be replaced
	void set_key(const utf8_string &keyname,const utf8_string &value) throw(error_obj);

	// if keyname wasn't found keyname is returned
	const char* get_text( const char *keyname );

	// if keyname wasn't found the fail_pointer is returned
	const utf8_string* get_key(const utf8_string &keyname, const utf8_string *fail_pointer=0);
	const char* get_key_c_str(const utf8_string &keyname, const char *fail_pointer=0);

	// this will remove the keyname, if it's not found nothing happens
	void remove_key(const utf8_string &keyname) throw(error_obj);

	// this doesn't call reset(), the keys in the file will be added to the ones already present
	void read(const char *filename) throw(error_obj);

	// note that intern_file isn't a filename but a file contant (NULL-terminated)
	void read_intern(const char *intern_file) throw(error_obj);

	void write(const char *filename) throw(error_obj);

	void operator=(const keyfile &src);
};



// load your translation into this object
extern keyfile translation;

// a gettext look-a-like macro
#define _(String) translation.get_text(String)


#define THROW_ERROR( error_type, string ) { \
                                     error_obj error; \
                                     error.type = error_type; \
                                     strncpy( error.msg, string, ERROR_OBJ_MSG_LEN ); \
                                     throw error; \
                                   }


#endif // KEYFILE_H_
