
/*
 *  Math2
 *  Copyright (c) 2003-2007 by Mattias Hultgren <tilda_o_tize@hotmail.com>
 *
 *  See math2.h
 */

#include "math2.h"
#include "math2.intern.h"
#include "keyfile.h"
#include <stdlib.h>
#include <sys/time.h>



namespace math
{

#ifdef DEVELOP
	#define both_left_and_right   if( left == 0  ||  right == 0 ) THROW_ERROR( ErrorType_General, "Internal error: builtin function wasn't called the right way" );
	#define only_left             if( left == 0  ||  right != 0 ) THROW_ERROR( ErrorType_General, "Internal error: builtin function wasn't called the right way" );
	#define only_right            if( left != 0  ||  right == 0 ) THROW_ERROR( ErrorType_General, "Internal error: builtin function wasn't called the right way" );
	#define neither_left_or_right if( left != 0  ||  right != 0 ) THROW_ERROR( ErrorType_General, "Internal error: builtin function wasn't called the right way" );
#else
	#define both_left_and_right   ;
	#define only_left             ;
	#define only_right            ;
	#define neither_left_or_right ;
#endif


void builtin_set(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if(left != right)
		*left = *right;

	if( res != 0 )
		*res = *left;
}

void builtin_set_add(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_plus( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_set_sub(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_minus( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_set_mul(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_mul( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_set_div(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_div( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_set_intdiv(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_intdiv( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_set_pow(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	builtin_pow( left, left, right, private_varlist );

	if( res != 0 )
		*res = *left;
}
void builtin_inc(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "++" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	left->get_complex_rw()->real++;
	left->signal_changed();


	if( res != 0 )
		*res = *left;
}
void builtin_decrement(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "--" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	left->get_complex_rw()->real--;
	left->signal_changed();

	if( res != 0 )
		*res = *left;
}

void builtin_and(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() != VariableType_Boolean  ||
	    right->get_type() != VariableType_Boolean )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "and" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	res->set_boolean( left->get_boolean() && right->get_boolean() );
}

void builtin_or(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() != VariableType_Boolean  ||
	    right->get_type() != VariableType_Boolean )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "or" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	res->set_boolean( left->get_boolean() || right->get_boolean() );
}

void builtin_xor(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() != VariableType_Boolean  ||
	    right->get_type() != VariableType_Boolean )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "xor" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	res->set_boolean( left->get_boolean() ^ right->get_boolean() );
}

void builtin_not(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )

	if(right->get_type() != VariableType_Boolean )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "not" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	res->set_boolean( !right->get_boolean() );
}

void builtin_equal(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res != 0 )
		res->set_boolean( *left == *right );
}

void builtin_notequal(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res != 0 )
		res->set_boolean( *left != *right );
}

void builtin_greater(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		if( res != 0 )
			res->set_boolean(*left->get_complex() > *right->get_complex());
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), ">" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_smaller(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		if( res != 0 )
			res->set_boolean(*left->get_complex() < *right->get_complex());
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "<" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_greater_eq(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		if( res != 0 )
			res->set_boolean(*left->get_complex() >= *right->get_complex());
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), ">=" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_smaller_eq(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		if( res != 0 )
			res->set_boolean(*left->get_complex() <= *right->get_complex());
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "<=" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_almost_eq(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	bool ret_bool;

	both_left_and_right;

	if( left->get_type() != right->get_type() )
		ret_bool = false;

	else if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
		ret_bool = complex_almost_equal(*(left->get_complex()),*(right->get_complex()));

	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Matrix )
	{
		const Matrix *matr1,*matr2;
		uint32 r,c;

		matr1 = left->get_matrix();
		matr2 = right->get_matrix();

		if((matr1->get_width() != matr2->get_width()) | (matr1->get_height() != matr2->get_height()))
			ret_bool = false;
		else
		{
			ret_bool = true;
			for(r=1;r<=matr1->get_height();r++)
			{
				for(c=1;c<=matr1->get_width();c++)
				{
					if( complex_almost_equal(matr1->get_complex(r,c),matr2->get_complex(r,c)) == false )
					{
						ret_bool = false;
						break;
					}
				}
				if(c <= matr1->get_height())
					break;
			}
		}
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "~=" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_boolean( ret_bool );
}

void builtin_plus(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
		res->set_complex( *(left->get_complex()) + *(right->get_complex()) );
	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Matrix )
	{
		Matrix matr_res;
		const Matrix *matr1,*matr2;

		matr1 = left->get_matrix();
		matr2 = right->get_matrix();
		matrix_add(&matr_res,matr1,matr2);

		res->set_matrix( &matr_res );
	}
	else if( left->get_type() == VariableType_String )
	{
		if( right->get_type() == VariableType_String )
		{
			utf8_string str;

			str = *(left->get_string());
			str.append( *(right->get_string()) );

			res->set_string( str );
		}
		else
		{
			utf8_string str;
			Format int_format;

			if(intern_format != 0)
				int_format = *intern_format;

			str = *(left->get_string());
			right->append_to_string( str, int_format, false );

			res->set_string( str );
		}
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "+" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_neg(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Matrix matr_res;

	only_right;

	if( res == 0 )
		return;

	if( right->get_type() == VariableType_Complex )
		res->set_complex( - *(right->get_complex()) );
	else if( right->get_type() == VariableType_Matrix )
	{
		matr_res = *right->get_matrix();

		for(uint32 y=1; y <= matr_res.get_height() ; y++)
		{
			for(uint32 x=1; x <= matr_res.get_width() ; x++)
				matr_res.set_complex(x,y,-matr_res.get_complex(x,y));
		}

		res->set_matrix( &matr_res );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "- (neg)" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_minus(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Matrix matr_res;
	const Matrix *matr1,*matr2;

	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
		res->set_complex( *(left->get_complex()) - *(right->get_complex()) );
	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Matrix )
	{
		matr1 = left->get_matrix();
		matr2 = right->get_matrix();
		matrix_sub(&matr_res,matr1,matr2);

		res->set_matrix( &matr_res );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "-" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_mul(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Matrix matr_res;
	const Matrix *matr1,*matr2;

	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
		res->set_complex( *(left->get_complex()) * *(right->get_complex()) );
	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Complex )
	{
		matr_res = *left->get_matrix();
		matr_res.mul( *(right->get_complex()) );

		res->set_matrix( &matr_res );
	}
	else if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Matrix )
	{
		matr_res = *right->get_matrix();
		matr_res.mul( *(left->get_complex()) );

		res->set_matrix( &matr_res );
	}
	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Matrix )
	{
		matr1 = left->get_matrix();
		matr2 = right->get_matrix();
		matrix_mul( &matr_res, matr1, matr2 );

		res->set_matrix( &matr_res );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "*" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_div(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
		res->set_complex( *(left->get_complex()) / *(right->get_complex()) );
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "/" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_intdiv(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		const Complex *a, *b;
		Integer ai, bi;

		a = left->get_complex();
		b = right->get_complex();
		if( floatx(a->imaginary) != 0.0  ||  floatx(b->imaginary) != 0.0  ||
		    !(a->real.isInteger())  ||  !(a->real.isInteger()) )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "\\" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		a->real.get_top( &ai );
		b->real.get_top( &bi );

		res->set_complex( Real( ai / bi ) );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "\\" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_modulo(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		const Complex *a, *b;
		Integer ai, bi;

		a = left->get_complex();
		b = right->get_complex();
		if( floatx(a->imaginary) != 0.0  ||  floatx(b->imaginary) != 0.0  ||
		    !(a->real.isInteger())  ||  !(b->real.isInteger()) )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "mod" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		a->real.get_top( &ai );
		b->real.get_top( &bi );

		res->set_complex( Real( ai % bi ) );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "mod" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_pow(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Complex tmp;

	both_left_and_right;

	if( res == 0 )
		return;

	if( left->get_type() == VariableType_Complex  &&  right->get_type() == VariableType_Complex )
	{
		if( left->has_name() )
		{
			if( *(left->get_name()) == "e"  && *(left->get_complex()) == Complex( E ) )
			{
				Complex tmp2;

				tmp = *(right->get_complex());
				tmp.real = expx(tmp.real);

				tmp2 = degfix * Complex( Real(0), tmp.imaginary );

				tmp.real *= expx(tmp2.real);
				tmp.imaginary = tmp2.imaginary;

				res->set_complex( Complex( tmp.real*Real(cosx(tmp.imaginary)),
				                  tmp.real*Real(sinx(tmp.imaginary)) ) );
			}
			else
				res->set_complex( power( *(left->get_complex()), *(right->get_complex()) ) );
		}
		else
			res->set_complex( power( *(left->get_complex()), *(right->get_complex()) ) );
	}
	else if( left->get_type() == VariableType_Matrix  &&  right->get_type() == VariableType_Complex )
	{
		Matrix matr,matr_res;
		Integer power;

		tmp = *(right->get_complex());
		if( floatx(tmp.imaginary) != 0.0)
			THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
		if( !tmp.real.isInteger() )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Can't raise a matrix to a non integer number.") );

		matr = *left->get_matrix();
		tmp.real.get_top( &power );

		matrix_pow( &matr_res, &matr, power );

		res->set_matrix( &matr_res );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "^" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_faculty(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if( res == 0 )
		return;

	if(left->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "!" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	Complex tmp;
	Integer val;

	tmp = *(left->get_complex());

	if(floatx(tmp.imaginary) != 0.0)
		THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );

	if( !tmp.real.isInteger() )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Can't calculate faculty of a non integer number.") );

	tmp.real.get_top( &val );

	val.faculty();
	tmp.real = val;
	tmp.imaginary = 0;

	res->set_complex( tmp );
}

void builtin_ipart(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( ipart( *(right->get_complex()) ) );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;

		matr = *right->get_matrix();

		for(uint32 y=1; y <= matr.get_height() ; y++)
		{
			for(uint32 x=1; x <= matr.get_width() ; x++)
				matr.set_complex(x,y,ipart(matr.get_complex(x,y)));
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "iPart" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_sin(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "sin" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( sinc( degfix * *(right->get_complex()) ) );
}

void builtin_cos(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "cos" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( cosc( degfix * *(right->get_complex()) ) );
}

void builtin_tan(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "tan" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( tanc( degfix * *(right->get_complex()) ) );
}

void builtin_asin(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "asin" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( asinc( *(right->get_complex()) ) / degfix );
}

void builtin_acos(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "acos" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( acosc( *(right->get_complex()) ) / degfix );
}

void builtin_atan(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "atan" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( atanc( *(right->get_complex()) ) / degfix );
}

void builtin_sqrt(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "sqrt" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( sqroot( *(right->get_complex()) ) );
}

void builtin_isqrt(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;
	const Complex *tmp;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "iSqrt" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	tmp = right->get_complex();
	if( floatx(tmp->imaginary) != 0.0 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
	if( !tmp->real.isInteger() ) 
		THROW_ERROR( ErrorType_Domain, _("Domain error: The number must be an integer.") );

	Integer tmp_int;

	tmp->real.get_top( &tmp_int );


	if( res != 0 )
		res->set_complex( Real( iSqrt( tmp_int ) ) );
}

void builtin_ln(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "ln" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( logc( *(right->get_complex()) ) );
}

void builtin_log(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "log" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( log10c( *(right->get_complex()) ) );
}

void builtin_log2(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "log2" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	if( res != 0 )
		res->set_complex( log2c( *(right->get_complex()) ) );
}

void builtin_abs(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( abs( *(right->get_complex()) ) );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;

		matr = *right->get_matrix();

		for(uint32 y=1; y <= matr.get_height() ; y++)
		{
			for(uint32 x=1; x <= matr.get_width() ; x++)
				matr.set_complex(x,y,abs(matr.get_complex(x,y)));
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "abs" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_re(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( Complex( right->get_complex()->real ) );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;
		uint32 x,y;

		matr = *right->get_matrix();

		for(y=1; y <= matr.get_height() ; y++)
		{
			for(x=1; x <= matr.get_width() ; x++)
				matr.set_complex(x,y, Complex( matr.get_complex(x,y).real ) );
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "re" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_im(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( Complex( right->get_complex()->imaginary ) );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;
		uint32 x,y;

		matr = *right->get_matrix();

		for(y=1; y <= matr.get_height() ; y++)
		{
			for(x=1; x <= matr.get_width() ; x++)
				matr.set_complex(x,y, Complex( matr.get_complex(x,y).imaginary ) );
		}
		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "im" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_rand(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	neither_left_or_right;

	if( res != 0 )
		res->set_complex( Complex( floatx(rand()) / floatx( int64(RAND_MAX) + 1 ) ) );
}

void builtin_arg(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( arg( *(right->get_complex()) ) / degfix );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;
		uint32 r,c;

		matr = *right->get_matrix();

		for(r=1;r<=matr.get_height();r++)
		{
			for(c=1;c<=matr.get_width();c++)
				matr.set_complex(r,c,arg(matr.get_complex(r,c)) / degfix );
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "arg" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_time(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	timeval tmp;

	neither_left_or_right;

	gettimeofday( &tmp, 0 );

	if( res != 0 )
		res->set_complex( Complex( floatx( int64(tmp.tv_sec)*int64(1000000) + tmp.tv_usec ) / 1000000.0 ) );
}

void builtin_fpart(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
		res->set_complex( fpart(*right->get_complex()) );
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;

		matr = *right->get_matrix();

		for(uint32 y=1; y <= matr.get_height() ; y++)
		{
			for(uint32 x=1; x <= matr.get_width() ; x++)
				matr.set_complex(x,y,fpart(matr.get_complex(x,y)));
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "fPart" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_sinh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "sinh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( sinhc( *right->get_complex() ) );
}

void builtin_cosh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "cosh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( coshc( *right->get_complex() ) );
}

void builtin_tanh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "tanh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( tanhc( *right->get_complex() ) );
}

void builtin_asinh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "asinh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( asinhc( *right->get_complex() ) );
}

void builtin_acosh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "acosh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( acoshc( *right->get_complex() ) );
}

void builtin_atanh(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Complex)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "atanh" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( atanhc( *right->get_complex() ) );
}

void builtin_det(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if(right->get_type() != VariableType_Matrix)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "det" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( right->get_matrix()->det() );
}

void builtin_adj(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	if( res == 0 )
		return;

	if(right->get_type() != VariableType_Matrix)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "adj" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	Matrix matr;

	matr = *right->get_matrix();
	matr.adjoint();

	res->set_matrix( &matr );
}

void builtin_delete(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const utf8_string *str;

	only_right;

	if( right->has_name() )
		str = right->get_name();
	else if( right->get_type() == VariableType_String )
		str = right->get_string();
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "delete" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	switch( function_list.delete_function( *str ) )
	{
	case 1:
		res->set_string( _("Function deleted") );
		return;

	case -1:
		THROW_ERROR( ErrorType_General, _("Function is undeletable") );

	default: // no function named str
		break;
	}

// Searching for a Variable
	if( private_varlist )
	{
		try{  private_varlist->remove( private_varlist->get_id(str->c_str()) );  }
		catch(error_obj error)
		{
			if( error.type != ErrorType_Variable_not_found )
				throw error;
		}
	}

	try{  global_varlist.remove( global_varlist.get_id(str->c_str()) );  }
	catch(error_obj error)
	{
		if( error.type == ErrorType_Variable_not_found )
			snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Variable/function not found") );
		throw error;
	}

	res->set_string( _("Variable deleted") );
}

void builtin_make_undeletable(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const utf8_string *str;

	only_right;

	if(right->has_name())
		str = right->get_name();
	else if(right->get_type() == VariableType_String)
		str = right->get_string();
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "make_undeletable" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( function_list.set_undeletable( *str ) )
	{
		res->set_string( "Function is undeletable" );
		return;
	}

// Searching for a Variable
	if(private_varlist)
	{
		try{  private_varlist->make_undeletable( private_varlist->get_id(str->c_str()) );  }
		catch(error_obj error)
		{
			if( error.type != ErrorType_Variable_not_found )
				throw error;
		}
	}

	try{  global_varlist.make_undeletable( global_varlist.get_id(str->c_str()) );  }
	catch(error_obj error)
	{
		if( error.type == ErrorType_Variable_not_found )
			snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Variable/function not found") );
		throw error;
	}

	res->set_string( _("Variable is undeletable") );
}

void builtin_make_constant(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const utf8_string *str;

	only_right;

	if(right->has_name())
		str = right->get_name();
	else if(right->get_type() == VariableType_String)
		str = right->get_string();
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "make_constant" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

// Searching for a Variable
	if(private_varlist)
	{
		try{  private_varlist->get_pointer_rw( private_varlist->get_id(str->c_str()) )->set_to_constant();  }
		catch(error_obj error)
		{
			if( error.type != ErrorType_Variable_not_found )
				throw error;
		}
	}

	try{  global_varlist.get_pointer_rw( global_varlist.get_id(str->c_str()) )->set_to_constant();  }
	catch(error_obj error)
	{

		throw error;
	}

	res->set_string( _("Variable is now a constant") );
}

void builtin_make_type_locked(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	const utf8_string *str;

	only_right;

	if(right->has_name())
		str = right->get_name();
	else if(right->get_type() == VariableType_String)
		str = right->get_string();
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "make_type_locked" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

// Searching for a Variable
	if(private_varlist)
	{
		try{  private_varlist->get_pointer_rw( private_varlist->get_id(str->c_str()) )->set_can_change_type(false);  }
		catch(error_obj error)
		{
			if( error.type != ErrorType_Variable_not_found )
				throw error;
		}
	}

	try{  global_varlist.get_pointer_rw( global_varlist.get_id(str->c_str()) )->set_can_change_type(false);  }
	catch(error_obj error)
	{
		throw error;
	}

	res->set_string( _("Variable is now type locked") );
}

void builtin_sum(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func,*n_var,tmp,tmp_n_var;
	int64 start,stop;
	Matrix matr1,matr_res;
	const Matrix *matr2;
	utf8_string func_str,var_name;

	only_right;

	if( res == 0 )
		return;

	argument_check( "sum", right,
	                type_string, &func_str,
	                type_name, &var_name,
	                type_int64, &start,
	                type_int64, &stop,
	                type_end );

	func.set_code_line( func_str.c_str() );

	if(private_varlist)
	{
		try{
			n_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
		}
		catch(...) {
			n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
		}
	}
	else
		n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
	tmp_n_var = *n_var;

	if(start > stop)
		THROW_ERROR( ErrorType_Domain, _("Domain error: Start mustn't be larger than stop.") );

	n_var->set_complex( Complex( start ) );
	try{  func.calc_code_line( res );  }
	catch(...)
	{
		try{  *n_var = tmp_n_var;  }   catch(...) {  } // this is empty by a reason
		throw;
	}

	if(res->get_type() == VariableType_Matrix)
		matr_res = *(res->get_matrix());

	for(start++;start <= stop;start++)
	{
		n_var->set_complex( Complex( start ) );
		try{  func.calc_code_line( &tmp );  }
		catch(...)
		{
			try{  *n_var = tmp_n_var;  }   catch(...) {  } // this is empty by a reason
			throw;
		}

		if( res->get_type() != tmp.get_type() )
		{
			try{  *n_var = tmp_n_var;  }   catch(...) {  } // this is empty by a reason

			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "sum" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		if( res->get_type() == VariableType_Complex )
			res->set_complex( *(res->get_complex()) + *(tmp.get_complex()) );
		else if( res->get_type() == VariableType_Matrix )
		{
			matr1 = matr_res;
			matr2 = tmp.get_matrix();

			matrix_add(&matr_res,&matr1,matr2);
		}
		else
		{
			try{  *n_var = tmp_n_var;  }   catch(...) {  } // this is empty by a reason

			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "sum" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	*n_var = tmp_n_var;
	if( res->get_type() == VariableType_Matrix )
		res->set_matrix( &matr_res );
}

void builtin_nderiv(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func, *x_var, tmp_res, tmp_x_var;
	Complex point, step, tmp;
	utf8_string func_str, var_name;

	only_right;

	if( res == 0 )
		return;

	step = Complex( Real(1,1000000) );
	argument_check( "nDeriv", right,
	                type_string, &func_str,
	                type_name, &var_name,
	                type_complex, &point,
	                type_complex_default, &step,
	                type_end );

	func.set_code_line( func_str.c_str() );

	if(private_varlist)
	{
		try{
			x_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
		}
		catch(...) {
			x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
		}
	}
	else
		x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
	tmp_x_var = *x_var;

	x_var->set_complex(point + step);
	try{  func.calc_code_line( &tmp_res );  }
	catch(...)
	{
		try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason
		throw;
	}

	if( tmp_res.get_type() != VariableType_Complex )
	{
		try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason

		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nDeriv" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	tmp = *(tmp_res.get_complex());

	x_var->set_complex(point - step);
	try{  func.calc_code_line( &tmp_res );  }
	catch(...)
	{
		try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason
		throw;
	}
	if( tmp_res.get_type() != VariableType_Complex )
	{
		try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason

		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nDeriv" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	tmp = (tmp - *(tmp_res.get_complex())) / (2.0 * step);

	*x_var = tmp_x_var;

	res->set_complex( tmp );
}

void builtin_nint(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func, *x_var, partres, tmp_x_var;
	const Complex *tmp;
	Complex prev;
	floatx start, stop, step;
	int64 steps;
	utf8_string func_str, var_name;
	bool gotprev=false;
	bool negate_result = false;

	only_right;

	if( res == 0 )
		return;

	steps = 1000;
	argument_check( "nInt", right,
	                type_string, &func_str,
	                type_name, &var_name,
	                type_floatx, &start,
	                type_floatx, &stop,
	                type_int64_default, &steps,
	                type_end );

	if( steps <= 0 || steps > 1000000000 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

	if( start > stop )
	{
		negate_result = true;
		step = start;
		start = stop;
		stop = step;
	}

	res->set_complex( Complex( 0 ) );
	if(start == stop)
	{
		res->set_complex( Real( 0 ) );
		return;
	}

	func.set_code_line( func_str.c_str() );

	if(private_varlist)
	{
		try{
			x_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
		}
		catch(...) {
			x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
		}      
	}
	else
		x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
	tmp_x_var = *x_var;

	step = (stop - start) / floatx(steps);
	for(; start <= stop; start+=step)
	{
		x_var->set_complex( Complex( start ) );
		try{  func.calc_code_line( &partres );  }
		catch(...)
		{
			try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason
			throw;
		}

		if( partres.get_type() != VariableType_Complex )
		{
			try{  *x_var = tmp_x_var;  }    catch(...) {  } // this is empty by a reason

			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nInt" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		if( partres.get_type() != VariableType_Complex )
		{
			gotprev = false;
			continue;
		}

		tmp = partres.get_complex();
		if(floatx(tmp->imaginary) != 0.0)
			gotprev = false;
		else
		{
			if(gotprev)
				res->set_complex( *(res->get_complex()) + (*tmp + prev) * step / 2.0 );

			gotprev = true;
			prev = *tmp;
		}
	}

	*x_var = tmp_x_var;
	if( negate_result )
		res->set_complex( Complex(Real(0)) - *res->get_complex() );
}

void builtin_nsolve(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func, *x_var, tmp_res;
	Complex point, step, tmp, prev;
	int sign;

	only_right;

	if( res == 0 )
		return;

	{
		utf8_string func_str,var_name;

		point = Complex( 0 );
		step = Complex( Real(1,1000000) );
		argument_check( "nSolve", right,
		                type_string, &func_str,
		                type_name, &var_name,
		                type_complex_default, &point,
		                type_complex_default, &step,
		                type_end );

		func.set_code_line( func_str.c_str() );

		if(private_varlist)
		{
			try{
				x_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
			}
			catch(...) {
				x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
			}
		}
		else
			x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
	}

	for(int i=0;i<1000;i++)
	{
// Calculates nDeriv at point
		x_var->set_complex(point + step);
		func.calc_code_line( &tmp_res );

		if( tmp_res.get_type() != VariableType_Complex )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nSolve" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		tmp = *tmp_res.get_complex();

		x_var->set_complex(point - step);
		func.calc_code_line( &tmp_res );

		if( tmp_res.get_type() != VariableType_Complex )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nSolve" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		tmp = (tmp - *tmp_res.get_complex()) / (2.0 * step);

		if( floatx(tmp.real) < 0.0 )
		{
			if((sign == 1) & (i > 10))
				THROW_ERROR( ErrorType_General, _("No solution found") );
			sign = -1;
		}
		else
		{
			if((sign == -1) & (i > 10))
				THROW_ERROR( ErrorType_General, _("No solution found") );
			sign = 1;
		}
//  tmp = nDeriv

		x_var->set_complex(point);
		func.calc_code_line( &tmp_res );

		if( tmp_res.get_type() != VariableType_Complex )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "nSolve" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		tmp = point - *tmp_res.get_complex() / tmp;

		if(tmp == point) // solution found
			break;

		if(i >= 1  &&  tmp == prev)
		{ // solution is between tmp and point...
			point = (point + tmp) / floatx(2.0); // this probobly doesn't make it more accurate..but it wont hurt
			break;
		}

		prev = point;
		point = tmp;
	}
	res->set_complex( point );
}

void builtin_picture(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Picture pic;
	int64 width,height;
	floatx xmin, xmax, bottom, top;
	Matrix colormatrix;

	only_right;

	if( res == 0 )
		return;

	colorrgb_to_colormatrix( picture_h::color::WHITE, &colormatrix);
	xmin = -10.0;
	bottom = xmin;
	xmax = 10.0;
	top = xmax;

	argument_check( "picture", right,
	                type_int64, &width,
	                type_int64, &height,
	                type_floatx_default, &xmin,
	                type_floatx_default, &xmax,
	                type_floatx_default, &bottom,
	                type_floatx_default, &top,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &pic.bgcolor );

	pic.set_size( width, height, xmin, xmax, bottom, top );
	pic.clear();

	res->set_picture( &pic );
}

void builtin_shade(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func1, func2, res1, res2, *x_var;
	Picture pic;
	const Complex *tmp1, *tmp2;
	picture_h::colorrgb color;
	floatx x, xstep, xmax;
	int64 shade = 1;
	int32 x_val;

	only_right;

	if( res == 0 )
		return;

	{
		utf8_string func1_str,func2_str,var_name;
		Matrix colormatrix;

		colorrgb_to_colormatrix( picture_h::color::BLUE, &colormatrix);

		argument_check( "shade", right,
		                type_picture, &pic,
		                type_string, &func1_str,
		                type_string, &func2_str,
		                type_name, &var_name,
		                type_floatx, &x,
		                type_floatx, &xmax,
		                type_int64_default, &shade,
		                type_matrix_default, &colormatrix,
		                type_end );

		func1.set_code_line( func1_str.c_str() );
		func2.set_code_line( func2_str.c_str() );

		if(private_varlist)
		{
			try{
				x_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
			}
			catch(...) {
				x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
			}
		}
		else
			x_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );

		if(x > xmax)
		{
			xstep = x;
			x = xmax;
			xmax = xstep;
		}

		if(shade < 0)
			shade = 1;
		colormatrix_to_colorrgb( colormatrix, &color );
	}

	xstep = (pic.get_right() - pic.get_left()) / floatx(pic.pic.get_width());
	xstep *= floatx(shade+1);

	x_val = pic.get_x_pixel( x );


	for( ;x<=xmax; x+=xstep, x_val += (1+int32(shade)) )
	{
		x_var->set_complex( Complex( x ) );
		func1.calc_code_line( &res1 );
		func2.calc_code_line( &res2 );

		if( res1.get_type() != VariableType_Complex  ||  res2.get_type() != VariableType_Complex )
			continue;

		tmp1 = res1.get_complex();
		tmp2 = res2.get_complex();
		if( floatx(tmp1->imaginary) == 0.0  &&  floatx(tmp2->imaginary) == 0.0 )
			pic.pic.line( x_val, pic.get_y_pixel( floatx(tmp1->real) ),
			              x_val, pic.get_y_pixel( floatx(tmp2->real) ), color);
	}

	res->set_picture( &pic );
}

void builtin_axes(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Picture *pic;
	picture_h::colorrgb color;
	Matrix colormatrix;
	floatx scalex, scaley;
	int64 length = 3;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "axes" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	
	pic = left->get_picture_rw();

	scalex = 1.0;
	scaley = scalex;
	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix);

	argument_check( "axes", right,
	                type_floatx_default, &scalex,
	                type_floatx_default, &scaley,
	                type_int64_default, &length,
	                type_matrix_default, &colormatrix,
	                type_end );

	if( length < 0  ||  length > 100  ||  scalex < 0.0  ||  scaley < 0.0 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );
	colormatrix_to_colorrgb( colormatrix, &color );

	pic->line( 0.0, pic->get_top(), 0.0, pic->get_bottom(), false, color );

	pic->line( pic->get_left(), 0.0, pic->get_right(), 0.0, false, color );

	{
		int32 x0,y0;
		floatx act_scale,start,stop;

		x0 = pic->get_x_pixel( 0.0 );
		y0 = pic->get_y_pixel( 0.0 );

		if((scalex != 0.0) & (length > 0))
		{
			start = truncx( pic->get_left() / scalex ) * scalex - scalex;
			if( (pic->get_right()-pic->get_left())/scalex > pic->pic.get_width())
				act_scale = (pic->get_right()-pic->get_left()) / floatx(pic->pic.get_width());
			else
				act_scale = scalex;
			stop = pic->get_right();
			for(;start <= stop;start += act_scale)
			{
				if( start > (act_scale/2.0)  ||  start < (-act_scale/2.0) )
					pic->pic.line( pic->get_x_pixel(start), y0, pic->get_x_pixel(start), y0-length, color );
			}
		}
		if((scalex != 0.0) & (length > 0))
		{
			start = truncx( pic->get_bottom() / scaley ) * scaley - scaley;
			if( (pic->get_top()-pic->get_bottom())/scaley > pic->pic.get_height())
				act_scale = (pic->get_top()-pic->get_bottom()) / floatx(pic->pic.get_height());
			else
				act_scale = scaley;
			stop = pic->get_top();
			for(;start <= stop;start += act_scale)
			{
				if( start > (act_scale/2.0)  ||  start < (-act_scale/2.0) )
					pic->pic.line( x0, pic->get_y_pixel(start), x0+length, pic->get_y_pixel(start), color );
			}
		}
	}
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_grid(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Picture *pic;
	picture_h::colorrgb color;
	Matrix colormatrix;
	floatx scale1, scale2;
	utf8_string type;
	bool antialiasing = false;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "grid" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	
	pic = left->get_picture_rw();

	scale1 = 1.0;
	scale2 = 1.0;
	type = "Rect";
	colorrgb_to_colormatrix( picture_h::color::LIGHT_GRAY, &colormatrix);

	argument_check( "grid", right,
	                type_string_default, &type,
	                type_floatx_default, &scale1,
	                type_floatx_default, &scale2,
	                type_boolean_default, &antialiasing,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );


	if(type == "Rect")
	{
		floatx act_scale,start,stop;
		int32 x0,y0;

		x0 = pic->get_x_pixel( 0.0 );
		y0 = pic->get_y_pixel( 0.0 );

		if(scale1 != 0.0)
		{
			start = truncx( pic->get_left() / scale1 ) * scale1 - scale1;
			if( (pic->get_right()-pic->get_left())/scale1 > pic->pic.get_width())
				act_scale = (pic->get_right()-pic->get_left()) / floatx(pic->pic.get_width());
			else
				act_scale = scale1;
			stop = pic->get_right();
			for(;start <= stop;start += act_scale)
				pic->line( start, pic->get_top(), start, pic->get_bottom(), false, color );
		}
		if(scale2 != 0.0)
		{
			start = truncx( pic->get_bottom() / scale2 ) * scale2 - scale2;
			if( (pic->get_top()-pic->get_bottom())/scale2 > pic->pic.get_height())
				act_scale = (pic->get_top()-pic->get_bottom()) / floatx(pic->pic.get_height());
			else
				act_scale = scale2;
			stop = pic->get_top();
			for(;start <= stop;start += act_scale)
				pic->line( pic->get_left(), start, pic->get_right(), start, false, color );
		}
	}
	else if(type == "Polar")
	{
		floatx act_scale,start,stop;

		if(scale1 != 0.0)
		{
			floatx tmp;
			stop = sqrtx( pic->get_left()*pic->get_left() + pic->get_top()*pic->get_top() );
			start = stop;
			tmp = sqrtx( pic->get_right()*pic->get_right() + pic->get_top()*pic->get_top() );
			stop = (stop > tmp) ? stop : tmp;
			start = (start < tmp) ? start : tmp;
			tmp = sqrtx( pic->get_right()*pic->get_right() + pic->get_bottom()*pic->get_bottom() );
			stop = (stop > tmp) ? stop : tmp;
			start = (start < tmp) ? start : tmp;
			tmp = sqrtx( pic->get_left()*pic->get_left() + pic->get_bottom()*pic->get_bottom() );
			stop = (stop > tmp) ? stop : tmp;
			start = (start < tmp) ? start : tmp;
			if( 0.0 >= pic->get_left()  &&  0.0 <= pic->get_right()  &&  0.0 >= pic->get_bottom()  &&
			    0.0 <= pic->get_top() )
				start = 0.0;
			else
			{
				if( fabsx(pic->get_left()) < start )
					start = fabsx(pic->get_left());
				if( fabsx(pic->get_right()) < start )
					start = fabsx(pic->get_right());
				if( fabsx(pic->get_top()) < start )
					start = fabsx(pic->get_top());
				if( fabsx(pic->get_bottom()) < start )
					start = fabsx(pic->get_bottom());
			}

			if( (stop-start)/scale1 > sqrtx( floatx(pic->pic.get_height())*floatx(pic->pic.get_height())
			                            + floatx(pic->pic.get_width())*floatx(pic->pic.get_width()) ) )
				act_scale = (stop-start) / sqrtx( floatx(pic->pic.get_height())*floatx(pic->pic.get_height())
				   + floatx(pic->pic.get_width())*floatx(pic->pic.get_width()) );
			else
				act_scale = scale1;

			start = truncx(start / act_scale) * act_scale;

			for(;start <= stop; start += act_scale)
				pic->pic.ellipse( pic->get_x_pixel(0.0), pic->get_y_pixel(0.0), uint32(start / pic->get_unit_x_pixel()),
				                  uint32(start / pic->get_unit_y_pixel()), color, antialiasing );

			{
				floatx twopi = 2.0 * PI;

				if(floatx(degfix.imaginary) != 0.0)
					THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
				scale2 *= floatx(degfix.real);

				for( tmp = 0.0; tmp < twopi; tmp += scale2 )
					pic->line( 0.0, 0.0, cosx(tmp)*stop, sinx(tmp)*stop, antialiasing, color );
			}
		}
	}
	else
	{
		error_obj error;

		error.type = ErrorType_General;
		snprintf(error.msg,ERROR_OBJ_MSG_LEN, "Unknown symbol '%s'",type.c_str());
		throw error;
	}
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_perm(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Integer i, i2, tmp;

	only_right;

	if( res == 0 )
		return;

	argument_check( "perm", right,
	                type_integer, &i,
	                type_integer, &i2,
	                type_end );

	tmp.perm( i, i2 );

	res->set_complex( Real( tmp ) );
}

void builtin_comb(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Integer up, down, tmp;

	only_right;

	if( res == 0 )
		return;

	argument_check( "comb", right,
	                type_integer, &up,
	                type_integer, &down,
	                type_end );

	tmp.comb( up, down );

	res->set_complex( Real( tmp ) );
}

void builtin_get_type(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if( res != 0 )
		res->set_string( VARIABLE_TYPE_NAMES[left->get_type()] );
}

void builtin_set_point(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	floatx x,y;
	Matrix colormatrix;
	picture_h::colorrgb color;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "set_point" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "set_point", right,
	                type_floatx, &x,
	                type_floatx, &y,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );

	left->get_picture_rw()->point( x, y, color);
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_round_eval(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	switch(left->get_type())
	{
	case VariableType_String:
eval_one_only:
		if(right->get_type() != VariableType_Array)
		{
// setting the x-value
			Variable *xvar;
			xvar = global_varlist.get_pointer_rw( global_varlist.get_id("x") );
			*xvar = *right;
// x is now set to the new value
		}
		else
		{
			Variable *xyvar;
			const Array *array;

			array = right->get_array();

			if( array->get_size() == 1 )
			{
				right = array->get_variable(0);
				goto eval_one_only;
			}

			if( array->get_size() != 2 )
				THROW_ERROR( ErrorType_General, _("function(variable_values): Array's size don't match.") );


// setting the x & y-value
			xyvar = global_varlist.get_pointer_rw( global_varlist.get_id("x") );
			*xyvar = *( array->get_variable(0) );

			xyvar = global_varlist.get_pointer_rw( global_varlist.get_id("y") );
			*xyvar = *( array->get_variable(1) );
// x & y is now set to the new value
		}
		left->calc_code_line( res );
		break;

	default:
		builtin_mul( res, left, right, private_varlist );
		break;
	}
}

void builtin_hook_set(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	switch( left->get_type() )
	{
	case VariableType_Array:
		{
			int64 index;
			Variable var;
			Array *array;

			argument_check( "Array[index]=...", right, type_int64, &index,
			                type_variable, &var, type_end );

			array = left->get_array_rw();

			if( index < 0  ||  index >= array->get_size() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Array[index]=...: index out of range") );

			array->set_variable( index, var );
			left->signal_changed();

			if( res != 0 )
				*res = var;
		}
		break;

	case VariableType_Matrix:
		{
			int64 row, col;
			Complex val;
			Variable var;
			Matrix *matr;

			argument_check( "Matrix[row,col]=...", right, type_variable, &var,
			                type_complex, &val, type_end );
			argument_check( "Matrix[row,col]=...", &var, type_int64, &row,
			                type_int64, &col, type_end );

			matr = left->get_matrix_rw();
			if( row < 1  ||  row > matr->get_height() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Matrix[row,col]: row out of range") );
			if( col < 1  ||  col > matr->get_width() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Matrix[row,col]: col out of range") );

			matr->set_complex( col, row, val);
			left->signal_changed();

			if( res != 0 )
				res->set_complex( val );
		}
		break;

	case VariableType_String:
		{
			utf8_string *str, ins_str;
			int64 start, length=1;
			Variable var;

			argument_check( "String[start (,length)]=...", right,
			                type_variable, &var,
			                type_string, &ins_str,
			                type_end );
			argument_check( "String[start (,length)]=...", &var,
			                type_int64, &start,
			                type_int64_default, &length,
			                type_end );

			str = left->get_string_rw();
			if( start < 0  ||  start >= str->get_length() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: String[start (,length)]: start out of range") );
			if( length < 1 )
				THROW_ERROR( ErrorType_Domain, _("Domain error: String[start (,length)]: length out of range") );

			try
			{
				str->replace( start, length, ins_str );
				left->signal_changed();

				if( res != 0 )
					res->set_string( *str );
			}
			catch(error_obj error) {  throw;  }
			catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
		}
		break;

	default:
		THROW_ERROR( ErrorType_General, _("Illegal variable type for 'var[index_expr]=...' syntax") );
	}
}

void builtin_hook_get(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	switch( left->get_type() )
	{
	case VariableType_Array:
		{
			const Array *array;
			int64 index;

			array = left->get_array();

			argument_check( "Array[index]", right, type_int64, &index, type_end );

			if( index < 0  ||  index >= array->get_size() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Array[index]: Index out of range") );

			if( res != 0 )
				*res = *( array->get_variable(index) );
		}
		break;

	case VariableType_Matrix:
		{
			const Matrix *matr;
			int64 row, col;

			matr = left->get_matrix();

			argument_check( "Matrix[row,col]", right, type_int64, &row,
			                type_int64, &col, type_end );

			if( row < 1  ||  row > matr->get_height() )
				THROW_ERROR( ErrorType_General, _("Domain error: Matrix[row,col]: row out of range") );
			if( col < 1  ||  col > matr->get_width() )
				THROW_ERROR( ErrorType_General, _("Domain error: Matrix[row,col]: col out of range") );

			if( res != 0 )
				res->set_complex( matr->get_complex(col,row) );
		}
		break;

	case VariableType_String:
		{
			const utf8_string *str;
			int64 start, length=1;

			str = left->get_string();

			argument_check( "String[start (,length)]", right,
			                type_int64, &start,
			                type_int64_default, &length,
			                type_end );

			if( start < 0  ||  start >= str->get_length() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: String[start (,length)]: start out of range") );
			if( length < 1 )
				THROW_ERROR( ErrorType_Domain, _("Domain error: String[start (,length)]: length out of range") );

			if( res != 0 )
			{
				try
				{
					res->set_string( str->substr( start, length ) );
				}
				catch(error_obj error) {  throw;  }
				catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
			}
		}
		break;

	default:
		THROW_ERROR( ErrorType_General, _("Illegal variable type for the 'var[index_expr]' syntax.") );
	}
}

void builtin_set_pixel(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	floatx x,y;
	Matrix colormatrix;
	picture_h::colorrgb color;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "set_pixel" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "set_pixel", right,
	                type_floatx, &x,
	                type_floatx, &y,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );

	left->get_picture_rw()->pic.set_pixel( int32(x), int32(y), color);
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_line(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	floatx x1,x2,y1,y2;
	Matrix colormatrix;
	picture_h::colorrgb color;
	bool antialiasing = false;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "line" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "line", right,
	                type_floatx, &x1, type_floatx, &y1,
	                type_floatx, &x2, type_floatx, &y2,
	                type_boolean_default, &antialiasing,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );

	left->get_picture_rw()->line( x1, y1, x2, y2, antialiasing, color );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_pixel_line(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	floatx x1,x2,y1,y2;
	Matrix colormatrix;
	picture_h::colorrgb color;
	bool antialiasing = false;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "pixel_line" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "pixel_line", right,
	                type_floatx, &x1, type_floatx, &y1,
	                type_floatx, &x2, type_floatx, &y2,
	                type_boolean_default, &antialiasing,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );
	if( antialiasing )
		left->get_picture_rw()->pic.line_antialiasing( x1, y1, x2, y2, color );
	else
		left->get_picture_rw()->pic.line( int32(x1), int32(y1), int32(x2), int32(y2), color );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_circle(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Picture *pic;
	floatx x,y;
	int64 r;
	Matrix colormatrix;
	picture_h::colorrgb color;
	bool filled = false, antialiasing = false;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "circle" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "circle", right,
	                type_floatx, &x,
	                type_floatx, &y,
	                type_int64, &r,
	                type_boolean_default, &filled,
	                type_boolean_default, &antialiasing,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );

	pic = left->get_picture_rw();
	if(filled)
		pic->pic.filled_circle( pic->get_x_pixel(x), pic->get_y_pixel(y), r, color, antialiasing );
	else
		pic->pic.circle( pic->get_x_pixel(x), pic->get_y_pixel(y), r, color, antialiasing );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_pixel_circle(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	floatx x,y;
	int64 r;
	Matrix colormatrix;
	picture_h::colorrgb color;
	bool filled = false, antialiasing = false;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "pixel_circle" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	colorrgb_to_colormatrix( picture_h::color::BLACK, &colormatrix );

	argument_check( "pixel_circle", right,
	                type_floatx, &x,
	                type_floatx, &y,
	                type_int64, &r,
	                type_boolean_default, &filled,
	                type_boolean_default, &antialiasing,
	                type_matrix_default, &colormatrix,
	                type_end );

	colormatrix_to_colorrgb( colormatrix, &color );
	if(filled)
		left->get_picture_rw()->pic.filled_circle( int32(x), int32(y), r, color);
	else
		left->get_picture_rw()->pic.circle( int32(x), int32(y), r, color, antialiasing );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_table2d(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func, *n_var, tmp, tmp_n_var, *n2_var, tmp_n2_var;
	Complex replace;
	Complex start, tmp_start, stop, step, start2, stop2, step2;
	Matrix matr;
	int64 steps, steps2;
	bool abort_on_error = true;
	bool all_failed_the_same = false;
	error_obj error;
	error.type = ErrorType_None;

	only_right;

	if( res == 0 )
		return;

	{
		utf8_string func_str, var_name, var_name2;

		replace = Complex( asinx(1.0) * 6.0, expx(2.0) ); // 3*pi + i*(e^2)
		argument_check( "table2d", right,
		                type_string, &func_str,
		                type_name, &var_name,
		                type_complex, &start,
		                type_complex, &stop,
		                type_int64, &steps,
		                type_name, &var_name2,
		                type_complex, &start2,
		                type_complex, &stop2,
		                type_int64, &steps2,
		                type_complex_default, &replace,
		                type_end );

		if( replace != Complex( asinx(1.0) * 6.0, expx(2.0) ) )
			abort_on_error = false;

		if( steps <= 1  ||  steps2 <= 1 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Step must be greater than 1.") );

		if( floatx(start.imaginary) != 0.0  ||  floatx(start2.imaginary) != 0.0 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Start must be a real value.") );

		if( floatx(stop.imaginary) != 0.0  ||  floatx(stop2.imaginary) != 0.0 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Stop must be a real value.") );

		step = (stop - start) / (steps-1);
		step2 = (stop2 - start2) / (steps2-1);

		func.set_code_line( func_str.c_str() );
		if( private_varlist )
		{
			try{
				n_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
			}
			catch(...) {
				n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
			}
		}
		else
			n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
		if( private_varlist )
		{
			try{
				n2_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name2.c_str() ) );
			}
			catch(...) {
				n2_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name2.c_str() ) );
			}
		}
		else
			n2_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name2.c_str() ) );

		matr.set_size( uint32(steps), uint32(steps2) );
	}
	tmp_n_var = *n_var;
	tmp_n2_var = *n2_var;

	if( start > stop  ||  start2 > stop2 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Start mustn't be larger than stop.") );

	tmp_start = start;
	for( int64 i2=0; i2 < steps2; start2 += step2, i2++ )
	{
		start = tmp_start;
		n2_var->set_complex( start2 );
		for( int64 i=0; i < steps; start += step, i++ )
		{
			n_var->set_complex( start );
			try{  func.calc_code_line( &tmp );  all_failed_the_same = false;  }
			catch(error_obj error2)
			{
				if( error.type == ErrorType_None  &&  i == 0  &&  i2 == 0 )
				{
					error = error2;
					all_failed_the_same = true;
				}
				else if( error.type != error2.type )
					all_failed_the_same = false;

				if(abort_on_error)
					throw;
				else
					tmp.set_complex( replace );
			}

			if( tmp.get_type() != VariableType_Complex )
			{
				if(abort_on_error)
				{
					try{  *n_var = tmp_n_var;  *n2_var = tmp_n2_var;  }
					catch(...) {  } // this catch is empty not by mistake
					{
						char tmp_err[ERROR_OBJ_MSG_LEN];
						snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "table2d" );
						THROW_ERROR( ErrorType_General, tmp_err );
					}
				}
				else
					tmp.set_complex( replace );
			}

			matr.set_complex( i+1, i2+1, *tmp.get_complex() );
		}
	}

	*n_var = tmp_n_var;
	*n2_var = tmp_n2_var;

	if( all_failed_the_same )
		throw error;

	res->set_matrix( &matr );
}

void builtin_table(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable func, *n_var, tmp, tmp_n_var;
	Complex replace;
	Complex start, stop, step;
	Matrix matr;
	int64 steps;
	bool abort_on_error = true;
	bool all_failed_the_same = false;
	error_obj error;
	error.type = ErrorType_None;

	only_right;

	if( res == 0 )
		return;

	{
		utf8_string func_str,var_name;

		replace = Complex( asinx(1.0) * 6.0, expx(2.0) ); // 3*pi + i*(e^2)
		argument_check( "table", right,
		                type_string, &func_str,
		                type_name, &var_name,
		                type_complex, &start,
		                type_complex, &stop,
		                type_int64, &steps,
		                type_complex_default, &replace,
		                type_end );

		if(replace != Complex( asinx(1.0) * 6.0, expx(2.0) ))
			abort_on_error = false;

		if(steps <= 1)
			THROW_ERROR( ErrorType_Domain, _("Domain error: Step must be greater than 1.") );

		if( floatx(start.imaginary) != 0.0 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );

		if( floatx(stop.imaginary) != 0.0 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );

		step = (stop - start) / (steps-1);

		func.set_code_line( func_str.c_str() );
		if(private_varlist)
		{
			try{
				n_var = private_varlist->get_pointer_rw( private_varlist->get_id( var_name.c_str() ) );
			}
			catch(...) {
				n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );
			}
		}
		else
			n_var = global_varlist.get_pointer_rw( global_varlist.get_id( var_name.c_str() ) );

		matr.set_size( uint32(steps), 1 );
	}
	tmp_n_var = *n_var;

	if(start > stop)
		THROW_ERROR( ErrorType_Domain, _("Domain error: Start mustn't be larger than stop.") );

	for( int64 i=0; i < steps; start += step,i++ )
	{
		n_var->set_complex( start );
		try{  func.calc_code_line( &tmp );  all_failed_the_same = false;  }
		catch(error_obj error2)
		{
			if( error.type == ErrorType_None  &&  i == 0 )
			{
				error = error2;
				all_failed_the_same = true;
			}
			else if( error.type != error2.type )
				all_failed_the_same = false;

			if(abort_on_error)
				throw;
			else
				tmp.set_complex( replace );
		}

		if(tmp.get_type() != VariableType_Complex)
		{
			if(abort_on_error)
			{
				try{  *n_var = tmp_n_var;  }
				catch(...) {  } // this catch is empty not by mistake
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "table" );
					THROW_ERROR( ErrorType_General, tmp_err );
				}
			}
			else
				tmp.set_complex( replace );
		}

		matr.set_complex( i+1, 1, *tmp.get_complex() );
	}

	*n_var = tmp_n_var;

	if(all_failed_the_same)
		throw error;

	res->set_matrix( &matr );
}

void builtin_plot(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	enum PLOT_MARKS { PLOT_DOT, PLOT_PLUS, PLOT_SQUARE, PLOT_RHOMB, PLOT_HEART };
	enum PLOT_MODES { MODE_DOTS, MODE_CONNECTED, MODE_CONNECTED_WITHOUT_ASYMPTOTES };
	Picture *pic;
	Complex tmp;
	picture_h::colorrgb color;
	Matrix xlist, ylist;
	floatx x, y;
	PLOT_MARKS mark = PLOT_PLUS;
	PLOT_MODES plot_mode;
	int64 size = 2;
	bool abort = true, prev = false, antialiasing = false;
	float *vertex=0, *vertex_mark=0;
	uint32 *lines=0, *lines_mark=0, nroflines=0, next_vertex=0;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "plot" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
	
	pic = left->get_picture_rw();

	{
		utf8_string smark("Plus"), splot_mode( "Dots" );
		Matrix colormatrix;

		colorrgb_to_colormatrix( picture_h::color::BLUE, &colormatrix );

		argument_check( "plot", right,
		                type_matrix, &xlist,
		                type_matrix, &ylist,
		                type_string_default, &splot_mode,
		                type_string_default, &smark,
		                type_int64_default, &size,
		                type_boolean_default, &abort,
		                type_boolean_default, &antialiasing,
		                type_matrix_default, &colormatrix,
		                type_end );

		if(smark == "Dot")
			mark = PLOT_DOT;
		else if(smark == "Plus")
			mark = PLOT_PLUS;
		else if(smark == "Square")
			mark = PLOT_SQUARE;
		else if(smark == "Rhomb")
			mark = PLOT_RHOMB;
		else if(smark == "Heart")
			mark = PLOT_HEART;
		else
		{
			smark.prepend( _("Unknown mark: ") );
			THROW_ERROR( ErrorType_General, smark.c_str() );
		}

		if( splot_mode == "Dots" )
			plot_mode = MODE_DOTS;
		else if(splot_mode == "Connected" )
			plot_mode = MODE_CONNECTED;
		else if(splot_mode == "Connected without asymptotes" )
			plot_mode = MODE_CONNECTED_WITHOUT_ASYMPTOTES;
		else
		{
			splot_mode.prepend( _("Unknown plot mode: ") );
			THROW_ERROR( ErrorType_General, splot_mode.c_str() );
		}

		if( size < 0  ||  size > 50 )
			THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

		colormatrix_to_colorrgb( colormatrix, &color );
	}

	if( xlist.get_width() != ylist.get_width()  ||  xlist.get_height() != 1  ||
		ylist.get_height() != 1 )
		THROW_ERROR( ErrorType_General, _("Matrix size don't match") );

	try
	{
		if( plot_mode != MODE_DOTS )
		{
			try{
				vertex = new float [xlist.get_width()*2];
				lines = new uint32 [xlist.get_width()*2];
			}
			catch( ... ) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
		}
		if( antialiasing )
		{
			try{
				vertex_mark = new float [12];
				lines_mark = new uint32 [12];
			}
			catch( ... ) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
			
			if( mark == PLOT_PLUS )
			{
				lines_mark[0] = 0;
				lines_mark[1] = 2;
				lines_mark[2] = 4;
				lines_mark[3] = 6;
			}
			else
			{
				lines_mark[0] = 0;
				lines_mark[1] = 2;
				lines_mark[2] = 2;
				lines_mark[3] = 4;
				lines_mark[4] = 4;
				lines_mark[5] = 6;
				lines_mark[6] = 6;
				lines_mark[7] = 8;
				lines_mark[8] = 8;
				lines_mark[9] = 10;
				lines_mark[10] = 10;
				lines_mark[11] = 0;
			}
		}
		
		for(uint32 i=0;i<xlist.get_width();i++)
		{
			tmp = xlist.get_complex(i+1,1);
			if( floatx(tmp.imaginary) != 0.0)
			{
				if(abort)
				{
					THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
				}
				else
				{
					prev = false;
					continue;
				}
			}
			x = floatx(tmp.real);

			tmp = ylist.get_complex(i+1,1);
			if( floatx(tmp.imaginary) != 0.0)
			{
				if(abort)
				{
					THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
				}
				else
				{
					prev = false;
					continue;
				}
			}
			y = floatx(tmp.real);

			if( size == 0 )
			{
				if( plot_mode == MODE_DOTS )
					pic->point( x, y, color );
			}
			else
			{
				switch(mark)
				{
				case PLOT_DOT:
					pic->pic.filled_circle( pic->get_x_pixel(x), pic->get_y_pixel(y), uint32(size),
					                        color, antialiasing);
					break;

				case PLOT_PLUS:
					{ // this mark doesn't look better in antialiased mode, so no such mode exists
						int32 tx,ty;
						tx = pic->get_x_pixel( x );
						ty = pic->get_y_pixel( y );
	
						pic->pic.line(tx-size,ty,tx+size,ty,color);
						pic->pic.line(tx,ty-size,tx,ty+size,color);
					}
					break;

				case PLOT_SQUARE:
					// this mark doesn't look better in antialiased mode, so no such mode exists
					pic->pic.box( pic->get_x_pixel( x )-size, pic->get_y_pixel( y )-size,
					              2*size, 2*size, color );
					break;

				case PLOT_RHOMB:
					if( antialiasing )
					{
						float fx, fy ,fsize=float(size);
						fx = pic->get_float_x_pixel(x);
						fy = pic->get_float_y_pixel(y);
						
						vertex_mark[0] = fx-fsize;
						vertex_mark[1] = fy;
						vertex_mark[2] = fx;
						vertex_mark[3] = fy-fsize;
						vertex_mark[4] = fx+fsize;
						vertex_mark[5] = fy;
						vertex_mark[6] = fx;
						vertex_mark[7] = fy+fsize;
						vertex_mark[8] = fx-fsize;
						vertex_mark[9] = fy;
						pic->pic.draw_lines( vertex_mark, lines_mark, 4, color, true );
					}
					else
					{
						int32 tx,ty;
						tx = pic->get_x_pixel( x );
						ty = pic->get_y_pixel( y );
	
						pic->pic.line(tx-size,ty,tx,ty-size,color);
						pic->pic.line(tx,ty-size,tx+size,ty,color);
						pic->pic.line(tx+size,ty,tx,ty+size,color);
						pic->pic.line(tx,ty+size,tx-size,ty,color);
					}
					break;

				case PLOT_HEART:
					if( antialiasing )
					{
						float fx, fy, fsize = float(size);
						fx = pic->get_float_x_pixel(x);
						fy = pic->get_float_y_pixel(y);

						vertex_mark[0] = fx;
						vertex_mark[1] = fy;
						vertex_mark[2] = fx+fsize/2.0f;
						vertex_mark[3] = fy-fsize/2.0f;
						vertex_mark[4] = fx+fsize;
						vertex_mark[5] = fy;
						vertex_mark[6] = fx;
						vertex_mark[7] = fy+fsize*1.5f;
						vertex_mark[8] = fx-fsize;
						vertex_mark[9] = fy;
						vertex_mark[10] = fx-fsize/2.0f;
						vertex_mark[11] = fy-fsize/2.0f;
						pic->pic.draw_lines( vertex_mark, lines_mark, 6, color, true );
					}
					else
					{
						int32 tx,ty;
						tx = pic->get_x_pixel( x );
						ty = pic->get_y_pixel( y );

						pic->pic.line( tx, ty, tx+size/2, ty-size/2, color );
						pic->pic.line( tx+size/2, ty-size/2, tx+size, ty, color );
						pic->pic.line( tx+size, ty, tx, ty+(size*3)/2, color );

						pic->pic.line( tx, ty, tx-size/2, ty-size/2, color );
						pic->pic.line( tx-size/2, ty-size/2, tx-size, ty, color );
						pic->pic.line( tx-size, ty, tx, ty+(size*3)/2, color );
					}
					break;

				default:
					THROW_ERROR( ErrorType_Internal, "Internal error: Unknown mark" );
				}
			}

			if( plot_mode != MODE_DOTS )
			{
				vertex[next_vertex] = pic->get_float_x_pixel(x);
				vertex[next_vertex+1] = pic->get_float_y_pixel(y);
				next_vertex += 2;
			}
			if( prev )
			{
				if( plot_mode == MODE_CONNECTED )
				{
					lines[nroflines*2] = next_vertex-4;
					lines[nroflines*2+1] = next_vertex-2;
					nroflines++;
				}
				else if( plot_mode == MODE_CONNECTED_WITHOUT_ASYMPTOTES )
				{
					if( !( vertex[next_vertex-3] < 0.0f  &&
					       vertex[next_vertex-1] > float(pic->pic.get_height())  ||  
					       vertex[next_vertex-3] > float(pic->pic.get_height())  &&
					       vertex[next_vertex-1] < 0.0f ) )
					{
						lines[nroflines*2] = next_vertex-4;
						lines[nroflines*2+1] = next_vertex-2;
						nroflines++;
					}
				}
			}
			prev = true;
		}
	}
	catch( ... )
	{
		if( plot_mode != MODE_DOTS )
		{
			delete [] vertex;
			delete [] lines;
		}
		if( antialiasing )
		{
			delete [] vertex_mark;
			delete [] lines_mark;
		}
		left->signal_changed();
		throw;
	}
	if( plot_mode != MODE_DOTS )
	{
		pic->pic.draw_lines( vertex, lines, nroflines, color, antialiasing );

		delete [] vertex;
		delete [] lines;
	}
	if( antialiasing )
	{
		delete [] vertex_mark;
		delete [] lines_mark;
	}
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_polyreg(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	utf8_string str, tmp_str;
	Matrix x, y, a, b;
	Complex tmp;
	int64 grad, max_decimals=10;
	Format fmt;

	only_right;

	if( res == 0 )
		return;

	argument_check( "polyreg", right,
	                type_matrix, &x, type_matrix, &y,
	                type_int64, &grad,
	                type_int64_default, &max_decimals,
	                type_end );

	if( x.get_width() != y.get_width()  ||  x.get_height() != 1  || y.get_height() != 1 )
		THROW_ERROR( ErrorType_General, _("Matrix size don't match") );

	if( max_decimals < 0  ||  max_decimals > 50 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

	if( grad < 0  ||  grad > 99 )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

	if( grad >= x.get_width() )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

//  Setting the a and b Matrix up
	a.set_size(grad+1,x.get_width());
	{
		uint32 row,col;
		for(row=1;row<=a.get_height();row++)
		{
			tmp = x.get_complex(row,1);
			if( floatx(tmp.imaginary) != 0.0)
				THROW_ERROR( ErrorType_Domain, ("Domain error: Imaginary part must be zero.") );

			for(col=1;col<=a.get_width();col++)
				a.set_complex( col, row, Complex( powx( floatx(tmp.real), floatx(col-1)) ) );
		}
	}
	b = y;
	b.trans();
//  a and b Matrix done

//  make the calculating bit
	x = a;
	x.trans();
	matrix_mul(&y,&x,&a); // y = (trans(a)*a)
	y.inverse();          // y = ((trans(a)*a)^-1)
	matrix_mul(&a,&y,&x); // a = ((trans(a)*a)^-1) * trans(a)
	matrix_mul(&x,&a,&b); // x = ((trans(a)*a)^-1) * trans(a) * b ... minstakvadrat anpassat
//  calculating done 

	{
		Complex tmpcomplex;
		bool changed_sign;

		fmt.set_max_decimals( max_decimals );

		try
		{
			for(int32 i=grad;i>=0;i--)
			{
				tmpcomplex = x.get_complex(1,i+1);
				changed_sign = false;
				if(( floatx(tmpcomplex.imaginary) == 0.0) & (floatx(tmpcomplex.real) < 0.0) & (i != grad))
				{
					changed_sign = true;
					tmpcomplex = -tmpcomplex;
				}

				if(i != grad)
				{
					if(changed_sign)
						tmp_str = " - ";
					else
						tmp_str = " + ";
					str.append( tmp_str );
				}

				if( floatx(tmpcomplex.imaginary) != 0.0 )
				{
					tmp_str = "(";
					str.append( tmp_str );
				}

				tmpcomplex.append_to_string( str, fmt );

				if( floatx(tmpcomplex.imaginary) != 0.0 )
				{
					tmp_str = ")";
					str.append( tmp_str );
				}

				if( i >= 2 )
				{
					char tmp[10];
					snprintf( tmp, 10, " x^%ld", i );
					tmp_str = tmp;
					str.append( tmp_str );
				}
				else if( i == 1 )
				{
					tmp_str = " x";
					str.append( tmp_str );
				}
			}
		}
		catch(error_obj error) {  throw;                       }
		catch(...)             {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
	}

	res->set_string( str );
}

void builtin_set_window(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Picture *pic;
	floatx newleft,newright,newbottom,newtop;

	both_left_and_right;

	if( left->get_type() != VariableType_Picture )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "set_window" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	argument_check( "set_window", right,
	                type_floatx, &newleft,
	                type_floatx, &newright,
	                type_floatx, &newbottom, 
	                type_floatx, &newtop,
	                type_end );

	pic = left->get_picture_rw();
	pic->set_window(newleft, newright,  newbottom, newtop);
	pic->clear();
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_get_width(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	uint32 width;

	only_left;

	switch( left->get_type() )
	{
	case VariableType_Picture:
		width = left->get_picture()->pic.get_width();
		break;

	case VariableType_Matrix:
		width = left->get_matrix()->get_width();
		break;

	case VariableType_Array:
		width = left->get_array()->get_size();
		break;

	case VariableType_String:
		width = left->get_string()->get_length();
		break;

	default:
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_width" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	if( res != 0 )
		res->set_complex( Complex( int64(width) ) );
}

void builtin_get_height(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	uint32 height;

	only_left;

	switch( left->get_type() )
	{
	case VariableType_Picture:
		height = left->get_picture()->pic.get_height();
		break;

	case VariableType_Matrix:
		height = left->get_matrix()->get_height();
		break;

	default:
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_height" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}

	if( res != 0 )
		res->set_complex( Complex( int64(height) ) );
}

void builtin_clear(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	switch(right->get_type())
	{
	case VariableType_Picture:
		{
			Picture pic = *right->get_picture();

			pic.clear();

			if( res != 0 )
				res->set_picture( &pic );
		}
		break;

	case VariableType_Matrix:
		{
			Matrix matr = *right->get_matrix();

			matr.fill( Complex( 0 ) );

			if( res != 0 )
				res->set_matrix( &matr );
		}
		break;

	default:
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "clear" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
	}
}

void builtin_get_left(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Picture)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_left" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( Complex( left->get_picture()->get_left() ) );
}

void builtin_get_right(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Picture)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_right" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( Complex( left->get_picture()->get_right() ) );
}

void builtin_get_top(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Picture)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_top" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( Complex( left->get_picture()->get_top() ) );
}

void builtin_get_bottom(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_left;

	if(left->get_type() != VariableType_Picture)
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "get_bottom" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( res != 0 )
		res->set_complex( Complex( left->get_picture()->get_bottom() ) );
}

void builtin_dec(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Complex tmp;

	only_right;

	if( res == 0 )
		return;

	if(right->get_type() == VariableType_Complex)
	{
		tmp = *(right->get_complex());

		tmp.real = floatx(tmp.real);
		tmp.imaginary = floatx(tmp.imaginary);

		res->set_complex( tmp );
	}
	else if(right->get_type() == VariableType_Matrix)
	{
		Matrix matr;

		matr = *right->get_matrix();

		for(uint32 y=1; y <= matr.get_height() ; y++)
		{
			for(uint32 x=1; x <= matr.get_width() ; x++)
			{
				tmp = matr.get_complex( x, y );

				tmp.real = floatx(tmp.real);
				tmp.imaginary = floatx(tmp.imaginary);

				matr.set_complex( x, y, tmp );
			}
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "dec" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_frac(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;
	Complex tmp;
	Variable var;
	Integer highest_bottom(1000);

	if( res == 0 )
		return;

	argument_check( "frac", right,
	                type_variable, &var,
	                type_integer_default, &highest_bottom,
	                type_end );

	if( highest_bottom > int32_max )
		THROW_ERROR( ErrorType_Domain, _("Domain error: The denominator mustn't be larger than 2^31-1.") );

	if(var.get_type() == VariableType_Complex)
	{
		tmp = *(var.get_complex());

		tmp.real = frac(tmp.real, int64(highest_bottom) );
		tmp.imaginary = frac(tmp.imaginary, int64(highest_bottom));

		res->set_complex( tmp );
	}
	else if(var.get_type() == VariableType_Matrix)
	{
		Matrix matr;
		uint32 x,y;

		matr = *(var.get_matrix());

		for(y=1; y <= matr.get_height(); y++)
		{
			for(x=1; x <= matr.get_width(); x++)
			{
				tmp = matr.get_complex( x, y );

				tmp.real = frac(tmp.real, int64(highest_bottom) );
				tmp.imaginary = frac(tmp.imaginary, int64(highest_bottom) );

				matr.set_complex( x, y, tmp );
			}
		}

		res->set_matrix( &matr );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "frac" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void builtin_get_col(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;
	Matrix matr, dest;
	int64 c;
	uint32 col, row;

	if( res == 0 )
		return;

	argument_check( "get_col", left,
	                type_matrix, &matr,
	                type_end );
	argument_check( "get_col", right,
	                type_int64, &c,
	                type_end );

	if( c <= 0  ||  c > int64(matr.get_width()) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

	col = c;
	dest.set_size( 1, matr.get_height() );

	for( row=1; row <= matr.get_height(); row++ )
		dest.set_complex( 1, row, matr.get_complex( col, row ) );

	res->set_matrix( &dest );
}

void builtin_get_row(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;
	Matrix matr, dest;
	int64 r;
	uint32 col, row;

	if( res == 0 )
		return;

	argument_check( "get_row", left,
	                type_matrix, &matr,
	                type_end );
	argument_check( "get_row", right,
	                type_int64, &r,
	                type_end );

	if( r <= 0  ||  r > int64(matr.get_width()) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Value out of range.") );

	row = r;
	dest.set_size( matr.get_width(), 1 );

	for( col=1; col <= matr.get_width(); col++ )
		dest.set_complex( col, 1, matr.get_complex( col, row ) );

	res->set_matrix( &dest );
}

void builtin_error(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;

	utf8_string str, type("General");
	error_obj error;

	argument_check( "error", right,
	                type_string, &str,
	                type_string_default, &type,
	                type_end );

	if( type == "General" )
		error.type = ErrorType_UmpCode_General;
	else if( type == "Domain" )
		error.type = ErrorType_UmpCode_Domain;
	else if( type == "Divide by zero" )
		error.type = ErrorType_UmpCode_Divide_by_zero;
	else
	{
		error_obj error;
		error.type = ErrorType_General;
		snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Unknown type (%s) in function error"), type.c_str() );
		throw error;
	}
	snprintf( error.msg, ERROR_OBJ_MSG_LEN, "%s", str.c_str() );

	throw error;
}

void builtin_merge(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;
	utf8_string str;
	Matrix *matr, matr2, tmp;
	uint32 col, row;

	if( left->get_type() != VariableType_Matrix )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "merge" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	argument_check( "merge", right,
	                type_matrix, &matr2,
	                type_string, &str,
	                type_end );

	matr = left->get_matrix_rw();
	if( matr->get_height() == 0 )
		*matr = matr2;
	else if( matr2.get_height() == 0 )
		;
	else if( str == "Side" )
	{
		if( matr->get_height() != matr2.get_height() )
			THROW_ERROR( ErrorType_General, _("Matrix size don't match") );

		tmp = *matr;
		matr->set_size( tmp.get_width() + matr2.get_width(), tmp.get_height() );

		for( row=1; row <= tmp.get_height(); row++ )
		{
			for( col=1; col <= tmp.get_width(); col++ )
				matr->set_complex( col, row, tmp.get_complex( col, row ) );

			for( col=1; col <= matr2.get_width(); col++ )
				matr->set_complex( col + tmp.get_width(), row, matr2.get_complex( col, row ) );
		}
	}
	else if( str == "Under" )
	{
		if( matr->get_width() != matr2.get_width() )
			THROW_ERROR( ErrorType_General, _("Matrix size don't match") );

		tmp = *matr;
		matr->set_size( tmp.get_width(), tmp.get_height() + matr2.get_height() );

		for( col=1; col <= tmp.get_width(); col++ )
		{
			for( row=1; row <= tmp.get_height(); row++ )
				matr->set_complex( col, row, tmp.get_complex( col, row ) );

			for( row=1; row <= matr2.get_height(); row++ )
				matr->set_complex( col, row + tmp.get_height(), matr2.get_complex( col, row ) );
		}
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Unknown symbol '%s'"), str.c_str() );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_hook_set_next(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() != VariableType_Matrix )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "[set_next]" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( right->get_type() != VariableType_Complex )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "[set_next]" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	left->get_matrix_rw()->set_next( *right->get_complex() );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_matrix(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;
	int64 rows, cols;
	Matrix matr;

	if( res == 0 )
		return;

	argument_check( "matrix", right,
	                type_int64, &rows,
	                type_int64, &cols,
	                type_end );

	if( rows < 0  ||  rows > int64(uint32_max)  ||  cols < 0  ||  cols > int64(uint32_max) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Rows/columns out of range.") );

	matr.set_size( cols, rows );

	res->set_matrix( &matr );
}

void builtin_array(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	only_right;
	int64 size;
	Array array;

	if( res == 0 )
		return;

	argument_check( "array", right,
	                type_int64, &size,
	                type_end );

	if( size < 0  ||  size > int64(uint32_max) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Array size out of range.") );

	array.set_size( uint32(size) );

	res->set_array( &array );
}

void builtin_insert(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	Variable var;
	int64 index;

	both_left_and_right;

	if( left->get_type() != VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "insert" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	argument_check( "insert", right,
	                type_variable, &var,
	                type_int64, &index,
	                type_end );

	if( index == -1 )
		index = int64(uint32_max); // this will append the variable

	if( index < 0  ||  index > int64(uint32_max) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Insert index out of range.") );

	left->get_array_rw()->insert( &var, uint32(index) );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_append(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;

	if( left->get_type() != VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "append" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	left->get_array_rw()->insert( right, uint32_max );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_remove(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;
	int64 index, range = 1;

	if( left->get_type() != VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "remove" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	argument_check( "remove", right,
	                type_int64, &index,
	                type_int64_default, &range,
	                type_end );

	if( index < 0  ||  index > int64(uint32_max) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Remove index out of range.") );

	if( range < 0  ||  range > int64(uint32_max) )
		THROW_ERROR( ErrorType_Domain, _("Domain error: Remove index out of range.") );

	left->get_array_rw()->remove_range( uint32(index), uint32(range) );
	left->signal_changed();

	if( res != 0 )
		res->set_void();
}

void builtin_array_of_two(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
// intern function used by CodeLine::parse_pow_mods
	both_left_and_right;
	Array array;

	if( res == 0 )
		return;

	array.add_variable( left );
	array.add_variable( right );
	
	res->set_array( &array );
}

void builtin_pow_mod(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj)
{
	both_left_and_right;
	try
	{
		Integer n, p, m, tmp;


		if( res == 0 )
			return;

		argument_check( "(powMod)", left,
		                type_integer, &n,
		                type_end );

		argument_check( "(powMod)", right,
		                type_integer, &p,
		                type_integer, &m,
		                type_end );

		powMod( &tmp, n, p, m );

		res->set_complex( Real( tmp ) );
	}
	catch( ... ) // if this failes, let's fall back to the normal pow and mod functions
	{
		Variable tmp;
		
		if( right->get_type() != VariableType_Array )
			THROW_ERROR( ErrorType_Internal, "Internal error: (powMod)'s right isn't an array!!" );
		
		const Array *array = right->get_array();
		if( array->get_size() != 2 )
			THROW_ERROR( ErrorType_Internal, "Internal error: (powMod)'s right array hasn't size 2!!" );
		
		builtin_pow( &tmp, left, array->get_variable(0), private_varlist );
		builtin_modulo( res, &tmp, array->get_variable(1), private_varlist );
	}
}

} // namespace math
