
/*
 *  Math2
 *  Copyright (c) 2003-2007 by Mattias Hultgren <tilda_o_tize@hotmail.com>
 *
 *  See math2.h
 */



#include "math2.h"
#include "math2.intern.h"
#include "keyfile.h"
#include <stdarg.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

namespace math
{

VariableList global_varlist;
Format *intern_format = 0;

Vector<utf8_string> autoload_directories;

Complex degfix( 1.0 );

const char *VARIABLE_TYPE_NAMES[] = {
                                     "Complex",
                                     "Matrix",
                                     "Array",
                                     "String",
                                     "Picture",
                                     "Boolean",
                                     "Void",
                                     0
                                    };


void init(void) throw(error_obj)
{
	static bool already_inited = false;
	Variable tmpvar;

	if(!already_inited)
	{
		already_inited = true;
		srand(time(0));

		tmpvar.set_boolean( false );
		tmpvar.set_name( "false" );
		global_varlist.create( tmpvar, true, false, false); // constant
		tmpvar.set_boolean( true );
		tmpvar.set_name( "true" );
		global_varlist.create( tmpvar, true,false,false); // constant

		tmpvar.set_complex( Complex( PI ) );
		tmpvar.set_name( "pi" );
		global_varlist.create( tmpvar , true,false,false); // constant
		tmpvar.set_complex( Complex(0, 1) );
		tmpvar.set_name( "i" );
		global_varlist.create( tmpvar , true,false,false);  // constant
		tmpvar.set_complex( Complex( E ) );
		tmpvar.set_name( "e" );
		global_varlist.create( tmpvar , true,false,false);  // constant

		tmpvar.set_complex( Complex( 0 ) );
		tmpvar.set_name( "x" );
		global_varlist.create( tmpvar , false,false,true);
		tmpvar.set_name( "y" );
		global_varlist.create( tmpvar , false,false,true);

		tmpvar.set_void();
		tmpvar.set_name( "void" );
		global_varlist.create( tmpvar, true, false, false);

		function_list.add_function( "append", builtin_append, 15, true, true );
		function_list.add_function( "(array_of_two)", builtin_array_of_two, 15, true, true );
		function_list.add_function( "axes", builtin_axes, 15, true, true );
		function_list.add_function( "circle", builtin_circle, 15, true, true );
		function_list.add_function( "(eval)", builtin_round_eval, 15, true, true );
		function_list.add_function( "[get]", builtin_hook_get, 15, true, true );
		function_list.add_function( "get_bottom", builtin_get_bottom, 15, true, false );
		function_list.add_function( "get_col", builtin_get_col, 15, true, true );
		function_list.add_function( "get_height", builtin_get_height, 15, true, false );
		function_list.add_function( "get_left", builtin_get_left, 15, true, false );
		function_list.add_function( "get_right", builtin_get_right, 15, true, false );
		function_list.add_function( "get_row", builtin_get_row, 15, true, true );
		function_list.add_function( "get_top", builtin_get_top, 15, true, false );
		function_list.add_function( "get_type", builtin_get_type, 15, true, false );
		function_list.add_function( "get_width", builtin_get_width, 15, true, false );
		function_list.add_function( "grid", builtin_grid, 15, true, true );
		function_list.add_function( "insert", builtin_insert, 15, true, true );
		function_list.add_function( "line", builtin_line, 15, true, true );
		function_list.add_function( "merge", builtin_merge, 15, true, true );
		function_list.add_function( "pixel_circle", builtin_pixel_circle, 15, true, true );
		function_list.add_function( "pixel_line", builtin_pixel_line, 15, true, true );
		function_list.add_function( "plot", builtin_plot, 15, true, true );
		function_list.add_function( "(powMod)", builtin_pow_mod, 15, true, true );
		function_list.add_function( "remove", builtin_remove, 15, true, true );
		function_list.add_function( "[set]", builtin_hook_set, 15, true, true );
		function_list.add_function( "[set_next]", builtin_hook_set_next, 15, true, true );
		function_list.add_function( "set_pixel", builtin_set_pixel, 15, true, true );
		function_list.add_function( "set_point", builtin_set_point, 15, true, true );
		function_list.add_function( "set_window", builtin_set_window, 15, true, true );


		function_list.add_function( "abs", builtin_abs, 14, false, true );
		function_list.add_function( "acos", builtin_acos, 14, false, true );
		function_list.add_function( "acosh", builtin_acosh, 14, false, true );
		function_list.add_function( "adj", builtin_adj, 14, false, true );
		function_list.add_function( "arg", builtin_arg, 14, false, true );
		function_list.add_function( "array", builtin_array, 14, false, true );
		function_list.add_function( "asin", builtin_asin, 14, false, true );
		function_list.add_function( "asinh", builtin_asinh, 14, false, true );
		function_list.add_function( "atan", builtin_atan, 14, false, true );
		function_list.add_function( "atanh", builtin_atanh, 14, false, true );
		function_list.add_function( "clear", builtin_clear, 14, false, true );
		function_list.add_function( "comb", builtin_comb, 14, false, true );
		function_list.add_function( "cos", builtin_cos, 14, false, true );
		function_list.add_function( "cosh", builtin_cosh, 14, false, true );
		function_list.add_function( "dec", builtin_dec, 14, false, true );
		function_list.add_function( "delete", builtin_delete, 14, false, true );
		function_list.add_function( "det", builtin_det, 14, false, true );
		function_list.add_function( "error", builtin_error, 14, false, true );
		function_list.add_function( "frac", builtin_frac, 14, false, true );
		function_list.add_function( "fPart", builtin_fpart, 14, false, true );
		function_list.add_function( "im", builtin_im, 14, false, true );
		function_list.add_function( "iPart", builtin_ipart, 14, false, true );
		function_list.add_function( "iSqrt", builtin_isqrt, 14, false, true );
		function_list.add_function( "log", builtin_log, 14, false, true );
		function_list.add_function( "log2", builtin_log2, 14, false, true );
		function_list.add_function( "ln", builtin_ln, 14, false, true );
		function_list.add_function( "make_constant", builtin_make_constant, 14, false, true );
		function_list.add_function( "make_type_locked", builtin_make_type_locked, 14, false, true );
		function_list.add_function( "make_undeletable", builtin_make_undeletable, 14, false, true );
		function_list.add_function( "matrix", builtin_matrix, 14, false, true );
		function_list.add_function( "nDeriv", builtin_nderiv, 14, false, true );
		function_list.add_function( "nInt", builtin_nint, 14, false, true );
		function_list.add_function( "nSolve", builtin_nsolve, 14, false, true );
		function_list.add_function( "perm", builtin_perm, 14, false, true );
		function_list.add_function( "picture", builtin_picture, 14, false, true );
		function_list.add_function( "polyreg", builtin_polyreg, 14, false, true );
		function_list.add_function( "rand", builtin_rand, 14, false, false );
		function_list.add_function( "re", builtin_re, 14, false, true );
		function_list.add_function( "shade", builtin_shade, 14, false, true );
		function_list.add_function( "sin", builtin_sin, 14, false, true );
		function_list.add_function( "sinh", builtin_sinh, 14, false, true );
		function_list.add_function( "sum", builtin_sum, 14, false, true );
		function_list.add_function( "sqrt", builtin_sqrt, 14, false, true );
		function_list.add_function( "table", builtin_table, 14, false, true );
		function_list.add_function( "table2d", builtin_table2d, 14, false, true );
		function_list.add_function( "tan", builtin_tan, 14, false, true );
		function_list.add_function( "tanh", builtin_tanh, 14, false, true );
		function_list.add_function( "time", builtin_time, 14, false, false );
		function_list.add_function( "++", builtin_inc, 14, true, false );
		function_list.add_function( "--", builtin_decrement, 14, true, false );
		function_list.add_function( "-(neg)", builtin_neg, 14, false, true );

		function_list.add_function( "!", builtin_faculty, 13, true, false );

		function_list.add_function( "^", builtin_pow, 12, true, true );

		function_list.add_function( "*", builtin_mul, 11, true, true );
		function_list.add_function( "/", builtin_div, 11, true, true );
		function_list.add_function( "mod", builtin_modulo, 11, true, true );
		function_list.add_function( "\\", builtin_intdiv, 11, true, true );

		function_list.add_function( "+", builtin_plus, 10, true, true );
		function_list.add_function( "-", builtin_minus, 10, true, true );

		function_list.add_function( "==", builtin_equal, 9, true, true );
		function_list.add_function( "!=", builtin_notequal, 9, true, true );
		function_list.add_function( ">", builtin_greater, 9, true, true );
		function_list.add_function( "<", builtin_smaller, 9, true, true );
		function_list.add_function( ">=", builtin_greater_eq, 9, true, true );
		function_list.add_function( "<=", builtin_smaller_eq, 9, true, true );
		function_list.add_function( "~=", builtin_almost_eq, 9, true, true );

		function_list.add_function( "and", builtin_and, 8, true, true );
		function_list.add_function( "or", builtin_or, 8, true, true );
		function_list.add_function( "xor", builtin_xor, 8, true, true );
		function_list.add_function( "not", builtin_not, 8, false, true );

		function_list.add_function( "=", builtin_set, 7, true, true );
		function_list.add_function( "+=", builtin_set_add, 7, true, true );
		function_list.add_function( "-=", builtin_set_sub, 7, true, true );
		function_list.add_function( "*=", builtin_set_mul, 7, true, true );
		function_list.add_function( "/=", builtin_set_div, 7, true, true );
		function_list.add_function( "\\=", builtin_set_intdiv, 7, true, true );
		function_list.add_function( "^=", builtin_set_pow, 7, true, true );
	}
}


void add_function( const utf8_string &name, BuiltinFunctionPtr newfunction, bool needleft,
                   bool needright ) throw(error_obj)
{
	function_list.add_function( name, newfunction, 14, needleft, needright );
}


void argument_check(const char *function, const Variable *input, ...) throw(error_obj)
{
	va_list ap;
	argument_types type;
	const Variable *var = 0;
	const Array *array;
	Array tmparray;
	Complex tmp;

	Complex *out_complex;
	floatx *out_floatx;
	int64 *out_int64;
	Integer *out_integer;
	bool *out_boolean;
	Picture *out_picture;
	Matrix *out_matrix;
	utf8_string *out_string;
	Variable *out_variable;

	va_start(ap,input);

	try
	{
		switch(input->get_type())
		{
		case VariableType_Array:
			array = input->get_array();
			argument_check_do_the_check:
			for(uint32 i=0;i<array->get_size();)
			{
				type = argument_types( va_arg(ap, int) );
				if(type == type_end)
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
					THROW_ERROR( ErrorType_General, tmp_err );
				}

				if(var == 0)
					var = array->get_variable(i);
				switch(type)
				{
				case type_complex:
				case type_complex_default:
					out_complex = va_arg(ap, Complex *);
					if(var->get_type() == VariableType_Complex)
					{
						*out_complex = *(var->get_complex());
						var = 0;
					}
					else
					{
						if(type == type_complex)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_floatx:
				case type_floatx_default:
					out_floatx = va_arg(ap, floatx *);
					if(var->get_type() == VariableType_Complex)
					{
						tmp = *(var->get_complex());
						if(floatx(tmp.imaginary) != 0.0)
							THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );

						*out_floatx = floatx(tmp.real);
						var = 0;
					}
					else
					{
						if(type == type_floatx)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_int64:
				case type_int64_default:
					out_int64 = va_arg(ap, int64 *);
					if(var->get_type() == VariableType_Complex)
					{
						tmp = *(var->get_complex());

						if(floatx(tmp.imaginary) != 0.0)
							THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
						if(ipart(tmp.real) != tmp.real)
							THROW_ERROR( ErrorType_Domain, _("Domain error: The number must be an integer.") );

						if(floatx(tmp.real) >= int64_max) {
							THROW_ERROR( ErrorType_Domain, _("Domain error: The number must be below 2^64.") );
						} else if(floatx(tmp.real) <= int64_min) {
							THROW_ERROR( ErrorType_Domain, _("Domain error: The number must be above -2^64.") );
						} else
							*out_int64 = int64(floatx(tmp.real));
						var = 0;
					}
					else
					{
						if(type == type_int64)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_integer:
				case type_integer_default:
					out_integer = va_arg(ap, Integer *);
					if(var->get_type() == VariableType_Complex)
					{
						tmp = *(var->get_complex());

						if(floatx(tmp.imaginary) != 0.0)
							THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
						if( !tmp.real.isInteger() )
							THROW_ERROR( ErrorType_Domain, _("Domain error: The number must be an integer.") );

						tmp.real.get_top( out_integer );
						var = 0;
					}
					else
					{
						if(type == type_integer)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_boolean:
				case type_boolean_default:
					out_boolean = va_arg(ap, bool *);
					if(var->get_type() == VariableType_Boolean)
					{
						*out_boolean = var->get_boolean();
						var = 0;
					}
					else
					{
						if(type == type_boolean)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_picture:
				case type_picture_default:
					out_picture = va_arg(ap, Picture *);
					if(var->get_type() == VariableType_Picture)
					{
						*out_picture = *(var->get_picture());
						var = 0;
					}
					else
					{
						if(type == type_picture)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_matrix:
				case type_matrix_default:
					out_matrix = va_arg(ap, Matrix *);
					if(var->get_type() == VariableType_Matrix)
					{
						*out_matrix = *(var->get_matrix());
						var = 0;
					}
					else
					{
						if(type == type_matrix)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_string:
				case type_string_default:
					out_string = va_arg(ap, utf8_string *);
					if(var->get_type() == VariableType_String)
					{
						*out_string = *(var->get_string());
						var = 0;
					}
					else
					{
						if(type == type_string)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_name:
				case type_name_default:
					out_string = va_arg(ap, utf8_string *);
					if(var->has_name())
					{
						*out_string = *(var->get_name());
						var = 0;
					}
					else
					{
						if(type == type_name)
						{
							char tmp_err[ERROR_OBJ_MSG_LEN];
							snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
							THROW_ERROR( ErrorType_General, tmp_err );
						}
					}
					break;

				case type_variable:
				case type_variable_default:
					out_variable = va_arg(ap, Variable *);

					*out_variable = *var;
					var = 0;
					break;

				default:
					printf("argument_check was called in an unsupported way\n\n");
					exit(-1);
				}
				if(var == 0)
					i++;
			}
			break;

		default:
			tmparray.clear();
			tmparray.add_variable( input );
			array = &tmparray;
			goto argument_check_do_the_check;
		}


		bool loop=true;
		do{
			switch(argument_types( va_arg(ap, int) ))
			{
			case type_end:
				loop = false;
				break;

			case type_complex_default:
			case type_floatx_default:
			case type_int64_default:
			case type_integer_default:
			case type_boolean_default:
			case type_picture_default:
			case type_matrix_default:
			case type_string_default:
			case type_name_default:
			case type_variable_default:
				break;

			default:
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Argument list doesn't match for %s"), function );
					THROW_ERROR( ErrorType_General, tmp_err );
				}
			}
			out_complex = va_arg(ap, Complex *); // this is not always a legal pointer, but all pointers are
		}while(loop);                            // equal in size so the code is OK
	}
	catch(error_obj error)
	{
		va_end(ap);
		throw;
	}
	catch(...)
	{
		va_end(ap);
		THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );
	}
	va_end(ap);
}

void colorrgb_to_colormatrix(const picture_h::colorrgb &color, Matrix *matr) throw(error_obj)
{
	Complex tmp;

	if(matr == 0)
		THROW_ERROR( ErrorType_General, _("Invalid matrix width") );

	matr->set_size( 3, 1 );

	tmp.imaginary = Real(0);

	tmp.real = Real( int64(color.red), 255 );
	matr->set_complex(1,1,tmp);

	tmp.real = Real( int64(color.green), 255 );
	matr->set_complex(2,1,tmp);

	tmp.real = Real( int64(color.blue), 255 );
	matr->set_complex(3,1,tmp);
}
void colormatrix_to_colorrgb(const Matrix &matr, picture_h::colorrgb *color, uint32 index)
                             throw(error_obj)
{
	Complex tmp;

	if( index == 0 )
		THROW_ERROR( ErrorType_General, _("colormatrix_to_colorrgb: Index mustn't be zero.") );

	if( matr.get_width() != 3  ||  matr.get_height() < index )
		THROW_ERROR( ErrorType_General, _("Invalid matrix width") );

	tmp = matr.get_complex( 1, index );
	tmp.real = floatx(tmp.real) * 255.0 + 0.5;
	if( floatx(tmp.real) > 255.0 )
		color->red = 255;
	else if( floatx(tmp.real) < 0.0 )
		color->red = 0;
	else
		color->red = uint8(floatx(tmp.real));

	tmp = matr.get_complex( 2, index );
	tmp.real = floatx(tmp.real) * 255.0 + 0.5;
	if( floatx(tmp.real) > 255.0 )
		color->green = 255;
	else if( floatx(tmp.real) < 0.0 )
		color->green = 0;
	else
		color->green = uint8(floatx(tmp.real));

	tmp = matr.get_complex( 3, index );
	tmp.real = floatx(tmp.real) * 255.0 + 0.5;
	if( floatx(tmp.real) > 255.0 )
		color->blue = 255;
	else if( floatx(tmp.real) < 0.0 )
		color->blue = 0;
	else
		color->blue = uint8(floatx(tmp.real));
}

void colormatrix_to_colorrgbs(const Matrix &matr, Vector<picture_h::colorrgb> &colors)
                              throw(error_obj)
{
	colors.set_size_filled( int(matr.get_height()) );
	
	for( uint32 i=1; i<=matr.get_height(); i++ )
		colormatrix_to_colorrgb( matr, colors[i-1], i );
}

void color_list_get_color( const Vector<picture_h::colorrgb> &colors, floatx min, floatx max,
                           floatx value, picture_h::colorrgb *out_color)
{
	int base;

	if( min >= max  ||  colors.get_size() == 0 )
		return;
	if( value < min )
		value = min;
	if( value > max )
		value = max;

	value = (value - min ) / (max - min );
	value = value * floatx(colors.get_size()-1);
	base = int(value);
	value -= truncx( value );

	if( base >= (colors.get_size()-1) )
		*out_color = *colors[base];
	else
	{
		out_color->red = uint8( floatx(colors[base]->red)*(1.0-value) +
		                        floatx(colors[base+1]->red)*value );
		out_color->green = uint8( floatx(colors[base]->green)*(1.0-value) +
		                          floatx(colors[base+1]->green)*value );
		out_color->blue = uint8( floatx(colors[base]->blue)*(1.0-value) +
		                         floatx(colors[base+1]->blue)*value );
	}
}

bool autoload_file( const utf8_string &name ) throw(error_obj)
{
	utf8_string filename;
	FILE *fp;
	CodeBlock code;

	for( int i=0; i<autoload_directories.get_size(); i++ )
	{
		filename = *autoload_directories[i];
		filename.append( "/" );
		filename.append( name );
		filename.append( ".ump" );

		fp = fopen( filename.c_str(), "r" );
		if( fp != 0 )
		{
			fclose(fp);

			code.set_code_from_file( filename, &name );

			return true;
		}
	}

	return false;
}

void parse_create_symbol_vector( const utf8_string &string, Vector<utf8_string> *symbols) throw(error_obj)
{
	utf8_string tmp;
	int start;

	try
	{
		for( int i=0; string.c_str_from( i )[0] != '\0'; i++ )
		{
			if( string.test_character( i, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_" ) )
			{
				start = i;

				for( i++; string.test_character( i,
				          "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_0123456789" ); i++ )
					;
				tmp.assign( string, start, i-start );
				symbols->append( tmp );
				i--;
			}
			else if( string.test_character( i, "0123456789" )  ||
			         ( string.test_character( i, "." )  &&  string.test_character( i+1, "0123456789" ) ))
			{
				start = i;

				for( i++; string.test_character( i, "0123456789." ); i++ )
					;
				tmp.assign( string, start, i-start );
				symbols->append( tmp );
				i--;
			}
			else if( string.test_character( i, "\"" ) )
			{
				start = i;

				for( i++; !string.test_character( i, "\"" ); i++ )
				{
					if( string.test_character( i, "\\" ) )
					{
						if( string.c_str_from(i)[0] == '\0' )
							THROW_ERROR( ErrorType_General, _("Expected closing \" but found line break or end of file.") );
						if( string.test_character( i, "\"\\" ) )
							i++;
					}
					if(  string.c_str_from(i)[0] == '\0'  ||  string.test_character( i, "\0\n\r" ) )
						THROW_ERROR( ErrorType_General, _("Expected closing \" but found line break or end of file.") );
				}
				tmp.assign( string, start, i-start+1 );
				tmp.remove_escape_sequences();
				symbols->append( tmp );
			}
			else if( !string.test_character( i, " \t\n\r" ) )
			{
				int len=1;
				const char *tmp_c_str = string.c_str_from( i );

				if( (tmp_c_str[0] == '+' && tmp_c_str[1] == '+')  ||  (tmp_c_str[0] == '+' && tmp_c_str[1] == '=')  ||
				    (tmp_c_str[0] == '-' && tmp_c_str[1] == '-')  ||  (tmp_c_str[0] == '-' && tmp_c_str[1] == '=')  ||
				    (tmp_c_str[0] == '*' && tmp_c_str[1] == '=')  ||  (tmp_c_str[0] == '/' && tmp_c_str[1] == '=')  ||
				    (tmp_c_str[0] == '\\' && tmp_c_str[1] == '=')  ||  (tmp_c_str[0] == '^' && tmp_c_str[1] == '=') ||
				    (tmp_c_str[0] == '=' && tmp_c_str[1] == '=')  ||  (tmp_c_str[0] == '~' && tmp_c_str[1] == '=') ||
				    (tmp_c_str[0] == '<' && tmp_c_str[1] == '=')  ||  (tmp_c_str[0] == '>' && tmp_c_str[1] == '=')  ||
				    (tmp_c_str[0] == '!' && tmp_c_str[1] == '=') )
					len = 2;

				tmp.assign( string, i, len );
				symbols->append( tmp );
				i += (len-1);
			}
		}
	}
	catch( error_obj error ) {  throw error;  }
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }
}

} // namespace math
