

/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See math2.h
 */


#include "math2.h"
#include "math2.intern.h"
#include "keyfile.h"
#include <stdlib.h>

namespace math
{

class CodeBlockFunction
{
public:
	utf8_string name;
	CodeBlock *function;
	Vector<ArgumentType> arguments;

	CodeBlockFunction() { name = ""; function = 0; }
	CodeBlockFunction(const CodeBlockFunction &src) throw(error_obj) { name = ""; function = 0; *this = src; }
	~CodeBlockFunction() { delete function; }

	void operator=(const CodeBlockFunction &src) throw(error_obj);
};

void CodeBlockFunction::operator=(const CodeBlockFunction &src) throw(error_obj)
{
	printf("\nCodeBlockFunction::operator=() not implemented yet\n");
	exit(-1);
}

Vector<CodeBlockFunction> code_block_functions;


FunctionList function_list;


void wrapper_check_argument( int i, int i2, const Variable *right ) throw(error_obj)
{
	Variable *var;
	const Variable *tmp;
	int max = code_block_functions[i]->arguments.get_size();
	var = code_block_functions[i]->function->private_varlist.get_pointer_rw( 
	        code_block_functions[i]->function->private_varlist.get_id( 
	        code_block_functions[i]->arguments[i2]->variable_name.c_str()) );

	switch( code_block_functions[i]->arguments[i2]->type )
	{
	case reserved_variable:
		if( max == 1 )
			*var = *right;
		else
			*var = *( right->get_array()->get_variable(i2) );
		break;
		
	case reserved_integer:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
		{
			if( tmp->get_complex()->imaginary == Real(0.0) )
			{
				if(  tmp->get_complex()->real.isInteger() )
					*var = *tmp;
				else
				{
					char tmp_err[ERROR_OBJ_MSG_LEN];
					snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
					THROW_ERROR( ErrorType_General, tmp_err );
				}
			}
			else
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
				THROW_ERROR( ErrorType_General, tmp_err );
			}
		}
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_real:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
		{
			if( tmp->get_complex()->imaginary == Real(0.0) )
				*var = *tmp;
			else
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
				THROW_ERROR( ErrorType_General, tmp_err );
			}
		}
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_complex:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Complex )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_string:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_String )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_matrix:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Matrix )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_picture:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Picture )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_array:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Array )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	case reserved_boolean:
		tmp = (max == 1) ? right : right->get_array()->get_variable(i2);
		if( tmp->get_type() == VariableType_Boolean )
			*var = *tmp;
		else
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), code_block_functions[i]->name.c_str() );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		break;

	default:
		THROW_ERROR( ErrorType_General, "Internal error: Data type unsupported by wrapper_code_block_function, please report it" );
	}
}

void wrapper_code_block_functions( Variable *res, Variable *left,const Variable *right,
                                         const utf8_string &name ) throw(error_obj)
{
	int i;

	for( i=0; i<code_block_functions.get_size(); i++ )
	{
		if( code_block_functions[i]->name == name )
			break;
	}

	if( i == code_block_functions.get_size() )
		THROW_ERROR( ErrorType_General, _("Unknown function") ); // name wasn't found

	if( code_block_functions[i]->arguments.get_size() == 0 )
	{
		if( left != 0  ||  right != 0 )
			THROW_ERROR( ErrorType_General, _("Syntax error") );
	}
	else
	{
		if( left != 0  ||  right == 0 )
			THROW_ERROR( ErrorType_General, _("Syntax error") );

		if( code_block_functions[i]->arguments.get_size() == 1 )
			wrapper_check_argument( i, 0, right );
		else
		{
			int i2;

			if( right->get_type() != VariableType_Array )
				THROW_ERROR( ErrorType_General, _("Too few arguments.") );

			if( right->get_array()->get_size() != code_block_functions[i]->arguments.get_size() )
				THROW_ERROR( ErrorType_General, _("Wrong number of arguments.") );

			for( i2=0; i2<code_block_functions[i]->arguments.get_size(); i2++ )
				wrapper_check_argument( i, i2, right );
		}
	}

	code_block_functions[i]->function->calc( res );
}


FunctionList::FunctionList(const FunctionList &src)
{
	list = src.list;
}


void FunctionList::add_function( const utf8_string &new_name, BuiltinFunctionPtr new_func, int new_priority,
                                 bool need_left, bool need_right, bool new_isdeletable, bool hard_linking ) throw(error_obj)
{
	FunctionListElement new_element;

	check_name_against_reserved(new_name);

	if( global_varlist.check_for( new_name ) )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("\"%s\" is not a unique name."), new_name.c_str() );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( check_for( new_name ) )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("\"%s\" is not a unique name."), new_name.c_str() );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	if( new_isdeletable  &&  hard_linking )
		THROW_ERROR( ErrorType_General, _("Deletable function tries to be added with hard linking.") );

	try{  new_element.name = new_name;  }
	catch(...) { THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }

	new_element.func = new_func;
	new_element.priority = new_priority;
	new_element.need_left = need_left;
	new_element.need_right = need_right;
	new_element.hard_linking = hard_linking;
	new_element.isdeletable = new_isdeletable;
	new_element.is_code_block_function = false;

	try
	{
		list.append( new_element );
	}
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }  
}

void FunctionList::add_this_code_block_function( const utf8_string &new_name,
                                                 CodeBlock *function, 
                                                 const Vector<ArgumentType> &arguments )
                                                 throw(error_obj)
{
	CodeBlockFunction *tmp = 0;

	add_function( new_name, BuiltinFunctionPtr(wrapper_code_block_functions), 14, false, (arguments.get_size()!=0), true, false );
	set_as_code_block_function( new_name );
	try
	{
		tmp = new CodeBlockFunction;

		tmp->name = new_name;
		tmp->function = function;
		tmp->arguments = arguments;

		code_block_functions.append_this( tmp );
	}
	catch(...)
	{
		delete tmp;
		delete_function( new_name );
		THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );
	}
}


// returns 1 if the function was found and deleted
// return -1 if the function was found but was undeletable
// return 0 if the function wasn't found
int FunctionList::delete_function(const utf8_string &name)
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return 0; // name wasn't found

	if(!list[i]->isdeletable)
		return -1; // found but was undeletable

	if( list[i]->is_code_block_function )
	{
		int i2,tmp_size;

		tmp_size = code_block_functions.get_size();
		for( i2=0; i2<tmp_size; i2++ )
		{
			if( code_block_functions[i2]->name == name )
			{
				if( code_block_functions[i2]->function != 0 )
					if( code_block_functions[i2]->function->get_inuse() )
						return -1;

				code_block_functions.remove( i2 );
				break;
			}
		}
		if( i2 == tmp_size )
			THROW_ERROR( ErrorType_General, _("Function to delete not found.") ); // name wasn't found
	}

	list.remove( i );

	return 1; // was found and deleted
}

// returns true if the function was found and isdeletable-flag is cleared
// returns false if the function wasn't found
bool FunctionList::set_undeletable(const utf8_string &name)
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return false; // name wasn't found

	list[i]->isdeletable = false;

	return true;
}

void FunctionList::set_as_code_block_function( const utf8_string &name )
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return; // name wasn't found

	list[i]->is_code_block_function = true;
}

// returns true if there is a function with this name, else false is returned
bool FunctionList::check_for(const utf8_string &name) const
{
	for( int i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			return true;
	}
	return false;
}

BuiltinFunctionPtr FunctionList::get( const utf8_string &name, int *priority, bool *need_left, bool *need_right, bool *hard_linking ) const
{
	int i;

	for( i=0; i<list.get_size(); i++ )
	{
		if( list[i]->name == name )
			break;
	}
	if( i == list.get_size() )
		return 0; // name wasn't found

	*priority = list[i]->priority;
	*need_left = list[i]->need_left;
	*need_right = list[i]->need_right;
	*hard_linking = list[i]->hard_linking;

	return list[i]->func;
}

} // namespace math
