
/*
 *  Math2
 *  Copyright (c) 2004-2007 by Mattias Hultgren <tilda_o_tize@hotmail.com>
 *
 *  See math2.h
 */

 /*
  * This file should NOT be include by more than math2-files
  */

#ifndef MATH2_INTERN_H_
#define MATH2_INTERN_H_



#include "math2.h"

namespace math
{

void parse_create_symbol_vector(const utf8_string &string, Vector<utf8_string> *symbols) throw(error_obj);


typedef void (*CodeBlockFunctionPtr)(Variable *res, Variable *left, const Variable *right,
                                     const utf8_string &name);

enum reserved_type { reserved_function, reserved_variable, reserved_integer,
                     reserved_real,     reserved_complex,  reserved_string,
                     reserved_matrix,   reserved_picture,  reserved_array,
                     reserved_boolean,  reserved_return,   reserved_if,
                     reserved_else,     reserved_while,    reserved_break,
                     reserved_continue };


class ArgumentType
{
public:
	reserved_type type;
	utf8_string variable_name;
};

class FunctionList
{
private:
	class FunctionListElement
	{
	public:
		utf8_string name;
		BuiltinFunctionPtr func;
		int priority;
		bool need_left, need_right;
		bool isdeletable, hard_linking;
		bool is_code_block_function;
	};

	Vector<FunctionListElement> list;

	void set_as_code_block_function( const utf8_string &name );

public:
	FunctionList() {}
	FunctionList(const FunctionList &src);
	~FunctionList() {}

	void add_function(const utf8_string &new_name, BuiltinFunctionPtr new_func, int new_priority, bool need_left, bool need_right,
	                  bool new_isdeletable = false, bool hard_linking = true ) throw(error_obj);

	void add_this_code_block_function(const utf8_string &new_name, CodeBlock *function,
	                                  const Vector<ArgumentType> &arguments) throw(error_obj);

	// returns 1 if the function was found and deleted
	// return -1 if the function was found but was undeletable
	// return 0 if the function wasn't found
	int delete_function(const utf8_string &name);

	// returns true if the function was found and isdeletable-flag is cleared
	// returns false if the function wasn't found
	bool set_undeletable(const utf8_string &name);

	// returns true if there is a function with this name, else false is returned
	bool check_for(const utf8_string &name) const;

	// returns pointer to function, also sets priority, need_left, need_right, hard_linking
	// if function wasn't found zero is returned
	BuiltinFunctionPtr get( const utf8_string &name, int *priority, bool *need_left, bool *need_right, bool *hard_linking ) const;
};
extern FunctionList function_list;

void builtin_hook_get(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_hook_set(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_round_eval(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_hook_set_next(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);

void builtin_append(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_array_of_two(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_axes(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_circle(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_clear(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_bottom(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_col(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_height(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_left(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_right(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_row(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_top(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_type(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_get_width(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_grid(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_insert(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_line(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_merge(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_pixel_circle(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_pixel_line(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_plot(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_pow_mod(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_remove(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_pixel(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_point(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_window(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);


void builtin_abs(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_acos(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_acosh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_adj(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_almost_eq(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_and(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_arg(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_array(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_asin(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_asinh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_atan(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_atanh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_comb(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_cos(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_cosh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_dec(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_decrement(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_delete(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_det(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_div(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_equal(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_error(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_faculty(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_fpart(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_frac(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_greater(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_greater_eq(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_im(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_inc(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_intdiv(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_ipart(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_isqrt(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_ln(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_log(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_log2(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_make_constant(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_make_type_locked(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_make_undeletable(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_matrix(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_minus(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_modulo(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_mul(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_nderiv(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_neg(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_nint(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_not(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_notequal(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_nsolve(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_or(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_perm(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_picture(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_plus(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_polyreg(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_pow(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_rand(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_re(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_add(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_div(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_intdiv(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_mul(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_pow(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_set_sub(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_shade(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_sin(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_sinh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_smaller(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_smaller_eq(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_sqrt(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_sum(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_table(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_table2d(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_tan(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_tanh(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_time(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);
void builtin_xor(Variable *res, Variable *left,const Variable *right, VariableList *private_varlist) throw(error_obj);





} // namespace math


#endif // MATH2_INTERN_H_
