
/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See math2.h
 */

#include "math2.h"

namespace math
{

Picture::Picture() throw(error_obj)
{
	set_size(20,20,-10.0,10.0,-10.0,10.0);
	pic.clear();

	bgcolor = picture_h::color::WHITE;
}
Picture::Picture(const Picture &src) throw(error_obj)
{
	*this = Picture();

	*this = src;
}

bool Picture::operator==(const Picture &pict) const
{
	if( left != pict.left  ||  right != pict.right  ||  top != pict.top  ||  bottom != pict.bottom )
		return false;
	if( !(bgcolor == pict.bgcolor) )
		return false;
	return (pic == pict.pic);
}

void Picture::set_size(int32 width,int32 height,floatx newleft,floatx newright,
                       floatx newbottom,floatx newtop) throw(error_obj)
{
	if( width <= 1  ||  height <= 1 )
		THROW_ERROR( ErrorType_General, _("Invalid width and/or height") );

	pic.set_isBackground( true );
	pic.set_size(width,height);
	set_window(newleft,newright,newbottom,newtop);
}

void Picture::set_window(floatx newleft,floatx newright,floatx newbottom,floatx newtop) throw(error_obj)
{
	if( newleft >= newright || newtop <= newbottom )
		THROW_ERROR( ErrorType_General, _("Invalid window size") );

	left = newleft;
	right = newright;
	top = newtop;
	bottom = newbottom;
}

void Picture::operator=(const Picture &src) throw(error_obj)
{
	pic = src.pic;

	left = src.left;
	right = src.right;
	top = src.top;
	bottom = src.bottom;

	bgcolor = src.bgcolor;
}

void Picture::line(floatx x1, floatx y1, floatx x2, floatx y2, bool antialiasing,
                   const picture_h::colorrgb &color)
{
	if( antialiasing )
		pic.line_antialiasing( get_float_x_pixel(x1), get_float_y_pixel(y1),
		                       get_float_x_pixel(x2), get_float_y_pixel(y2), color );
	else
		pic.line(get_x_pixel(x1),get_y_pixel(y1),get_x_pixel(x2),get_y_pixel(y2),color);
}

floatx Picture::get_x_value(int32 xpixel) const
{
	if(pic.get_width() == 0)
		return 0.0;

	return left + floatx(xpixel) * ( (right - left) / floatx(pic.get_width()) );
}
floatx Picture::get_y_value(int32 ypixel) const
{
	if(pic.get_height() == 0)
		return 0.0;

	return bottom + floatx(pic.get_height() -1 - ypixel) * ( (top - bottom) / floatx(pic.get_height()) );
}

floatx Picture::get_unit_x_pixel(void) const
{
	if(pic.get_width() == 0)
		return 0.0;

	return (right - left) / floatx( pic.get_width() );
}

floatx Picture::get_unit_y_pixel(void) const
{
	if(pic.get_height() == 0)
		return 0.0;

	return (top - bottom) / floatx( pic.get_height() );
}

float Picture::get_float_x_pixel(floatx xvalue) const
{
	float tmp;

	if(pic.get_width() == 0)
		return 0.0f;

	tmp = (right - left) / floatx(pic.get_width());
	tmp = (xvalue - left) / tmp;

	return tmp;
}

float Picture::get_float_y_pixel(floatx yvalue) const
{
	float tmp;

	if(pic.get_height() == 0)
		return 0.0f;

	tmp = (bottom - top) / floatx(pic.get_height());
	tmp = (yvalue - top) / tmp;

	return tmp;
}

int32 Picture::get_x_pixel(floatx xvalue) const
{
	floatx tmp;
	floatx max = floatx(int32_max);
	floatx min = floatx(int32_min);

	if(pic.get_width() == 0)
		return 0;

	tmp = (right - left) / floatx(pic.get_width());
	tmp = (xvalue - left) / tmp;
	if(tmp > max)
		return int32_max;
	else if(tmp < min)
		return int32_min;
	else
		return int32(tmp);
}
int32 Picture::get_y_pixel(floatx yvalue) const
{
	floatx tmp;
	floatx max = floatx(int32_max);
	floatx min = floatx(int32_min);

	if(pic.get_height() == 0)
		return 0;

	tmp = (bottom - top) / floatx(pic.get_height());
	tmp = (yvalue - top) / tmp;
	if(tmp > max)
		return int32_max;
	else if(tmp < min)
		return int32_min;
	else
		return int32(tmp);
}

void Picture::point(floatx x,floatx y,const picture_h::colorrgb &color)
{
	pic.set_pixel(get_x_pixel(x),get_y_pixel(y),color);
}


} // namespace math
