
/*
 *  Ump - Unnamed Math Program
 *  Copyright (c) 2005 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */

#include <math.h>
#include "matrix_editor.h"
#include "main.h"
#include "ump_editor.h"
#include "ump_simplewin.h"
#include "ump_advancedwin.h"
#include "ump_graphwin.h"
#include "ump_prefwin.h"
#include "ump_aboutwin.h"
#include "vector.h"

const int matrix_editor_max_size = 500;

struct MatrixEditorPage
{
	Matrix matrix;
	GtkWidget *rows, *cols, *rows_label, *cols_label;
	gint notebook_index; 
};

struct MatrixEditor
{
	Vector<MatrixEditorPage> pages;
	GtkWidget *window, *notebook, *vbox, *menu, *bottom_label;
	Format format;
}matrix_editor;


void aboutwin_show_matrix_editor(void) { aboutwin_show(GTK_WINDOW(matrix_editor.window)); }
void prefwin_show_matrix_editor(void) { prefwin_show(GTK_WINDOW(matrix_editor.window)); }


void matrix_editor_add_matrix( const char *name, const Matrix &src, gint pos ) throw(error_obj);

void matrix_element_got_focus( GtkWidget *widget, gpointer user_data );
gboolean matrix_size_update( GtkWidget *widget, GdkEventFocus *event, gpointer user_data );
void matrix_element_update_insert( GtkEditable *editable, gchar *new_text, gint new_text_length,
gint *position, gpointer user_data );
void matrix_element_update_delete( GtkEditable *editable, gint start_pos, gint end_pos,
gpointer user_data);
void matrix_element_update( GtkEditable *editable, gpointer user_data );
gboolean matrix_element_focus_out( GtkWidget *widget, GdkEventFocus *event, gpointer user_data );

void menu_matrix_new(void);
void menu_matrix_new_with_name_func(void);
void menu_matrix_open(void);
void menu_matrix_close(void);
void menu_matrix_delete(void);
void menu_matrix_calc_inverse(void);
void menu_matrix_calc_transpose(void);
void menu_matrix_calc_determinant(void);
void menu_matrix_calc_adjoint(void);
void menu_matrix_calc_identity(void);
void menu_matrix_calc_add(void);
void menu_matrix_calc_substract(void);
void menu_matrix_calc_multiply(void);
void menu_matrix_calc_power(void);
void menu_matrix_calc_fill_with(void);


GtkWidget* build_menu(void)
{
	GtkActionGroup *actions;
	GtkUIManager *ui_manager;
	GError *error = NULL;
	static const gchar *menu_str =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Preferences'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='MatrixMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='NewName'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='CloseMatrix'/>"
"      <menuitem action='Delete'/>"
"    </menu>"
"    <menu action='CalculateMenu'>"
"      <menuitem action='Add'/>"
"      <menuitem action='Substract'/>"
"      <menuitem action='Multiply'/>"
"      <menuitem action='Power'/>"
"      <separator/>"
"      <menuitem action='Adjoint'/>"
"      <menuitem action='Determinant'/>"
"      <menuitem action='Inverse'/>"
"      <menuitem action='Transpose'/>"
"      <separator/>"
"      <menuitem action='Identity'/>"
"      <menuitem action='FillWith'/>"
"      <separator/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Simple'/>"
"      <menuitem action='Advanced'/>"
"      <menuitem action='Editor'/>"
"      <menuitem action='Graphtool'/>"
"    </menu>"
"    <menuitem action='About'/>"
"  </menubar>"
"</ui>";
	GtkActionEntry entries[] = {
{ "FileMenu",    NULL,           menu_file },
{ "Preferences", GTK_STOCK_PREFERENCES, menu_pref,     NULL,       NULL, G_CALLBACK(prefwin_show_matrix_editor) },
{ "Close",       GTK_STOCK_CLOSE, menu_file_close,     "<ctrl>W",  NULL, G_CALLBACK(matrix_editor_close) },
{ "Quit",        GTK_STOCK_QUIT, menu_file_quit,       "<ctrl>Q",  NULL, G_CALLBACK(UMP_gtk_quit) },
{ "MatrixMenu",  NULL,           _("_Matrix") },
{ "New",         GTK_STOCK_NEW,  menu_file_new,        "",         NULL, G_CALLBACK(menu_matrix_new) },
{ "NewName",     GTK_STOCK_NEW,  _("New with name..."), "<ctrl>N", NULL, G_CALLBACK(menu_matrix_new_with_name_func) },
{ "Open",        GTK_STOCK_OPEN, menu_file_open,       "<ctrl>O",  NULL, G_CALLBACK(menu_matrix_open) },
{ "CloseMatrix", GTK_STOCK_CLOSE, menu_file_close,     "",         NULL, G_CALLBACK(menu_matrix_close) },
{ "Delete",      NULL,           edit_edit_delete,     "",         NULL, G_CALLBACK(menu_matrix_delete) },
{ "CalculateMenu", NULL,         _("_Calculate") },
{ "Inverse",     NULL,           _("Inverse"),         NULL,       NULL, G_CALLBACK(menu_matrix_calc_inverse) },
{ "Transpose",   NULL,           _("Transpose"),       NULL,       NULL, G_CALLBACK(menu_matrix_calc_transpose) },
{ "Determinant", NULL,           _("Determinant"),     NULL,       NULL, G_CALLBACK(menu_matrix_calc_determinant) },
{ "Adjoint",     NULL,           _("Adjoint"),         NULL,       NULL, G_CALLBACK(menu_matrix_calc_adjoint) },
{ "Identity",    NULL,           _("Identity"),        NULL,       NULL, G_CALLBACK(menu_matrix_calc_identity) },
{ "Power",       NULL,           _("Power..."),        NULL,       NULL, G_CALLBACK(menu_matrix_calc_power) },
{ "FillWith",    NULL,           _("Fill with..."),    NULL,       NULL, G_CALLBACK(menu_matrix_calc_fill_with) },
{ "Add",         NULL,           _("Add..."),          NULL,       NULL, G_CALLBACK(menu_matrix_calc_add) },
{ "Substract",   NULL,           _("Substract..."),    NULL,       NULL, G_CALLBACK(menu_matrix_calc_substract) },
{ "Multiply",    NULL,           _("Multiply..."),     NULL,       NULL, G_CALLBACK(menu_matrix_calc_multiply) },
{ "ToolsMenu",   NULL,           menu_tools },
{ "Simple",      NULL,           menu_tools_simple,    NULL,       NULL, G_CALLBACK(simplewin_show) },
{ "Advanced",    NULL,           menu_tools_advanced,  NULL,       NULL, G_CALLBACK(advancedwin_show) },
{ "Editor",      NULL,           menu_tools_editor,    NULL,       NULL, G_CALLBACK(editor_showwin) },
{ "Graphtool",   NULL,           menu_tools_graphtool, NULL,       NULL, G_CALLBACK(graphwin_show) },
{ "About",       NULL,           menu_about,           NULL,       NULL, G_CALLBACK(aboutwin_show_matrix_editor) }
};
	guint n_entries = G_N_ELEMENTS(entries);


	actions = gtk_action_group_new("Actions");
	gtk_action_group_add_actions(actions, entries, n_entries, NULL);
	ui_manager = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui_manager, actions, 0);
	gtk_window_add_accel_group( GTK_WINDOW(matrix_editor.window), gtk_ui_manager_get_accel_group(ui_manager) );
	if(!gtk_ui_manager_add_ui_from_string(ui_manager, menu_str, -1, &error))
	{
		g_message("building menus failed: %s", error->message);
		g_error_free(error);
		return 0;
	}
	gtk_ui_manager_ensure_update(ui_manager);

	return gtk_ui_manager_get_widget(ui_manager, "/MenuBar");
}


GtkWindow* matrix_editor_show(void)
{
	matrix_editor.format.set_max_decimals( 20 );
	if( matrix_editor.window == 0 )
	{
		GtkWidget *tmp, *tmp2;

		matrix_editor.window = gtk_window_new( GTK_WINDOW_TOPLEVEL );

		gtk_window_set_title( GTK_WINDOW(matrix_editor.window), _("Ump - Matrix editor") );
		g_signal_connect_swapped( G_OBJECT(matrix_editor.window), "delete-event",
		                          G_CALLBACK(matrix_editor_close), NULL );

		matrix_editor.vbox = gtk_vbox_new(FALSE,0);
		gtk_container_add( GTK_CONTAINER(matrix_editor.window), matrix_editor.vbox );


		if( (matrix_editor.menu = build_menu()) == 0 )
		{
			gtk_widget_destroy( matrix_editor.window );
			matrix_editor.window = 0;
			return 0;
		}
		gtk_box_pack_start( GTK_BOX(matrix_editor.vbox), matrix_editor.menu, FALSE, FALSE, 0 );

		matrix_editor.notebook = gtk_notebook_new();
		gtk_box_pack_start( GTK_BOX(matrix_editor.vbox), matrix_editor.notebook, TRUE, TRUE, 3 );

		gtk_notebook_set_scrollable( GTK_NOTEBOOK(matrix_editor.notebook), TRUE );

		tmp = gtk_hbox_new( FALSE, 0 );
		gtk_box_pack_start( GTK_BOX(matrix_editor.vbox), tmp, FALSE, TRUE, 0 );

		matrix_editor.bottom_label = gtk_label_new( PRG_NAME );
		gtk_misc_set_alignment(GTK_MISC(matrix_editor.bottom_label), 0.0f, 0.5f);
		gtk_misc_set_padding(GTK_MISC(matrix_editor.bottom_label), 2, 2);
		gtk_box_pack_start( GTK_BOX(tmp), matrix_editor.bottom_label, FALSE, TRUE, 0 );

		tmp2 = gtk_statusbar_new();
		gtk_box_pack_start( GTK_BOX(tmp), tmp2, TRUE, TRUE, 0 );


		gtk_window_resize( GTK_WINDOW(matrix_editor.window), 500, 400 );
		open_windows |= MATRIX_EDITOR_WIN;

		menu_matrix_new();
	}
	else
		gtk_window_present( GTK_WINDOW( matrix_editor.window ) );

	return GTK_WINDOW(matrix_editor.window);
}

void matrix_editor_close(void)
{
	if( matrix_editor.window != 0 )
	{
		while( gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) ) != 0 )
			menu_matrix_close();

		gtk_widget_destroy( matrix_editor.window );
		matrix_editor.window = 0;

		matrix_editor.pages.clear();

		open_windows &= ~MATRIX_EDITOR_WIN;
		if(open_windows == 0)
			UMP_gtk_quit();
	}
}

void matrix_editor_translate(void)
{
	if( matrix_editor.window != 0 )
	{
		GtkWidget *new_menu;

		gtk_window_set_title( GTK_WINDOW( matrix_editor.window ), _("Ump - Matrix editor") );


		if(( new_menu = build_menu()) == 0 )
			return;

		gtk_widget_destroy( matrix_editor.menu );
		matrix_editor.menu = new_menu;

		gtk_box_pack_start(GTK_BOX(matrix_editor.vbox), matrix_editor.menu, FALSE, FALSE, 0);
		gtk_box_reorder_child( GTK_BOX(matrix_editor.vbox), matrix_editor.menu, 0 );

		for( uint32 i=0; i<matrix_editor.pages.get_size(); i++ )
		{
			gtk_label_set_label( GTK_LABEL(matrix_editor.pages[i]->rows_label), _("_Rows: ") );

			gtk_label_set_label( GTK_LABEL(matrix_editor.pages[i]->cols_label), _("_Columns: ") );
		}
	}
}


void matrix_editor_add_matrix( const char *name, const Matrix &src, gint pos ) throw(error_obj)
{
	MatrixEditorPage *page=0;
	GtkAdjustment *tmp_adj;
	GtkWidget *tmp, *vbox, *table;
	utf8_string str;

	try
	{
		page = new MatrixEditorPage;
		page->matrix = src;
		matrix_editor.pages.insert_this( page, pos );
	}
	catch( error_obj error )
	{
		delete page;
		throw;
	}
	catch( ... ) {  delete page;  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }

	if( pos == -1 )
		pos = matrix_editor.pages.get_size() -1;

	matrix_editor.pages[pos]->notebook_index = 0;
	for( gint i=pos-1; i >= 0; i-- )
	{
		if( matrix_editor.pages[i]->notebook_index >= 0 )
		{
			matrix_editor.pages[pos]->notebook_index = matrix_editor.pages[i]->notebook_index +1;
			break;
		}
	}

	vbox = gtk_vbox_new(FALSE,0);
	gtk_notebook_insert_page( GTK_NOTEBOOK(matrix_editor.notebook), vbox,
	gtk_label_new( name ), matrix_editor.pages[pos]->notebook_index );

	tmp = gtk_hbox_new(FALSE, 3);
	gtk_box_pack_start( GTK_BOX(vbox), tmp, FALSE, TRUE, 3 );

	page->rows_label = gtk_label_new_with_mnemonic( _("_Rows: ") );
	gtk_box_pack_start( GTK_BOX(tmp), page->rows_label, FALSE, TRUE, 3 );

	tmp_adj = (GtkAdjustment *) gtk_adjustment_new( gdouble(src.get_height()), 1.0, gdouble(matrix_editor_max_size), 1.0, 5.0, 5.0);
	page->rows = gtk_spin_button_new(tmp_adj, 1.0, 0);
	gtk_box_pack_start( GTK_BOX(tmp), page->rows, FALSE, TRUE, 3 );
	gtk_label_set_mnemonic_widget( GTK_LABEL(page->rows_label), page->rows );


	page->cols_label = gtk_label_new_with_mnemonic( _("_Columns: ") );
	gtk_box_pack_start( GTK_BOX(tmp), page->cols_label, FALSE, TRUE, 3 );

	tmp_adj = (GtkAdjustment *) gtk_adjustment_new( gdouble(src.get_width()), 1.0, gdouble(matrix_editor_max_size), 1.0, 5.0, 5.0);
	page->cols = gtk_spin_button_new(tmp_adj, 1.0, 0);
	gtk_box_pack_start( GTK_BOX(tmp), page->cols, FALSE, TRUE, 3 );
	gtk_label_set_mnemonic_widget( GTK_LABEL(page->cols_label), page->cols );


	tmp = gtk_scrolled_window_new( NULL, NULL );
	gtk_box_pack_start( GTK_BOX(vbox), tmp,  TRUE, TRUE, 3 );
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(tmp), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

	table = gtk_table_new( src.get_height(), src.get_width(), TRUE );
	gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW(tmp), table );

	for( uint32 row=0; row<src.get_height(); row++ )
	{
		for( uint32 col=0; col<src.get_width(); col++ )
		{
			tmp = gtk_entry_new();

			gtk_table_attach( GTK_TABLE(table), tmp, col, col+1, row, row+1, GTK_FILL, GTK_FILL, 0, 0 );
			gtk_widget_set_size_request( tmp, 100, 30 );

			str.clear();
			src.get_complex( col+1, row+1 ).append_to_string( str, matrix_editor.format );

			gtk_entry_set_text( GTK_ENTRY(tmp), str.c_str() );

			g_signal_connect( GTK_OBJECT(tmp), "grab-focus",
			                  G_CALLBACK(matrix_element_got_focus),
			                  GUINT_TO_POINTER((guint(col+1)<<9) + guint(row+1)) );
			
			g_signal_connect_after( GTK_OBJECT(tmp), "focus-in-event",
	    	                        G_CALLBACK(matrix_size_update), GINT_TO_POINTER(pos) );


			g_signal_connect_after( GTK_OBJECT(tmp), "focus-out-event",
			                        G_CALLBACK(matrix_element_focus_out),
			             GUINT_TO_POINTER( (guint(pos)<<18) + (guint(col+1)<<9) + guint(row+1) ) );

			g_signal_connect_after( GTK_OBJECT(tmp), "insert-text",
			                        G_CALLBACK(matrix_element_update_insert),
			             GUINT_TO_POINTER( (guint(pos)<<18) + (guint(col+1)<<9) + guint(row+1) ) );
			g_signal_connect_after( GTK_OBJECT(tmp), "delete-text",
			                        G_CALLBACK(matrix_element_update_delete),
			             GUINT_TO_POINTER( (guint(pos)<<18) + (guint(col+1)<<9) + guint(row+1) ) );
		}
	}
}


void matrix_close_by_name( const char *name )
{
	gint notebook_index, index;

	for( notebook_index=0; ; notebook_index++ )
	{
		if( notebook_index == gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) ) )
			return;

		if( strcmp( gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		      gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) ),
			  name ) == 0 )
			break;
	}

	for( index=0; index<matrix_editor.pages.get_size(); index++ )
	{
		if( notebook_index == matrix_editor.pages[index]->notebook_index )
			break;
	}
	if( index == matrix_editor.pages.get_size() )
	{
		printf( "index == matrix_editor.pages.get_size()....this is a bug\n" );
		return;
	}

	gtk_notebook_remove_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index );

	if( index == (matrix_editor.pages.get_size() -1) )
	{
		matrix_editor.pages.remove( index );

		while( index != 0 )
		{
			if( matrix_editor.pages[ index-1 ]->matrix.get_width() == 0 )
			{
				index--;
				matrix_editor.pages.remove( index );
			}
			else
				break;
		}
	}
	else if( index != matrix_editor.pages.get_size() )
	{
		matrix_editor.pages[index]->matrix.set_size( 0, 0 );
		matrix_editor.pages[index]->notebook_index = -1;
		for( ; index < matrix_editor.pages.get_size(); index++ )
			matrix_editor.pages[index]->notebook_index--;
	}
}

/****************************************
*  Below follows all signal callbacks  *
****************************************/

void matrix_element_got_focus( GtkWidget *widget, gpointer user_data )
{
	uint32 row, col;
	static char str[200];

	col = GPOINTER_TO_UINT(user_data);
	row = 0x1ff & col;
	col >>= 9;

	snprintf( str, 200, "%s: %lu %s: %lu", _("Column"), col, _("Row"), row );

	gtk_label_set_label( GTK_LABEL(matrix_editor.bottom_label), str );
}

void matrix_update_page( int index ) throw(error_obj)
{
	gint notebook_index = matrix_editor.pages[index]->notebook_index;

	matrix_editor_add_matrix(
	       gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
	       gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index )),
	       matrix_editor.pages[index]->matrix, index );

	gtk_widget_show_all( matrix_editor.window );
	gtk_notebook_set_current_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index );
	gtk_notebook_remove_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index+1 );

	matrix_editor.pages.remove( index+1 );
}

gboolean matrix_size_update( GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
	int index, notebook_index;
	uint32 new_width, new_height;
	math::Variable *ptr;

	index = GPOINTER_TO_INT(user_data);
	notebook_index = matrix_editor.pages[index]->notebook_index;

	new_width = uint32(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(matrix_editor.pages[index]->cols)));
	new_height = uint32(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(matrix_editor.pages[index]->rows)));

	try
	{
		if( matrix_editor.pages[index]->matrix.get_height() != new_height  ||
		    matrix_editor.pages[index]->matrix.get_width() != new_width )
		{
			Matrix matr, *src;
			matr.set_size( new_width, new_height );
			src = &(matrix_editor.pages[index]->matrix);

			for( int r=1; r<=matr.get_height()  &&  r<=src->get_height(); r++ )
			{
				for( int c=1; c<=matr.get_width()  &&  c<=src->get_width(); c++ )
					matr.set_complex( c, r, src->get_complex( c, r ) );
			}
			*src = matr;

			matrix_update_page( index );

			ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( 
			gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
			gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ))));

			*(ptr->get_matrix_rw()) = matr;
		}
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	return FALSE;
}

gboolean matrix_element_focus_out( GtkWidget *widget, GdkEventFocus *event, gpointer user_data )
{
	math::CodeLine calculater;
	math::Variable ans;
	utf8_string outstr;
	uint32 index, row, col;
	const gchar *str;

	index = GPOINTER_TO_UINT(user_data);
	row = 0x1ff & index;
	index >>= 9;
	col = 0x1ff & index;
	index >>= 9;

	str = gtk_entry_get_text( GTK_ENTRY(widget) );

	try
	{
		calculater.set_code_line( str, false );
		calculater.calc( &ans );

		matrix_editor.pages[index]->matrix.get_complex( col, row ).append_to_string( outstr,
		                                                                matrix_editor.format );

		gtk_entry_set_text( GTK_ENTRY(widget), outstr.c_str() );
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
}

void matrix_element_update( GtkEditable *editable, gpointer user_data )
{
	math::CodeLine calculater;
	math::Variable ans, *ptr;
	uint32 index, row, col;
	gchar *str;

	index = GPOINTER_TO_UINT(user_data);
	row = 0x1ff & index;
	index >>= 9;
	col = 0x1ff & index;
	index >>= 9;

	str = gtk_editable_get_chars( editable, 0, -1 );

	try
	{
		calculater.set_code_line( str, false );
		calculater.calc( &ans );

		matrix_editor.pages[index]->matrix.set_complex( col, row, *ans.get_complex() );

		ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( 
		gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), 
		matrix_editor.pages[index]->notebook_index ))));

		*(ptr->get_matrix_rw()) = matrix_editor.pages[index]->matrix;
	}
	catch( ... ) { }

	g_free( str );
}

void matrix_element_update_insert( GtkEditable *editable, gchar *new_text, gint new_text_length,
                                   gint *position, gpointer user_data )
{
	matrix_element_update( editable, user_data );
}

void matrix_element_update_delete( GtkEditable *editable, gint start_pos, gint end_pos,
                                   gpointer user_data )
{
	matrix_element_update( editable, user_data );
}

void matrix_changed( math::Variable *ptr )
{
	gint notebook_index, index;
	const char *name = ptr->get_name()->c_str();

	if( ptr->get_type() != math::VariableType_Matrix )
	{
		matrix_close_by_name( ptr->get_name()->c_str() );
		return;
	}

	if( ptr->get_matrix()->get_width() > matrix_editor_max_size  ||
	    ptr->get_matrix()->get_width() == 0  ||
	    ptr->get_matrix()->get_height() > matrix_editor_max_size  ||
	    ptr->get_matrix()->get_height() == 0 )
	{
		matrix_close_by_name( ptr->get_name()->c_str() );
		return;
	}

	for( notebook_index=0; ; notebook_index++ )
	{
		if( notebook_index == gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) ) )
			return;

		if( strcmp( gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		      gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) ),
		    name ) == 0 )
			break;
	}

	for( index=0; index<matrix_editor.pages.get_size(); index++ )
	{
		if( notebook_index == matrix_editor.pages[index]->notebook_index )
			break;
	}

	if( index != matrix_editor.pages.get_size() )
	{
		try{
			matrix_editor.pages[index]->matrix = *(ptr->get_matrix());

			matrix_update_page( index );
		}
		catch( error_obj error )
		{
			GtkWidget *dialog;
			dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT,
			                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}
}

void matrix_deleted( math::Variable *ptr )
{
	matrix_close_by_name( ptr->get_name()->c_str() );
}

/***************************************
*  Below follows only menu callbacks  *
***************************************/

void create_new_matrix_page( const char *name )
{
	Matrix tmp_matr;
	math::Variable var, *ptr;
	Complex value(0);

	try
	{
		tmp_matr.set_size( 3, 3 );
		for( int i=1; i<=3; i++ )
			for( int i2=1; i2<=3; i2++ )
				tmp_matr.set_complex( i, i2, value );

		matrix_editor_add_matrix( name, tmp_matr, -1 );

		var.set_matrix( &tmp_matr );
		var.set_name( name );

		math::global_varlist.create( var, false, true, true );

		ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( name ) );

		ptr->set_change_func( matrix_changed );
		ptr->set_delete_func( matrix_deleted );
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}

	gtk_widget_show_all( matrix_editor.window );

	gtk_notebook_set_current_page( GTK_NOTEBOOK(matrix_editor.notebook), 
	gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) )-1 );
}

void menu_matrix_new(void)
{
	char str[100];
	int i;

	for( i=1; ; i++ )
	{
		snprintf( str, 100, "m%ld", i );

		try{  math::global_varlist.get_id( str );  }
		catch( ... )  {  break;  }  // only throw "Variable not found"
	}

	create_new_matrix_page( str );
}

void menu_matrix_new_with_name_func(void)
{
	GtkWidget *align, *dialog, *entry, *hbox, *label;
	const char *str;

	dialog = gtk_dialog_new_with_buttons( _("New with name..."), GTK_WINDOW(matrix_editor.window),
	                                      GTK_DIALOG_MODAL,	GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );

	align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
	gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0);

	hbox = gtk_hbox_new( FALSE, 3 );
	gtk_container_add( GTK_CONTAINER(align), hbox );

	label = gtk_label_new( _("Name: ") );
	gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, TRUE, 0 );

	entry = gtk_entry_new();
	gtk_box_pack_start( GTK_BOX(hbox), entry, TRUE, TRUE, 0 );

	gtk_widget_show_all( dialog );

	while( true )
	{
		if( gtk_dialog_run( GTK_DIALOG(dialog) ) == GTK_RESPONSE_OK )
			str = gtk_entry_get_text( GTK_ENTRY(entry) );
		else
			break;

		if( str[0] == '\0' )
			break;
		try
		{
			math::check_name_against_reserved( str );

			try{  math::global_varlist.get_id( str );  }
			catch( ... )
			{
				create_new_matrix_page( str );
				break;
			}
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("\"%s\" is not a unique name."), str );
			THROW_ERROR( ErrorType_General, tmp_err );
		}
		catch( error_obj error )
		{
			GtkWidget *dialog2;
			dialog2 = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
			                                  GTK_DIALOG_DESTROY_WITH_PARENT,
			                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
			gtk_dialog_run(GTK_DIALOG(dialog2));
			gtk_widget_destroy(dialog2);
		}
	}
	gtk_widget_destroy( dialog );
}

void menu_matrix_open(void)
{
	GtkWidget *dialog, *treeview;
	GtkListStore *list_store;
	GtkTreeIter iter;
	gint i, i2;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	bool empty_list = true;
	char tmp_str[30];

	enum { COLUMN_NAME, COLUMN_SIZE, N_COLUMNS };

	list_store = gtk_list_store_new( N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING );

	for( i=1; i<=math::global_varlist.get_highest_idnr(); i++ )
	{
		const math::Variable *ptr;
		const char *name;

		try{ ptr = math::global_varlist.get_pointer( i );  }
		catch( ... )  {  continue;  }  // only throw "Variable not found"

		if( ptr->get_type() == math::VariableType_Matrix )
		{
			if( ptr->get_matrix()->get_width() == 0  ||  // don't add matrices of zero size
			    ptr->get_matrix()->get_height() == 0 )
				continue;

			if( ptr->get_matrix()->get_width() > matrix_editor_max_size  ||  // don't add matrices of zero size
			    ptr->get_matrix()->get_height() > matrix_editor_max_size )
				continue;

			name = ptr->get_name()->c_str();

			for( i2=0; i2<gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) ); i2++ )
			{
				if( strcmp( name, gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
				           gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), i2 ))) == 0 )
					break;
			}

			if( i2 != gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) ) )
				continue; // this matrix is already open

			snprintf( tmp_str, 30, "%lux%lu", ptr->get_matrix()->get_height(), ptr->get_matrix()->get_width() );
			gtk_list_store_append( list_store, &iter );
			gtk_list_store_set( list_store, &iter, COLUMN_NAME, name, COLUMN_SIZE, tmp_str, -1);
			empty_list = false;
		}
	}

	if( empty_list )
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
		                                 _("There is no matrix variable to open.") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);

		return;
	}

	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE(list_store), COLUMN_NAME,
	                                      GTK_SORT_ASCENDING );

	dialog = gtk_dialog_new_with_buttons ( _("Open a matrix variable"),
	                                       GTK_WINDOW(matrix_editor.window),
	                                       GTK_DIALOG_DESTROY_WITH_PARENT,
	                                       GTK_STOCK_OK,     GTK_RESPONSE_ACCEPT,
	                                       GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
	                                       NULL );

	treeview = gtk_tree_view_new_with_model( GTK_TREE_MODEL(list_store) );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), treeview, TRUE, TRUE, 3 );

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes( _("Name"), renderer, "text", COLUMN_NAME, NULL);
	gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column );

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes( _("Size"), renderer, "text", COLUMN_SIZE, NULL);
	gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column );

	gtk_widget_show( treeview );

	switch( gtk_dialog_run( GTK_DIALOG(dialog) ) )
	{
	case GTK_RESPONSE_ACCEPT:
		{
			GtkTreeSelection *sel;
			GtkTreeModel *model;
			gchar *name;
			math::Variable *ptr;

			sel = gtk_tree_view_get_selection( GTK_TREE_VIEW(treeview) );

			if( gtk_tree_selection_get_selected( sel, &model, &iter ) )
			{
				gtk_tree_model_get( model, &iter, COLUMN_NAME, &name, -1 );

				try
				{
					ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( name ) );

					matrix_editor_add_matrix( name, *ptr->get_matrix(), -1 );

					ptr->set_change_func( matrix_changed );
					ptr->set_delete_func( matrix_deleted );
				}
				catch( error_obj error )
				{
					GtkWidget *dialog;
					dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
					                                 GTK_DIALOG_DESTROY_WITH_PARENT,
					                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
					gtk_dialog_run(GTK_DIALOG(dialog));
					gtk_widget_destroy(dialog);
				}
				gtk_widget_show_all( matrix_editor.window );
				gtk_notebook_set_current_page( GTK_NOTEBOOK(matrix_editor.notebook), 
				           gtk_notebook_get_n_pages( GTK_NOTEBOOK(matrix_editor.notebook) )-1 );
				g_free( name );
			}
		}
		break;

	case GTK_RESPONSE_REJECT:
		break;
	}
	gtk_widget_destroy( dialog );
}

void menu_matrix_close(void)
{
	gint notebook_index;
	math::Variable *ptr;

	notebook_index = gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) );

	if( notebook_index == -1 )
		return;

	try
	{
		ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id( 
		         gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		         gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ))));

		ptr->set_change_func( 0 );
		ptr->set_delete_func( 0 );

		matrix_close_by_name(
		       gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		       gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) ) );
	}
	catch( ... ){}
}

void menu_matrix_delete(void)
{
	gint notebook_index;

	notebook_index = gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) );

	if( notebook_index == -1 )
		return;

	try
	{
		math::global_varlist.remove( math::global_varlist.get_id(
		        gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		        gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) )
		      ) );
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
}

void menu_matrix_calc_inv_trans_adj( int which ) //_identity
{
	gint notebook_index;
	math::Variable *ptr;

	notebook_index = gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) );

	if( notebook_index == -1 )
		return;

	try
	{
		ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id(
		        gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		        gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) )
		      ) );
		if( which == 0 )
			ptr->get_matrix_rw()->inverse();
		else if( which == 1 )
			ptr->get_matrix_rw()->trans();
		else if( which == 2 )
			ptr->get_matrix_rw()->adjoint();
		else
			ptr->get_matrix_rw()->identity();
		ptr->signal_changed();
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
}

void menu_matrix_calc_inverse(void)
{
	menu_matrix_calc_inv_trans_adj( 0 );
}
void menu_matrix_calc_transpose(void)
{
	menu_matrix_calc_inv_trans_adj( 1 );
}
void menu_matrix_calc_adjoint(void)
{
	menu_matrix_calc_inv_trans_adj( 2 );
}
void menu_matrix_calc_identity(void)
{
	menu_matrix_calc_inv_trans_adj( 3 );
}

void menu_matrix_calc_determinant(void)
{
	gint notebook_index;
	Complex det;
	const char *name;

	notebook_index = gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) );

	if( notebook_index == -1 )
		return;

	try
	{
		name = gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		         gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) );
		det = math::global_varlist.get_pointer( math::global_varlist.get_id(name) )
		                                                   ->get_matrix()->det();

		try
		{
			GtkWidget *dialog;
			utf8_string ostr, tmp_str;

			ostr = "det ";
			tmp_str = name;
			ostr.append( tmp_str );
			tmp_str = " = ";
			ostr.append( tmp_str );

			det.append_to_string( ostr, matrix_editor.format );
			dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT,
			                                 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, ostr.c_str() );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
		catch( error_obj error ) { throw; }
		catch( ... ) { THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
}

void menu_matrix_calc_add_sub_mul( int which )
{
	utf8_string base, str;
	GtkWidget *dialog, *align, *vbox;
	const char *name;
	math::Variable *ptr;
	const math::Variable *ptr2;
	Vector<GtkWidget *> radios;
	int i,nr;
	bool valid;

	try{
		try{
			if( which == 0 )
				name = _("Add...");
			else if( which == 1 )
				name = _("Substract...");
			else
				name = _("Multiply...");
			dialog = gtk_dialog_new_with_buttons( name, GTK_WINDOW(matrix_editor.window),
			                                      GTK_DIALOG_MODAL, GTK_STOCK_OK,
			                                      GTK_RESPONSE_OK, NULL );

			base = gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
			         gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), 
			         gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) ) ) );
			ptr = math::global_varlist.get_pointer_rw( math::global_varlist.get_id(base.c_str()) );
			if( which == 0 )
				base.append( " += " );
			else if( which == 1 )
				base.append( " -= " );
			else
				base.append( " *= " );

			align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
			gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0);

			vbox = gtk_vbox_new( FALSE, 3 );
			gtk_container_add( GTK_CONTAINER(align), vbox );

			for( i=0,nr=0; i<gtk_notebook_get_n_pages(GTK_NOTEBOOK(matrix_editor.notebook)); i++ )
			{
				name = gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
				  gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), i ) );
				ptr2 = math::global_varlist.get_pointer( math::global_varlist.get_id(name) );
				if( which == 0  ||  which == 1 ) // add  or  substract
					valid = ( ptr->get_matrix()->get_width() == ptr2->get_matrix()->get_width()  &&
				              ptr->get_matrix()->get_height() == ptr2->get_matrix()->get_height() );
				else // multiply
					valid = ( ptr->get_matrix()->get_width() == ptr2->get_matrix()->get_height() );
				if( valid )
				{
					str = base;

					str.append( *ptr2->get_name() );

					if(nr == 0)
					{
						radios.append( gtk_radio_button_new_with_label( NULL, str.c_str() ) );
						gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(*radios[0]), TRUE );
					}
					else
						radios.append( gtk_radio_button_new_with_label_from_widget(
						                 GTK_RADIO_BUTTON(*radios[0]), str.c_str() ) );

					gtk_box_pack_start(GTK_BOX(vbox), *radios[nr], FALSE, FALSE, 0);
					nr++;
				}
			}

			if( nr == 0 )
				THROW_ERROR( ErrorType_General, _("No matching matrix found") );

			gtk_widget_show_all(dialog);

			switch(gtk_dialog_run( GTK_DIALOG(dialog) ))
			{
			case GTK_RESPONSE_OK:
				for( i=0,nr=0; i<gtk_notebook_get_n_pages(GTK_NOTEBOOK(matrix_editor.notebook)); i++ )
				{
					name = gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
					  gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), i ) );
					ptr2 = math::global_varlist.get_pointer( math::global_varlist.get_id(name) );
					if( which == 0  ||  which == 1 ) // add  or  substract
						valid = ( ptr->get_matrix()->get_width() == ptr2->get_matrix()->get_width()  &&
								  ptr->get_matrix()->get_height() == ptr2->get_matrix()->get_height() );
					else // multiply
						valid = ( ptr->get_matrix()->get_width() == ptr2->get_matrix()->get_height() );
					if( valid )
					{
						if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(*radios[nr]) ) == TRUE )
						{
							Matrix matr;

							if( which == 0) // add
								matrix_add( &matr, ptr->get_matrix(), ptr2->get_matrix() );
							else if( which == 1 ) // substract
								matrix_sub( &matr, ptr->get_matrix(), ptr2->get_matrix() );
							else // multiply
								matrix_mul( &matr, ptr->get_matrix(), ptr2->get_matrix() );
							ptr->set_matrix( &matr );
							ptr->signal_changed();
							break;
						}
						nr++;
					}
				}
				break;

			default:
				break;
			}
		}
		catch( error_obj error ) {  gtk_widget_destroy(dialog); throw error;  }
		catch( ... ) { gtk_widget_destroy(dialog);  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		gtk_widget_destroy(dialog);
	}
	catch( error_obj error )
	{
		GtkWidget *dialog;
		dialog = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
}

void menu_matrix_calc_add(void)
{
	menu_matrix_calc_add_sub_mul( 0 );
}
void menu_matrix_calc_substract(void)
{
	menu_matrix_calc_add_sub_mul( 1 );
}
void menu_matrix_calc_multiply(void)
{
	menu_matrix_calc_add_sub_mul( 2 );
}

void menu_matrix_calc_power_fill( int val )
{
	GtkWidget *align, *dialog, *entry, *hbox, *label;
	const char *str, *name;
	gint notebook_index;
	

	notebook_index = gtk_notebook_get_current_page( GTK_NOTEBOOK(matrix_editor.notebook) );
	if( notebook_index == -1 )
		return;
	
	if( val == 0 ) // power
		dialog = gtk_dialog_new_with_buttons( _("Power..."), GTK_WINDOW(matrix_editor.window),
		                                      GTK_DIALOG_MODAL,	GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );
	else // if( val == 1 ) // fill with
		dialog = gtk_dialog_new_with_buttons( _("Fill with..."), GTK_WINDOW(matrix_editor.window),
		                                      GTK_DIALOG_MODAL,	GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );

	align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
	gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0);

	hbox = gtk_hbox_new( FALSE, 3 );
	gtk_container_add( GTK_CONTAINER(align), hbox );

	{
		const char *trans;
		if( val == 0 )
			trans = _("Power: ");
		else
			trans = _("Fill with: ");
		label = gtk_label_new( trans );
	}
	gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, TRUE, 0 );

	entry = gtk_entry_new();
	gtk_box_pack_start( GTK_BOX(hbox), entry, TRUE, TRUE, 0 );

	gtk_widget_show_all( dialog );

	if( gtk_dialog_run( GTK_DIALOG(dialog) ) == GTK_RESPONSE_OK )
		str = gtk_entry_get_text( GTK_ENTRY(entry) );
	else
	{
		gtk_widget_destroy( dialog );
		return;
	}

	try
	{
		math::CodeLine code;
		math::Variable *var, ans;
		Matrix matr_ans;

		code.set_code_line( str );
		
		code.calc( &ans );
		
		name = gtk_notebook_get_tab_label_text( GTK_NOTEBOOK(matrix_editor.notebook),
		         gtk_notebook_get_nth_page( GTK_NOTEBOOK(matrix_editor.notebook), notebook_index ) );
		var = math::global_varlist.get_pointer_rw( math::global_varlist.get_id(name) );
		
		if( val == 0 )
		{
			Integer power;
			
			if( ans.get_complex()->imaginary != Real(0.0) )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Imaginary part must be zero.") );
			if( !ans.get_complex()->real.isInteger() )
				THROW_ERROR( ErrorType_Domain, _("Domain error: Can't raise a matrix to a non integer number.") );
			ans.get_complex()->real.get_top( &power );
			matrix_pow( &matr_ans, var->get_matrix(), power );
			var->set_matrix( &matr_ans );
		}
		else //if( val == 1 )
		{
			var->get_matrix_rw()->fill( *ans.get_complex() );
			var->signal_changed();
		}
	}
	catch( error_obj error )
	{
		GtkWidget *dialog2;
		dialog2 = gtk_message_dialog_new( GTK_WINDOW(matrix_editor.window),
										  GTK_DIALOG_DESTROY_WITH_PARENT,
										  GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg );
		gtk_dialog_run( GTK_DIALOG(dialog2) );
		gtk_widget_destroy( dialog2 );
	}
	gtk_widget_destroy( dialog );
}

void menu_matrix_calc_power(void)
{
	menu_matrix_calc_power_fill( 0 );
}
void menu_matrix_calc_fill_with(void)
{
	menu_matrix_calc_power_fill( 1 );
}
