
/*
 *  picture-object...
 *  Copyright (c) 2002-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */


#ifndef PICTURE_H_
#define PICTURE_H_

#include "vartypes.h"

#define PICTURE_H_VERSION "v16"
#define PICTURE_H_DATE    "2002-12 - 2006-07"

namespace picture_h
{

enum PictureType { PictureType_RGB, PictureType_GRAYSCALE };


struct colorrgb
{
	uint8 red,green,blue;

	bool operator==(const colorrgb &color) const;
};

namespace color
{
	extern const colorrgb RED;
	extern const colorrgb GREEN;
	extern const colorrgb BLUE;
	extern const colorrgb DARK_RED;
	extern const colorrgb DARK_GREEN;
	extern const colorrgb DARK_BLUE;
	extern const colorrgb YELLOW;
	extern const colorrgb WHITE;
	extern const colorrgb LIGHT_GRAY;
	extern const colorrgb DARK_GRAY;
	extern const colorrgb BLACK;
	extern const colorrgb PURPLE;
	extern const colorrgb PINK;
	extern const colorrgb ORANGE;
}

class picture
{
protected:
	int32 width,height;
	PictureType type;
	bool isBackground;

	struct
	{
		uint8 *buffer;
		int32 buffersize;
	}data[3],solidness;

public:

	picture();
	picture(const picture &src) throw(error_obj);
	virtual ~picture();

	void set_isBackground(bool value) throw(error_obj);
	bool get_isBackground(void) const { return isBackground; }


	void set_size(int32 newwidth,int32 newheight) throw(error_obj);
	void erase(void);

	inline int32 get_height(void) const { return height; }
	inline int32 get_width(void) const { return width; }
	inline PictureType get_type(void) const { return type; }

	colorrgb get_pixel(int32 x,int32 y) const;
	uint8 get_solidness(int32 x,int32 y) const;

	void convto_rgb(void) throw(error_obj);
	void convto_grayscale(void) throw(error_obj);

	void set_solidness(int32 x,int32 y,uint8 newsolidness) throw(error_obj);
	void set_solidness_color(const colorrgb &color,uint8 newsolidness) throw(error_obj);
	void set_solidness_all(uint8 newsolidness) throw(error_obj);
	void set_pixel(int32 x,int32 y,const colorrgb &color = color::WHITE,
	               uint8 solid = 255) throw(error_obj);
	void set_pixel_grayscale(int32 x,int32 y,uint8 grey = 255,uint8 solid = 255) throw(error_obj);
	void clear(const colorrgb &color = color::BLACK) throw(error_obj);

	void draw_lines( const float *vertex, const uint32 *lines, uint32 nroflines,
	                 const colorrgb &color = color::WHITE,
	                 bool antialiased = false, uint8 solid = 255 ) throw(error_obj);
	
	void line(int32 x1,int32 y1,int32 x2,int32 y2,const colorrgb &color = color::WHITE,
	          uint8 solid = 255) throw(error_obj);

	void line_antialiasing(float x1,float y1,float x2,float y2,const colorrgb &color = color::WHITE,
	                       uint8 solid = 255) throw(error_obj);

	void box(int32 x,int32 y,int32 width,int32 height,const colorrgb &color = color::WHITE,
	         uint8 solid = 255) throw(error_obj);

	void filled_box(int32 x,int32 y,int32 width,int32 height,const colorrgb &color = color::WHITE,
	                uint8 solid = 255) throw(error_obj);

	void circle( int32 x, int32 y, uint32 r, const colorrgb &color = color::WHITE,
	             bool antialiased = false ) throw(error_obj);
	void filled_circle( int32 x, int32 y, uint32 r, const colorrgb &color = color::WHITE,
	                    bool antialiased = false ) throw(error_obj);

	void ellipse(int32 x, int32 y, uint32 rx, uint32 ry, const colorrgb &color = color::WHITE,
	             bool antialiased = false) throw(error_obj);

	void draw(int32 x,int32 y) const throw(error_obj);
	void draw_shadow(int32 x,int32 y,const colorrgb &color = color::BLACK) const throw(error_obj);

	void mirror(void) throw(error_obj);
	void upside_down(void) throw(error_obj);
	void rotate90(void) throw(error_obj);
	void rotate180(void) throw(error_obj);
	void rotate270(void) throw(error_obj);

	void capture(const picture &source,int32 x,int32 y,int32 width,int32 height) throw(error_obj);

	void load_pcx(const char *filename) throw(error_obj);

	void save_pcx(const char *filename) const throw(error_obj);

	void save_ppm(const char *filename, bool save_as_ascii = false) const throw(error_obj);

	void operator=(const picture &source) throw(error_obj);

	bool operator==(const picture &pic) const;
};

extern picture *dest; // this is a pointer to the picture that all pictures will be drawn to

} // end of namespace picture_h

#endif
