
/*
 *  picture2gtk - makes pictures viewable using GTK
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See picture2gtk.h
 */


/*
v7  2006-08-11
--

	Small changes to make the file compilable with gcc 4.1.x

v6  2006-04-22
--

	Improved the constructors

v1.0.4  2004-11-13 - 2004-12-19
------

	Event-function-pointers are public
	Fixed some compiler warnings

v1.0.3  2004-07-14  &  2004-08-14 - 2004-09-12
------

	Removed all appearance of Thrower
	Added Copy-constructor, (width,height,color)-constructor
	Renamed a lot of functions
	Added gdkcolor_to_colorrgb, colorrgb_to_gdkcolor
	Changed so that the button is sent also to mousedown and mouseup
	Added force_expose, force_expose_line, force_expose_col

v1.0.2  2004-06-15
------

	Fixed a few bugs

v1.0.1  2004-04-20
------

	Added events: mousemove, mousedown, mouseup

v1.0.0  2003-12-10 - 2003-12-18  &  2004-03-20 - 2004-03-25
------

	Basic functions works.
*/

#include "picture2gtk.h"
#include "keyfile.h"

#include <stdio.h>
#include <string.h>


gboolean picture2gtk_expose(GtkWidget *widget,GdkEventExpose *event,gpointer user_data)
{
	picture2gtk *pic;

	pic = picture2gtk_ptr(user_data);

	gdk_draw_rgb_image( widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
	                    0, 0, pic->raw.width, pic->raw.height,
	                    GDK_RGB_DITHER_MAX, pic->raw.buffer, pic->raw.width*3);

	return TRUE;
}
gboolean picture2gtk_destroy(GtkWidget *widget,gpointer user_data)
{
	picture2gtk *pic;

	pic = picture2gtk_ptr(user_data);

	pic->raw.drawarea = 0;

	return TRUE;
}
gboolean picture2gtk_mousemove(GtkWidget *widget,GdkEventMotion *event,gpointer user_data)
{
	picture2gtk *pic;

	pic = picture2gtk_ptr(user_data);

	if(pic->mousemove_func != 0)
	{
		(*pic->mousemove_func)(pic,int32(event->x),int32(event->y));
		return TRUE;
	}
	else
		return FALSE;
}
gboolean picture2gtk_mousedown(GtkWidget *widget,GdkEventButton *event,gpointer user_data)
{
	picture2gtk *pic;

	pic = picture2gtk_ptr(user_data);

	if(pic->mousedown_func != 0)
	{
		(*pic->mousedown_func)( pic, int32(event->x), int32(event->y), int32(event->button) );
		return TRUE;
	}
	else
		return FALSE;
}
gboolean picture2gtk_mouseup(GtkWidget *widget,GdkEventButton *event,gpointer user_data)
{
	picture2gtk *pic;

	pic = picture2gtk_ptr(user_data);

	if(pic->mouseup_func != 0)
	{
		(*pic->mouseup_func)( pic, int32(event->x), int32(event->y), int32(event->button) );
		return TRUE;
	}
	else
		return FALSE;
}





void gdkcolor_to_colorrgb( const GdkColor &src, picture_h::colorrgb &dest )
{
	dest.red = uint8(src.red >> 8);
	dest.green = uint8(src.green >> 8);
	dest.blue = uint8(src.blue >> 8);
}
void colorrgb_to_gdkcolor( const picture_h::colorrgb &src, GdkColor &dest )
{
	dest.red = (guint16(src.red) << 8) + 128;
	dest.green = (guint16(src.green) << 8) + 128;
	dest.blue = (guint16(src.blue) << 8) + 128;
}



picture2gtk::picture2gtk() : picture()
{
	raw.width = 0;
	raw.height = 0;
	raw.buffer = 0;
	raw.buffersize = 0;
	raw.drawarea = 0;
	mousemove_func = 0;
	mousedown_func = 0;
	mouseup_func = 0;
}
picture2gtk::picture2gtk(const picture2gtk &src) throw(error_obj) : picture() 
{
	raw.width = 0;
	raw.height = 0;
	raw.buffer = 0;
	raw.buffersize = 0;
	raw.drawarea = 0;
	mousemove_func = 0;
	mousedown_func = 0;
	mouseup_func = 0;

	*this = src;
}
picture2gtk::picture2gtk(int32 newwidth, int32 newheight, const picture_h::colorrgb &newcolor) throw(error_obj)
{
	*this = picture2gtk();
	set_size(newwidth,newheight);
	clear(newcolor);
}

picture2gtk::~picture2gtk()
{
	delete [] raw.buffer;
	disconnect_from_drawing_area();
}

void picture2gtk::set_raw_size(int32 newwidth,int32 newheight) throw(error_obj)
{
	int32 newsize = newwidth * newheight * 3;

	if(newsize > raw.buffersize)
	{
		delete [] raw.buffer;
		raw.buffer = 0;
		raw.buffersize = 0;
		raw.width = 0;
		raw.height = 0;

		try{ raw.buffer = new guchar[newsize]; }
		catch(...){ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }

		raw.buffersize = newsize;
	}

	raw.width = newwidth;
	raw.height = newheight;

	if(raw.drawarea != 0)
		gtk_widget_set_size_request(raw.drawarea,raw.width,raw.height);
}

void picture2gtk::set_size(int32 newwidth,int32 newheight) throw(error_obj)
{
	this->picture_h::picture::set_size(newwidth,newheight);

	try{ set_raw_size(newwidth,newheight); }
	catch(...)
	{
		this->erase();
		throw;
	}
}

void picture2gtk::update(void) throw(error_obj)
{
	int32 x,y;
	picture_h::colorrgb color;
	guchar *tbuffer;

	set_raw_size(width,height); // this makes sure that raw.buffer has a correct size

	tbuffer = raw.buffer;

	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			color = get_pixel(x,y);

			*tbuffer++ = guchar(color.red);
			*tbuffer++ = guchar(color.green);
			*tbuffer++ = guchar(color.blue);
		}
	}

	if(raw.drawarea != 0)
		gtk_widget_queue_draw_area(raw.drawarea,0,0,raw.width,raw.height);
}

void picture2gtk::update_from(const picture_h::picture &src) throw(error_obj)
{
	int32 x,y;
	picture_h::colorrgb color;
	guchar *tbuffer;

	set_raw_size(src.get_width(),src.get_height());

	tbuffer = raw.buffer;

	for(y=0;y<raw.height;y++)
	{
		for(x=0;x<raw.width;x++)
		{
			color = src.get_pixel(x,y);

			*tbuffer++ = guchar(color.red);
			*tbuffer++ = guchar(color.green);
			*tbuffer++ = guchar(color.blue);
		}
	}

	if(raw.drawarea != 0)
		gtk_widget_queue_draw_area(raw.drawarea,0,0,raw.width,raw.height);
}

void picture2gtk::operator=(const picture2gtk &src) throw(error_obj)
{
	set_size(src.width,src.height);

	this->picture_h::picture::operator=(src);

	mousemove_func = src.mousemove_func;
	mouseup_func = src.mouseup_func;
	mousedown_func = src.mousedown_func;
}

void picture2gtk::operator=(const picture_h::picture &src) throw(error_obj)
{
	set_size(src.get_width(),src.get_height());

	this->picture_h::picture::operator=(src);
}

void picture2gtk::connect_to_drawing_area(GtkWidget *newdrawarea)
{
	disconnect_from_drawing_area();

	if(newdrawarea != 0)
	{
		raw.drawarea = newdrawarea;

		gtk_widget_add_events(raw.drawarea,GDK_POINTER_MOTION_MASK);
		gtk_widget_add_events(raw.drawarea,GDK_BUTTON_PRESS_MASK);
		gtk_widget_add_events(raw.drawarea,GDK_BUTTON_RELEASE_MASK);

		g_signal_connect( GTK_OBJECT(raw.drawarea), "expose-event",
		                  G_CALLBACK(picture2gtk_expose), gpointer(this) );

		g_signal_connect( GTK_OBJECT(raw.drawarea), "destroy",
		                  G_CALLBACK(picture2gtk_destroy), gpointer(this) );

		g_signal_connect( GTK_OBJECT(raw.drawarea), "motion-notify-event",
		                  G_CALLBACK(picture2gtk_mousemove), gpointer(this) );

		g_signal_connect( GTK_OBJECT(raw.drawarea), "button-press-event",
		                  G_CALLBACK(picture2gtk_mousedown), gpointer(this) );

		g_signal_connect( GTK_OBJECT(raw.drawarea), "button-release-event",
		                  G_CALLBACK(picture2gtk_mouseup), gpointer(this) );

		gtk_widget_set_size_request(GTK_WIDGET(raw.drawarea),raw.width,raw.height);
	}
}

void picture2gtk::disconnect_from_drawing_area(void)
{
	if(raw.drawarea != 0)
	{
		g_signal_handlers_disconnect_by_func( GTK_OBJECT(raw.drawarea),
		                                      gpointer(picture2gtk_expose),
		                                      gpointer(this) );

		g_signal_handlers_disconnect_by_func( GTK_OBJECT(raw.drawarea),
		                                      gpointer(picture2gtk_destroy),
		                                      gpointer(this) );

		g_signal_handlers_disconnect_by_func( GTK_OBJECT(raw.drawarea),
		                                      gpointer(picture2gtk_mousemove),
		                                      gpointer(this) );

		g_signal_handlers_disconnect_by_func( GTK_OBJECT(raw.drawarea),
		                                      gpointer(picture2gtk_mousedown),
		                                      gpointer(this) );

		g_signal_handlers_disconnect_by_func( GTK_OBJECT(raw.drawarea),
		                                      gpointer(picture2gtk_mouseup),
		                                      gpointer(this) );

		raw.drawarea = 0;
	}
}


void picture2gtk::force_expose(void)
{
	picture2gtk_expose( raw.drawarea, NULL, gpointer (this) );
}
void picture2gtk::force_expose_line(int32 line)
{
	if( line < 0  ||  line >= raw.height )
		return;

	gdk_draw_rgb_image( raw.drawarea->window, raw.drawarea->style->fg_gc[GTK_STATE_NORMAL],
	                    0, line, raw.width, 1, GDK_RGB_DITHER_MAX, &raw.buffer[line*raw.width*3],
	                    raw.width*3);
}
void picture2gtk::force_expose_col(int32 col)
{
	if( col < 0  ||  col >= raw.width )
		return;

	gdk_draw_rgb_image( raw.drawarea->window, raw.drawarea->style->fg_gc[GTK_STATE_NORMAL],
	                    col, 0, 1, raw.height, GDK_RGB_DITHER_MAX, &raw.buffer[col*3],
	                    raw.width*3);
}
