
/*
 *  Ump - Unnamed Math Program
 *  Copyright (c) 2005-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */
 
/*
	This is based on $XFree86: xc/programs/glxgears/glxgears.c,v 1.3tsi Exp $
*/
 
#ifdef PLOT_3D

#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>

#include "ump_3d_viewer.h"
#include "main.h"
#include "vector.h"
#include "utf8_string.h"
#include "keyfile.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>
#undef Complex


void addon_3d_clear(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                    throw(error_obj);
void addon_3d_render(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                     throw(error_obj);
void addon_3d_set_prop(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                       throw(error_obj);
void addon_3d_get_prop(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                       throw(error_obj);
void addon_3d_add_plane(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                        throw(error_obj);

struct _3d_vertex
{
	float x,y,z;
};

struct _3d_point
{
	float x,y,z;
	picture_h::colorrgb color;
	bool isValid;
};

enum _3d_plane_type { _3d_plane_wireframed, _3d_plane_solid, _3d_plane_semi_solid };
class _3d_plane
{
public:
	uint32 width, height;
	_3d_point *points;
	_3d_plane_type type;
	GLint draw_list;

	_3d_plane() { width = 0; height = 0; points = 0; type = _3d_plane_solid; }
	_3d_plane( const _3d_plane &src ) throw(error_obj) { width=0; height=0; points=0; type=_3d_plane_solid; *this=src; }
	~_3d_plane() { clear(); }
	void operator=(const _3d_plane &src) throw(error_obj)
		{
			set_size( src.width, src.height );
			for( uint32 i=0; i<width*height; i++ )
				points[i] = src.points[i];
			type = src.type;
		}
	void clear(void) { delete [] points; width = 0; height = 0; points = 0; }
	void set_size(int new_width, int new_height) throw(error_obj)
		{
			clear();
			
			if( new_width <= 0  ||  new_height <= 0 )
				return;
			
			try{ points = new _3d_point[new_width*new_height]; }
			catch(...) { THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
			width = new_width;
			height = new_height;
		}
};


struct _3d_Properties
{
	int width, height;
	_3d_vertex look_at;
	float camera_dist, camera_angle1, camera_angle2;
	picture_h::colorrgb bg_color;
	bool only_one_viewer;

	
	Vector<_3d_plane> planes;
}_3d_prop;


void render_plane( const _3d_plane *plane )
{
	bool prev_line_full, line_full;

	if( plane->type == _3d_plane_solid )
		glPolygonMode( GL_BACK, GL_FILL );
	else
		glPolygonMode( GL_BACK, GL_LINE );
	
	if( plane->type == _3d_plane_wireframed )
		glPolygonMode( GL_FRONT, GL_LINE );
	else
		glPolygonMode( GL_FRONT, GL_FILL );

	line_full = true;
	for( uint32 x=0; x<plane->width; x++ )
		line_full &= plane->points[x].isValid;

	for( uint32 y=0; y<(plane->height-1); y++ )
	{
		prev_line_full = line_full;

		line_full = true;
		for( uint32 x=0,tmp=(y+1)*plane->width; x<plane->width; x++ )
			line_full &= plane->points[x + tmp].isValid;

		if( prev_line_full  &&  line_full )
		{
			glBegin( GL_QUAD_STRIP );
			for( uint32 x=0; x<plane->width; x++ )
			{
				glColor3f( plane->points[x + (y+1)*plane->width].color.red/255.0f,
					   plane->points[x + (y+1)*plane->width].color.green/255.0f,
					   plane->points[x + (y+1)*plane->width].color.blue/255.0f );
				glVertex3f( plane->points[x + (y+1)*plane->width].x,
					    plane->points[x + (y+1)*plane->width].y,
					    plane->points[x + (y+1)*plane->width].z );
				glColor3f( plane->points[x + y*plane->width].color.red/255.0f,
					   plane->points[x + y*plane->width].color.green/255.0f,
					   plane->points[x + y*plane->width].color.blue/255.0f );
				glVertex3f( plane->points[x + y*plane->width].x,
					    plane->points[x + y*plane->width].y,
					    plane->points[x + y*plane->width].z );
			}
			glEnd();
		}
		else
		{
			glBegin( GL_QUADS );
			for( uint32 x=0; x<(plane->width-1); x++ )
			{
				if( plane->points[x + y*plane->width].isValid  &&
				    plane->points[x+1 + y*plane->width].isValid  &&
				    plane->points[x+1 + (y+1)*plane->width].isValid  &&
				    plane->points[x + (y+1)*plane->width].isValid )
				{
					glColor3f( plane->points[x + y*plane->width].color.red/255.0f,
						   plane->points[x + y*plane->width].color.green/255.0f,
						   plane->points[x + y*plane->width].color.blue/255.0f );
					glVertex3f( plane->points[x + y*plane->width].x,
						    plane->points[x + y*plane->width].y,
						    plane->points[x + y*plane->width].z );
					glColor3f( plane->points[x+1 + y*plane->width].color.red/255.0f,
						   plane->points[x+1 + y*plane->width].color.green/255.0f,
						   plane->points[x+1 + y*plane->width].color.blue/255.0f );
					glVertex3f( plane->points[x+1 + y*plane->width].x,
						    plane->points[x+1 + y*plane->width].y,
						    plane->points[x+1 + y*plane->width].z );
					glColor3f( plane->points[x+1 + (y+1)*plane->width].color.red/255.0f,
						   plane->points[x+1 + (y+1)*plane->width].color.green/255.0f,
						   plane->points[x+1 + (y+1)*plane->width].color.blue/255.0f );
					glVertex3f( plane->points[x+1 + (y+1)*plane->width].x,
						    plane->points[x+1 + (y+1)*plane->width].y,
						    plane->points[x+1 + (y+1)*plane->width].z );
					glColor3f( plane->points[x + (y+1)*plane->width].color.red/255.0f,
						   plane->points[x + (y+1)*plane->width].color.green/255.0f,
						   plane->points[x + (y+1)*plane->width].color.blue/255.0f );
					glVertex3f( plane->points[x + (y+1)*plane->width].x,
						    plane->points[x + (y+1)*plane->width].y,
						    plane->points[x + (y+1)*plane->width].z );
				}
			}
			glEnd();
		}
	}
}

void render_scene( const _3d_Properties &render_prop )
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	
	for( int i=0; i<render_prop.planes.get_size(); i++ )
	{
		// draw the plane from list if it's avaliable
		if( render_prop.planes[i]->draw_list != 0 )
			glCallList( render_prop.planes[i]->draw_list );
		else
			render_plane( render_prop.planes[i] );
	}

	glPopMatrix();
	glFlush();
}

void make_window( const _3d_Properties &render_prop, Display *dpy, Window *winRet,
                  GLXContext *ctxRet) throw(error_obj)
{
	int attrib[] = { GLX_RGBA,
	                 GLX_RED_SIZE, 1,
	                 GLX_GREEN_SIZE, 1,
	                 GLX_BLUE_SIZE, 1,
	                 GLX_DOUBLEBUFFER,
	                 GLX_DEPTH_SIZE, 1,
	                 None };
	int scrnum;
	XSetWindowAttributes attr;
	unsigned long mask;
	Window root;
	Window win;
	GLXContext ctx;
	XVisualInfo *visinfo;
	GLint max[2] = { 0, 0 };

	scrnum = XScreenNumberOfScreen( XDefaultScreenOfDisplay(dpy) );
	root   = XRootWindow(dpy, scrnum);

	visinfo = glXChooseVisual( dpy, scrnum, attrib );
	if( !visinfo )
		THROW_ERROR( ErrorType_General, _("Couldn't get a RGB, Double-buffered visual.") );

	// window attributes
	attr.background_pixel = 0;
	attr.border_pixel = 0;
	attr.colormap = XCreateColormap( dpy, root, visinfo->visual, AllocNone );
	attr.event_mask = StructureNotifyMask | ExposureMask | KeyPressMask;
	mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

	win = XCreateWindow( dpy, root, 0, 0, render_prop.width, render_prop.height,
	                     0, visinfo->depth, InputOutput,
	                     visinfo->visual, mask, &attr );

   // set hints and properties
	{
		XSizeHints sizehints;
		sizehints.width  = render_prop.width;
		sizehints.height = render_prop.height;
		sizehints.flags = USSize;
		XSetNormalHints( dpy, win, &sizehints );
		XSetStandardProperties( dpy, win, PRG_NAME, PRG_NAME, None, (char **)NULL, 0, &sizehints);
	}

	ctx = glXCreateContext( dpy, visinfo, NULL, True );
	if( !ctx )
		THROW_ERROR( ErrorType_General, _("glXCreateContext failed") );

	XFree(visinfo);

	XMapWindow(dpy, win);
	glXMakeCurrent(dpy, win, ctx);

	// Check for maximum size supported by the GL rasterizer
	glGetIntegerv(GL_MAX_VIEWPORT_DIMS, max);
	if( render_prop.width > max[0]  ||  render_prop.height > max[1] )
		THROW_ERROR( ErrorType_General, _("Requested window size is larger than maximum supported by GL engine.") );

	*winRet = win;
	*ctxRet = ctx;
}

void update_camera( const _3d_Properties &render_prop )
{
	glLoadIdentity();
	
	gluLookAt( render_prop.camera_dist + render_prop.look_at.x, render_prop.look_at.y, render_prop.look_at.z,
		       render_prop.look_at.x, render_prop.look_at.y, render_prop.look_at.z,
			   0.0f, 1.0f, 0.0f );

	glTranslatef( render_prop.look_at.x, render_prop.look_at.y, render_prop.look_at.z );
	glRotatef( render_prop.camera_angle1, 0.0, 1.0, 0.0);
	glRotatef( render_prop.camera_angle2, 1.0, 0.0, 0.0);
	glTranslatef( -render_prop.look_at.x, -render_prop.look_at.y, -render_prop.look_at.z );
}

void size_changed( const _3d_Properties &render_prop, int w, int h )
{
	if(h == 0)
		h = 1;

	float ratio = float(w) / float(h);

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);

	// Set the correct perspective.
	gluPerspective( 45, ratio, 1, 1000 );
	glMatrixMode(GL_MODELVIEW);
	
	update_camera( render_prop );
}

void event_loop( _3d_Properties &render_prop, Display *dpy, Window win )
{
	XEvent event;
	char buffer[10];
	int code;

	while(true)
	{
		XNextEvent( dpy, &event );
		switch( event.type )
		{
		case Expose:
			render_scene( render_prop );
			glXSwapBuffers( dpy, win );
			break;
		case ConfigureNotify:
		   size_changed( render_prop, event.xconfigure.width, event.xconfigure.height );
		   break;
		case KeyPress:
			code = XLookupKeysym(&event.xkey, 0);
			if( code == XK_Left )
			{
				render_prop.camera_angle1 += 5.0f;
				if( render_prop.camera_angle1 >= 360.0f )
					render_prop.camera_angle1 -= 360.0f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Right )
			{
				render_prop.camera_angle1 -= 5.0f;
				if( render_prop.camera_angle1 <= 0.0f )
					render_prop.camera_angle1 += 360.0f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Page_Up )
			{
				render_prop.camera_dist *= 0.95f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Page_Down )
			{
				render_prop.camera_dist *= 1.05f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Down )
			{
				render_prop.camera_angle2 += 5.0f;
				if( render_prop.camera_angle2 >= 360.0f )
					render_prop.camera_angle2 -= 360.0f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Up )
			{
				render_prop.camera_angle2 -= 5.0f;
				if( render_prop.camera_angle2 <= 0.0f )
					render_prop.camera_angle2 += 360.0f;
				update_camera( render_prop );
				render_scene( render_prop );
				glXSwapBuffers( dpy, win );
			}
			else if( code == XK_Escape )
				return;
			else
			{
				XLookupString( &event.xkey, buffer, sizeof(buffer), NULL, NULL );
				switch( buffer[0] )
				{
				case 'Q': case 'q': case 'X': case 'x':
					return;
				}
			}
			break;
		}
	}
}

void _3d_viewer_child( _3d_Properties render_prop )
{
	try
	{
		Display *disp;
		Window win;
		GLXContext glx_cont;

		//setup a glx-window
		disp = XOpenDisplay(NULL);
		if( !disp )
			THROW_ERROR( ErrorType_General, _("Couldn't open display") );

		make_window( render_prop, disp, &win, &glx_cont );
		size_changed( render_prop, render_prop.width, render_prop.height );

		glClearColor( render_prop.bg_color.red/255.0f, render_prop.bg_color.green/255.0f,
		              render_prop.bg_color.blue/255.0f, 1.0f );
		glClearDepth( 1.0f );
		glDepthFunc( GL_LESS );
		glEnable( GL_DEPTH_TEST );
		glShadeModel( GL_SMOOTH );



		// creating draw lists
		for( int i=0; i<render_prop.planes.get_size(); i++ )
		{
			if( (render_prop.planes[i]->draw_list = glGenLists(1)) != 0 )
			{
				glNewList( render_prop.planes[i]->draw_list, GL_COMPILE );
				render_plane( render_prop.planes[i] );
				glEndList();
			}
		}

		// enter viewing/interaction mode
		event_loop( render_prop, disp, win );
		
		// cleaning up
		glXDestroyContext( disp, glx_cont );
		XDestroyWindow( disp, win );
		XCloseDisplay( disp );
	}
	catch( error_obj error )
	{
		printf("Error: _3d_viewer: %s\n", error.msg );
	}
	exit(0); // terminating the child process
}

void _3d_viewer_init(void) throw(error_obj)
{
	math::add_function( "_3d_clear", addon_3d_clear, false, false );
	math::add_function( "_3d_render", addon_3d_render, false, false );
	math::add_function( "_3d_set_prop", addon_3d_set_prop, false, true );
	math::add_function( "_3d_get_prop", addon_3d_get_prop, false, true );
	math::add_function( "_3d_add_plane", addon_3d_add_plane, false, true );
	
	_3d_prop.width = 400;
	_3d_prop.height = 400;
	_3d_prop.look_at.x = 0.0f;
	_3d_prop.look_at.y = 0.0f;
	_3d_prop.look_at.z = -0.0f;
	
	_3d_prop.camera_dist = 10.0f;
	_3d_prop.camera_angle1 = 0.0f;
	_3d_prop.camera_angle2 = 0.0f;
	
	_3d_prop.bg_color.red = 255;
	_3d_prop.bg_color.green = 255;
	_3d_prop.bg_color.blue = 255;

	_3d_prop.only_one_viewer = true;	
}










///////////////////////////////////////
// Here comes the 3d addon functions //
///////////////////////////////////////

void addon_3d_clear(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                    throw(error_obj)
{
	if( left != 0  ||  right != 0 )
		THROW_ERROR( ErrorType_General, "Internal error: _3d_clear wasn't called the right way" );

	_3d_prop.planes.clear();

	if( res != 0 )
		res->set_void();
}

Vector<pid_t> pid_list;

void addon_3d_render(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                     throw(error_obj)
{
	pid_t child_pid;
	if( left != 0  ||  right != 0 )
		THROW_ERROR( ErrorType_General, "Internal error: _3d_render wasn't called the right way" );

	if( _3d_prop.only_one_viewer )
	{
// terminate all old viewers
		for( int i=0; i<pid_list.get_size(); i++ )
			kill( *pid_list[i], 15 );
	}
	
	// this loops throu all old child and removes the zombies...
	for( int i=0; i<pid_list.get_size(); i++ )
	{
		if( waitpid( *pid_list[i], 0, WNOHANG ) != 0 )
		{
			pid_list.remove( i );
			i--;
		}
	}
	
	switch( child_pid = fork() )
	{
	case -1:
		(*TEXTVIEW_FUNCTION)("\nfork failed\n");
		break;
	case 0:
		_3d_viewer_child( _3d_prop );
		break;
	default:
		pid_list.append( child_pid );
		break;
	}

	if( res != 0 )
		res->set_void();
}
void addon_3d_set_prop(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                       throw(error_obj)
{
	Matrix matr;
	Complex tmp;
	const math::Array *array;
	const utf8_string *str;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: _3d_set_prop wasn't called the right way" );

	if( right->get_type() != math::VariableType_Array )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	array = right->get_array();

	for( uint32 i=0; i<array->get_size(); i++ )
	{
		if( array->get_variable(i)->get_type() != math::VariableType_String )
		{
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
			THROW_ERROR( ErrorType_General, tmp_err );
		}

		str = array->get_variable(i)->get_string();

		if( *str == "Width" )
		{
			Integer w;

			if( array->get_variable(i+1)->get_type() != math::VariableType_Complex )
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
				THROW_ERROR( ErrorType_General, tmp_err );
			}
			tmp = *( array->get_variable(i+1)->get_complex() );

			if( tmp.imaginary != Real(0.0)  ||  !tmp.real.isInteger() )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: 'Width' must be an integer.") );

			tmp.real.get_top( &w );

			_3d_prop.width = int( int64(w) );

			i++;
		}
		else if( *str == "Height" )
		{
			Integer h;

			if( array->get_variable(i+1)->get_type() != math::VariableType_Complex )
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
				THROW_ERROR( ErrorType_General, tmp_err );
			}
			tmp = *( array->get_variable(i+1)->get_complex() );

			if( tmp.imaginary != Real(0.0)  ||  !tmp.real.isInteger() )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: 'Width' must be an integer.") );

			tmp.real.get_top( &h );

			_3d_prop.height = int( int64(h) );

			i++;
		}
		else if( *str == "Camera distance" )
		{
			if( array->get_variable(i+1)->get_type() != math::VariableType_Complex )
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
				THROW_ERROR( ErrorType_General, tmp_err );
			}
			tmp = *( array->get_variable(i+1)->get_complex() );
			if( tmp.imaginary != Real(0.0)  ||  tmp.real <= Real(0.0) )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: 'Camera distance' must be a positive real value.") );
			_3d_prop.camera_dist = float(tmp.real);

			i++;
		}
		else if( *str == "Look at" )
		{
			float x,y,z;

			if( array->get_variable(i+1)->get_type() != math::VariableType_Matrix )
			{
				char tmp_err[ERROR_OBJ_MSG_LEN];
				snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_set_prop" );
				THROW_ERROR( ErrorType_General, tmp_err );
			}

			matr = *( array->get_variable(i+1)->get_matrix() );

			if( matr.get_height() != 3  ||  matr.get_width() != 1 )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: 'Look at's vertex matrix's size dosn't match.") );

			tmp = matr.get_complex( 1, 1 );
			if( tmp.imaginary != Real(0.0) )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: 'Look at's coordinates must be real values.") );
			x = float(tmp.real);

			tmp = matr.get_complex( 1, 2 );
			if( tmp.imaginary != Real(0.0) )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: \"Look at\"s coordinates must be real values.") );
			y = float(tmp.real);

			tmp = matr.get_complex( 1, 3 );
			if( tmp.imaginary != Real(0.0) )
				THROW_ERROR( ErrorType_General, _("_3d_set_prop: \"Look at\"s coordinates must be real values.") );
			z = float(tmp.real);

			_3d_prop.look_at.x = x;
			_3d_prop.look_at.y = z;
			_3d_prop.look_at.z = -y;

			i++;
		}
		else
		{
			utf8_string str2( _("_3d_set_prop: Unknown property: ") );
			str2.append( *str );
			THROW_ERROR( ErrorType_General, str2.c_str() );
		}
	}

	if( res != 0 )
		res->set_void();
}

void addon_3d_get_prop(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                       throw(error_obj)
{
	Matrix matr_res;
	Complex tmp;
	const utf8_string *str;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: _3d_get_prop wasn't called the right way" );

	if( res == 0 )
		return;

	if( right->get_type() != math::VariableType_String )
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Data type is unsupported by '%s'"), "_3d_get_prop" );
		THROW_ERROR( ErrorType_General, tmp_err );
	}

	str = right->get_string();

	if( *str == "Width" )
	{
		tmp.imaginary = Real(0.0);
		tmp.real = Real(_3d_prop.width);
		res->set_complex( tmp );
	}
	else if( *str == "Height" )
	{
		tmp.imaginary = Real(0.0);
		tmp.real = Real(_3d_prop.height);
		res->set_complex( tmp );
	}
	else if( *str == "Camera distance" )
	{
		tmp.imaginary = Real(0.0);
		tmp.real = Real(_3d_prop.camera_dist);
		res->set_complex( tmp );
	}
	else if( *str == "Look at" )
	{
		tmp.imaginary = Real(0.0);
		matr_res.set_size( 1, 3 );
		tmp.real = Real(_3d_prop.look_at.x);
		matr_res.set_complex( 1, 1, tmp );
		tmp.real = Real(_3d_prop.look_at.y);
		matr_res.set_complex( 1, 3, tmp );
		tmp.real = Real(-_3d_prop.look_at.z);
		matr_res.set_complex( 1, 2, tmp );

		res->set_matrix( &matr_res );
	}
	else
	{
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Unknown 3d property: %s"), str->c_str() );
		THROW_ERROR( ErrorType_General, tmp_err );
	}
}

void addon_3d_add_plane(math::Variable *res, math::Variable *left, const math::Variable *right, math::VariableList *private_varlist)
                        throw(error_obj)
{
	Matrix x_matr, y_matr, z_matr;
	floatx min=0.0, max=1.0;
	Vector<picture_h::colorrgb> colors;
	_3d_plane *plane;
	_3d_point *point;

	if( left != 0  ||  right == 0 )
		THROW_ERROR( ErrorType_General, "Internal error: _3d_ad_plane wasn't called the right way" );


	try{  plane = new _3d_plane;  }
	catch(...) {  THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") );  }

	try
	{
		Matrix colormatrix;
		math::colorrgb_to_colormatrix( picture_h::color::BLUE, &colormatrix);
		utf8_string str("Solid");

		argument_check( "_3d_add_plane", right,
		                math::type_matrix, &x_matr,
		                math::type_matrix, &y_matr,
		                math::type_matrix, &z_matr,
		                math::type_string_default, &str,
		                math::type_floatx_default, &min,
		                math::type_floatx_default, &max,
		                math::type_matrix_default, &colormatrix,
		                math::type_end );

		if( str == "Solid" )
			plane->type = _3d_plane_solid;
		else if( str == "Semi-solid" )
			plane->type = _3d_plane_semi_solid;
		else if( str == "Wireframed" )
			plane->type = _3d_plane_wireframed;
		else
			THROW_ERROR( ErrorType_General, _("Unknown 3D plane-type") );
		
		math::colormatrix_to_colorrgbs( colormatrix, colors );

		if( min >= max )
			THROW_ERROR( ErrorType_General, _("_3d_add_plane: Max must be larger than min.") );

		if( x_matr.get_width() <= 1  || x_matr.get_height() < 1 ||
		    y_matr.get_width() <= 1  || y_matr.get_height() < 1 ||
		    z_matr.get_width() <= 1  || z_matr.get_height() <= 1 )
		{
			if( res != 0 )
				res->set_void();
			return;
		}

		if( x_matr.get_width() != z_matr.get_width()  ||
		    y_matr.get_width() != z_matr.get_height() )
			THROW_ERROR( ErrorType_General, _("_3d_add_plane: X, Y & Z's matrix sizes doesn't match.") );

		plane->set_size( z_matr.get_width(), z_matr.get_height() );

		for( uint32 x=0; x<plane->width; x++ )
		{
			for( uint32 y=0; y<plane->height; y++ )
			{
				point = &(plane->points[x + y*plane->width]);

				if( x_matr.get_complex( x+1, 1 ).imaginary != Real(0.0)  ||
				    y_matr.get_complex( y+1, 1 ).imaginary != Real(0.0)  ||
				    z_matr.get_complex( x+1, y+1 ).imaginary != Real(0.0) )
					point->isValid = false;
				else
				{
					point->isValid = true;

					// rearranging the axis...
					point->x = float(  x_matr.get_complex( x+1, 1 ).real );
					point->y = float(  z_matr.get_complex( x+1, y+1 ).real );
					point->z = float( -y_matr.get_complex( y+1, 1 ).real );

					math::color_list_get_color( colors, min, max,
					                            z_matr.get_complex(x+1,y+1).real, &(point->color) );
				}
			}
		}
		_3d_prop.planes.append_this( plane );
	}
	catch(...)
	{
		delete plane;
		throw;
	}

	if( res != 0 )
		res->set_void();
}

#else
	void _3d_viewer_init(void) {} // dummy function when opengl support shouldn't get compiled
#endif // PLOT_3D
