
/*
 *  Ump - Unnamed Math Program
 *  Copyright (c) 2004-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */

#include "ump_advancedwin.h"
#include "ump_aboutwin.h"
#include "ump_simplewin.h"
#include "ump_editor.h"
#include "ump_graphwin.h"
#include "ump_prefwin.h"
#include "matrix_editor.h"
#include "main.h"
#include "math2.h"
#include "utf8_string.h"
#include "keyfile.h"

Vector<utf8_string> advanced_history;

GtkWidget *advancedwin = 0, *advancedwin_menu, *advanced_vbox, *advanced_button;


void advancedwin_close(void);
void advanced_calculate(void);
void advanced_file_load(void);
void advanced_file_save_as(void);
void advanced_opt_radian(void);
void advanced_opt_degree(void);
void advanced_opt_format(void);
GtkWidget *advanced_combo, *advanced_textview, *advanced_degfix;
GtkTextBuffer *advanced_textbuffer;

void aboutwin_show_advanced(void) { aboutwin_show(GTK_WINDOW(advancedwin)); }
void prefwin_show_advanced(void) { prefwin_show(GTK_WINDOW(advancedwin)); }


void advanced_entry_insert(GtkEditable *editable,gchar *new_text,gint new_text_length,gint *position);



void degfix_changed(void)
{
	const Complex degree( PI/180.0 ), radian( 1.0 );

	if(advancedwin == 0)
		return;

	if( math::degfix == radian )
		gtk_label_set_text(GTK_LABEL(advanced_degfix), _("Radian") );
	else if( math::degfix == degree )
		gtk_label_set_text(GTK_LABEL(advanced_degfix), _("Degree") );
	else
		gtk_label_set_text(GTK_LABEL(advanced_degfix), PRG_NAME );
}

GtkWidget* advancedwin_build_menu(void)
{
	GtkActionGroup *actions;
	GtkUIManager *ui_manager;
	GError *error = NULL;
	static const gchar *menu =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Clear'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Load'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='PrefMenu'>"
"      <menuitem action='Radian'/>"
"      <menuitem action='Degree'/>"
"      <separator/>"
"      <menuitem action='Preferences'/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Simple'/>"
"      <menuitem action='Editor'/>"
"      <menuitem action='Graphtool'/>"
"      <menuitem action='MatrixEdit'/>"
"    </menu>"
"    <menuitem action='About'/>"
"  </menubar>"
"</ui>";
	GtkActionEntry entries[] = {
{ "FileMenu",    NULL,                  menu_file },
{ "Clear",       GTK_STOCK_NEW,         _("_Clear"),          "<ctrl>N", NULL, G_CALLBACK(advancedwin_file_clear) },
{ "SaveAs",      GTK_STOCK_SAVE_AS,     menu_file_save_as,    "<ctrl>S", NULL, G_CALLBACK(advanced_file_save_as) },
{ "Load",        GTK_STOCK_EXECUTE,     _("Load a .Ump-file..."),NULL,   NULL, G_CALLBACK(advanced_file_load) },
{ "Close",       GTK_STOCK_CLOSE,       menu_file_close,      "<ctrl>W", NULL, G_CALLBACK(advancedwin_close) },
{ "Quit",        GTK_STOCK_QUIT,        menu_file_quit,       "<ctrl>Q", NULL, G_CALLBACK(UMP_gtk_quit) },
{ "PrefMenu",    NULL,                  menu_pref },
{ "Radian",      NULL,                  _("_Radian"),         NULL,      NULL, G_CALLBACK(advanced_opt_radian) },
{ "Degree",      NULL,                  _("_Degree"),         NULL,      NULL, G_CALLBACK(advanced_opt_degree) },
{ "Preferences", GTK_STOCK_PREFERENCES, menu_pref,            NULL,      NULL, G_CALLBACK(prefwin_show_advanced) },
{ "ToolsMenu",   NULL,                  menu_tools },
{ "Simple",      NULL,                  menu_tools_simple,    NULL,      NULL, G_CALLBACK(simplewin_show) },
{ "Editor",      NULL,                  menu_tools_editor,    NULL,      NULL, G_CALLBACK(editor_showwin) },
{ "Graphtool",   NULL,                  menu_tools_graphtool, NULL,      NULL, G_CALLBACK(graphwin_show) },
{ "MatrixEdit",  NULL,                  menu_tools_matrixedit,NULL,      NULL, G_CALLBACK(matrix_editor_show) },
{ "About",       NULL,                  menu_about,           NULL,      NULL, G_CALLBACK(aboutwin_show_advanced) }
};
	guint n_entries = G_N_ELEMENTS(entries);

	actions = gtk_action_group_new("Actions");
	gtk_action_group_add_actions(actions, entries, n_entries, NULL);
	ui_manager = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui_manager, actions, 0);
	gtk_window_add_accel_group( GTK_WINDOW(advancedwin), gtk_ui_manager_get_accel_group(ui_manager) );
	if(!gtk_ui_manager_add_ui_from_string(ui_manager, menu, -1, &error))
	{
		g_message("building menus failed: %s", error->message);
		g_error_free(error);
		return 0;
	}
	gtk_ui_manager_ensure_update(ui_manager);

	return gtk_ui_manager_get_widget(ui_manager, "/MenuBar");
}

GtkWindow* advancedwin_show(void)
{
	if(advancedwin == 0)
	{
		GtkWidget *advanced_hbox, *advanced_scrolled, *tmp;

		advancedwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_widget_set_size_request(GTK_WIDGET(advancedwin), 150,100);
		gtk_window_set_title(GTK_WINDOW(advancedwin), PRG_NAME );
		g_signal_connect(GTK_OBJECT(advancedwin), "delete-event", G_CALLBACK(advancedwin_close), NULL);

		advanced_vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(advancedwin), advanced_vbox);


		if((advancedwin_menu = advancedwin_build_menu()) == 0)
		{
			gtk_widget_destroy( advancedwin );
			advancedwin = 0;
			return 0;
		}
		gtk_box_pack_start(GTK_BOX(advanced_vbox), advancedwin_menu, FALSE, FALSE, 0);


		advanced_textbuffer = gtk_text_buffer_new(NULL);

		advanced_textview = gtk_text_view_new_with_buffer(advanced_textbuffer);
		gtk_text_view_set_editable(GTK_TEXT_VIEW(advanced_textview),FALSE);
		gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW(advanced_textview),FALSE);

		{
			PangoFontDescription *font_desc;
			font_desc = pango_font_description_from_string("Monospace 10");
			gtk_widget_modify_font(advanced_textview, font_desc);
			pango_font_description_free(font_desc);
		}
		{
			GtkTextIter iter;
			gtk_text_buffer_get_end_iter( advanced_textbuffer, &iter);
			gtk_text_buffer_create_mark(advanced_textbuffer, "the_end", &iter, FALSE);
		}
		advancedwin_file_clear();


		advanced_scrolled = gtk_scrolled_window_new( NULL, NULL);
		gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(advanced_scrolled), GTK_POLICY_AUTOMATIC,
		                                GTK_POLICY_AUTOMATIC );

		gtk_container_add(GTK_CONTAINER(advanced_scrolled), advanced_textview);


		advanced_combo = gtk_combo_box_entry_new_text();
		gtk_entry_set_max_length(GTK_ENTRY(GTK_BIN(advanced_combo)->child),2000);
		g_signal_connect_swapped(GTK_OBJECT(GTK_BIN(advanced_combo)->child), "activate", G_CALLBACK(advanced_calculate), NULL);
		g_signal_connect(GTK_OBJECT(GTK_BIN(advanced_combo)->child), "insert-text", G_CALLBACK(advanced_entry_insert), NULL);
		if( advanced_history.get_size() == 0 )
			gtk_combo_box_prepend_text(GTK_COMBO_BOX(advanced_combo), "");
		else
		{
			for( int i=0; i<advanced_history.get_size(); i++ )
				gtk_combo_box_append_text( GTK_COMBO_BOX(advanced_combo),
				                           advanced_history[i]->c_str() );
		}
#ifdef USE_AUTO_COMPLETION
		{
			GtkEntryCompletion *complete = gtk_entry_completion_new();
			gtk_entry_completion_set_model( complete, gtk_combo_box_get_model( GTK_COMBO_BOX(advanced_combo) ) );
			gtk_entry_completion_set_inline_completion( complete, TRUE );
			gtk_entry_completion_set_text_column( complete, 0 );
			gtk_entry_set_completion( GTK_ENTRY(GTK_BIN(advanced_combo)->child), complete );
		}
#endif

		advanced_button = gtk_button_new_with_mnemonic( _("Calculate") );
		g_signal_connect_swapped(GTK_OBJECT(advanced_button), "clicked", G_CALLBACK(advanced_calculate), NULL);



		gtk_box_pack_start(GTK_BOX(advanced_vbox), advanced_scrolled, TRUE, TRUE, 0);

		advanced_hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(advanced_hbox), advanced_combo, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(advanced_hbox), advanced_button, FALSE, FALSE, 0);

		gtk_box_pack_start(GTK_BOX(advanced_vbox), advanced_hbox, FALSE, FALSE, 0);

		advanced_hbox = gtk_hbox_new(FALSE, 0);
		advanced_degfix = gtk_label_new( "" );
		gtk_misc_set_alignment(GTK_MISC(advanced_degfix), 0.0f, 0.5f);
		gtk_misc_set_padding(GTK_MISC(advanced_degfix), 2, 2);
		gtk_box_pack_start(GTK_BOX(advanced_hbox), advanced_degfix, FALSE, FALSE, 0);

		tmp = gtk_statusbar_new();
		gtk_box_pack_start(GTK_BOX(advanced_hbox), tmp, TRUE, TRUE, 0);

		gtk_box_pack_start(GTK_BOX(advanced_vbox), advanced_hbox, FALSE, FALSE, 0);

		gtk_window_resize(GTK_WINDOW(advancedwin),625,400);
		gtk_widget_show_all(advancedwin);
		gtk_widget_grab_focus(GTK_BIN(advanced_combo)->child);

		degfix_changed();
		open_windows |= ADVANCED_WIN;
	}
	else
		gtk_window_present(GTK_WINDOW(advancedwin));

	return GTK_WINDOW( advancedwin );
}

void advanced_entry_insert(GtkEditable *editable, gchar *new_text, gint new_text_length,
                           gint *position)
{
	const char *text;

	if(new_text_length == 1)
	{
		text = gtk_entry_get_text(GTK_ENTRY(GTK_BIN(advanced_combo)->child));
		if(text[0] == '\0')
		{
			switch(new_text[0])
			{
			case '+':  case '-':  case '*':
			case '/':  case '\\': case '^':
				gtk_editable_insert_text(editable, "ans ", 4, position);
			}
		}
	}
}
void advanced_add_line_in_combo(const char *text)
{
	utf8_string str(text);

	if( advanced_history.get_size() == 0)
		gtk_combo_box_remove_text( GTK_COMBO_BOX(advanced_combo), 0 );

	for( int i=0; i<advanced_history.get_size(); i++ )
	{
		if( str == *advanced_history[i] )
		{
			gtk_combo_box_remove_text( GTK_COMBO_BOX(advanced_combo), i );
			advanced_history.remove( i );
			i = -1; // this restarts the search in history list
		}
	}

	if( advanced_history.get_size() >= 100)
	{
		gtk_combo_box_remove_text( GTK_COMBO_BOX(advanced_combo), 0 );
		advanced_history.remove( 0 );
	}

	gtk_combo_box_append_text( GTK_COMBO_BOX(advanced_combo), text );
	advanced_history.append( str );
}
void advanced_calculate(void)
{
	const char *text;

	text = gtk_entry_get_text(GTK_ENTRY(GTK_BIN(advanced_combo)->child));

	for(int i=0;;i++)
	{
		if(text[i] == 0) // only spaces or tabs on this line
		{
			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(advanced_combo)->child), "");
			gtk_widget_grab_focus(GTK_BIN(advanced_combo)->child);
			return;
		}

		if( text[i] != ' '  &&  text[i] != '\t' )
			break;
	}

	advanced_add_line_in_combo( text );

	advancedwin_showtext( "\n" );
	advancedwin_showtext( text );
	advancedwin_showtext( "\n" );

	try
	{
		utf8_string str;
		math::Variable *varpointer;

		varpointer = math::global_varlist.get_pointer_rw( math::global_varlist.get_id("ans") );

		calc_function.set_code( text );
		calc_function.calc( varpointer );
		calc_function.clear();
		
		if( !varpointer->is_void() )
		{
			varpointer->append_to_string( str, format );
			advancedwin_showtext( str );
			advancedwin_showtext( "\n" );
		}
	}
	catch(error_obj error)
	{

		advancedwin_showtext( "\n  " );
		advancedwin_showtext( error.msg );
		advancedwin_showtext( "\n\n" );
	}

	gtk_entry_set_text(GTK_ENTRY( GTK_BIN(advanced_combo)->child), "" );
	gtk_widget_grab_focus( GTK_BIN(advanced_combo)->child );
}


void advancedwin_showtext(const utf8_string &string)
{
	if(advancedwin == 0)
		return;

	GtkTextIter iter;

	gtk_text_buffer_get_end_iter( advanced_textbuffer, &iter);
	gtk_text_buffer_insert( advanced_textbuffer, &iter, string.c_str(), -1);

	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(advanced_textview),
	gtk_text_buffer_get_mark(advanced_textbuffer, "the_end"));
}

void advanced_file_load(void)
{
	GtkWidget *file_chooser;
	GtkFileFilter *filter;

	file_chooser = gtk_file_chooser_dialog_new( "Select a file", GTK_WINDOW(advancedwin),
	                                            GTK_FILE_CHOOSER_ACTION_OPEN,
	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL );

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.ump");
	gtk_file_filter_set_name(filter, filter_ump_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_filter_set_name(filter, filter_all_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		math::CodeBlock code;
		char *filename;

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
		try
		{
			code.set_code_from_file( filename );
		}
		catch(error_obj error)
		{
			advancedwin_showtext(error.msg);
			advancedwin_showtext("\n");
		}
		g_free(filename);
	}

	gtk_widget_destroy(file_chooser);
}

void file_save(const char *filename)
{
	GtkTextIter start, end;
	char *text;
	uint32 bufferlen;
	FILE *fp;

	gtk_text_buffer_get_bounds( advanced_textbuffer, &start, &end );
	text = gtk_text_iter_get_text( &start, &end );

	fp = fopen( filename, "wb" );
	if( fp == 0 )
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
		dialog = gtk_message_dialog_new( GTK_WINDOW(advancedwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run( GTK_DIALOG(dialog) );
		g_free(text);
		return;
	}

	bufferlen = strlen(text);
	if( fwrite( text, 1, bufferlen, fp ) != bufferlen )
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
		dialog = gtk_message_dialog_new( GTK_WINDOW(advancedwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run( GTK_DIALOG(dialog) );
		g_free(text);
		return;
	}
	g_free(text);

	if( fclose(fp) != 0 )
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), filename );
		dialog = gtk_message_dialog_new( GTK_WINDOW(advancedwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run( GTK_DIALOG(dialog) );
		return;
	}
}

void advanced_file_save_as(void)
{
	GtkWidget *file_chooser;
	GtkFileFilter *filter;

	file_chooser = gtk_file_chooser_dialog_new( dialog_title_save_as, GTK_WINDOW(advancedwin),
	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                            GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL );

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern( filter, "*.ump" );
	gtk_file_filter_set_name( filter, filter_ump_files );
	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(file_chooser), filter );

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_filter_set_name(filter, filter_all_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		char *filename;

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		file_save(filename);

		g_free(filename);
	}

	gtk_widget_destroy(file_chooser);
}

void advancedwin_file_clear(void)
{
	if(advancedwin == 0)
		return;

	char tmp_str[400];
	snprintf(tmp_str,400,"\n%s %s\n\n  Copyright (c) %s Mattias Hultgren\n",PRG_NAME,PRG_VERSION,PRG_DATE);
	gtk_text_buffer_set_text(advanced_textbuffer,tmp_str,-1);

	GtkTextIter iter;

	gtk_text_buffer_get_end_iter( advanced_textbuffer, &iter);
	gtk_text_buffer_insert( advanced_textbuffer, &iter, get_start_message(), -1);

	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(advanced_textview),
	gtk_text_buffer_get_mark(advanced_textbuffer, "the_end"));
}
void advancedwin_translate(void)
{
	if(advancedwin != 0)
	{
		GtkWidget *new_menu;

		gtk_button_set_label( GTK_BUTTON(advanced_button), _("Calculate") );

		if(( new_menu = advancedwin_build_menu()) == 0 )
			return;

		gtk_widget_destroy( advancedwin_menu );
		advancedwin_menu = new_menu;

		gtk_box_pack_start(GTK_BOX(advanced_vbox), advancedwin_menu, FALSE, FALSE, 0);
		gtk_box_reorder_child( GTK_BOX(advanced_vbox), advancedwin_menu, 0 );

		try{ degfix_changed(); }
		catch(...) { }
	}
}
void advancedwin_close(void)
{
	if(advancedwin != 0)
	{
		gtk_widget_destroy(advancedwin);
		advancedwin = 0;

		open_windows &= ~ADVANCED_WIN;
		if(open_windows == 0)
			UMP_gtk_quit();
	}
}
void advanced_opt_radian(void)
{
	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(advanced_combo)->child), "degfix = radian");
	gtk_widget_activate(GTK_BIN(advanced_combo)->child);
}
void advanced_opt_degree(void)
{
	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(advanced_combo)->child), "degfix = degree");
	gtk_widget_activate(GTK_BIN(advanced_combo)->child);
}
