
/*
 *  UMP - Unnamed Math Program
 *  Copyright (c) 2004-2005 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */

#include <gtk/gtk.h>
#include "ump_editor.h"
#include "ump_simplewin.h"
#include "ump_aboutwin.h"
#include "ump_advancedwin.h"
#include "ump_prefwin.h"
#include "ump_graphwin.h"
#include "matrix_editor.h"
#include "vartypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"

GtkWidget *editorwin = 0, *editorwin_menu, *editorwin_vbox, *editor_status;
GtkTextBuffer *textbuffer;

char *filename = 0;
uint32 namelen = 0;

bool modified = false;
bool editor_autoindent = true;

void aboutwin_show_editor(void) { aboutwin_show(GTK_WINDOW(editorwin)); }
void prefwin_show_editor(void) { prefwin_show(GTK_WINDOW(editorwin)); }

void file_save(void);
void file_save_as(void);

void update_title(void)
{
	char *str;

	const char *trans = _("Ump Code editor");

	try{ str = new char [strlen(filename) + 30]; }
	catch(...)
	{
		gtk_window_set_title(GTK_WINDOW(editorwin), trans);
		return;
	}

	if(modified == false)
		snprintf(str,strlen(filename) + 30,"%s - %s",filename,trans);
	else
		snprintf(str,strlen(filename) + 30,"%s * - %s",filename,trans);
	gtk_window_set_title(GTK_WINDOW(editorwin), str);
	delete [] str;
}
void setfilename(const char *newfilename)
{
	uint32 newnamelen;

	modified = false;

	newnamelen = strlen(newfilename) + 1;
	if(newnamelen > namelen)
	{
		char *newfile;

		try{  newfile = new char [newnamelen];  }
		catch(...)
		{
			delete [] filename;
			filename = 0;
			gtk_window_set_title(GTK_WINDOW(editorwin), _("Ump Code editor") );
		}
		delete [] filename;
		filename = newfile;
		namelen = newnamelen;
	}
	strcpy(filename,newfilename);

	update_title();
}

void check_save(void)
{
	GtkWidget *dialog;

	if(modified == true)
	{
		gint response;

		dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
		             _("File '%s' has been changed.\nDo you want to save the changes?"), filename );

		gtk_window_set_title( GTK_WINDOW(dialog), _("Save file?") );
		response = gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);

		if(response == GTK_RESPONSE_YES)
			file_save();
	}
}

void file_new(void)
{
	check_save();

	gtk_text_buffer_set_text(textbuffer,"",-1);

	setfilename( _("[Unsaved]") );
}
void file_open(void)
{
	GtkWidget *file_chooser;
	GtkFileFilter *filter;
	char *selected_filename;
	FILE *fp;
	long filesize;
	char *str;

	check_save();


	file_chooser = gtk_file_chooser_dialog_new( dialog_title_open_file, GTK_WINDOW(editorwin),
	                                            GTK_FILE_CHOOSER_ACTION_OPEN,
	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL );

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.ump");
	gtk_file_filter_set_name(filter, filter_ump_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_filter_set_name(filter, filter_all_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		selected_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		fp = fopen(selected_filename,"rb");
		if(fp == 0)
		{
			setcode_thrower_error:
			if(fp != 0)
				fclose(fp);

			GtkWidget *dialog;
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
			dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
			                                 GTK_BUTTONS_CLOSE, tmp_err );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			g_free(selected_filename);
			gtk_widget_destroy(file_chooser);
			return;
		}

		if(fseek(fp,0L,SEEK_END) != 0)
			goto setcode_thrower_error;
		filesize = ftell(fp);
		if(filesize == -1)
			goto setcode_thrower_error;
		if(fseek(fp,0L,SEEK_SET) != 0)
			goto setcode_thrower_error;

		try{  str = new char [filesize + 1];  }
		catch(...)
		{
			fclose(fp);
			GtkWidget *dialog;
			try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
			catch(error_obj error)
			{
				dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin),
				                                 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
				                                 GTK_BUTTONS_CLOSE, error.msg );
			}
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			g_free(selected_filename);
			gtk_widget_destroy(file_chooser);
			return;
		}

		if(fread(str,1,filesize,fp) != filesize)
		{
			delete [] str;
			goto setcode_thrower_error;
		}
		if(fclose(fp) != 0)
		{
			delete [] str;

			GtkWidget *dialog;
			char tmp_err[ERROR_OBJ_MSG_LEN];
			snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), selected_filename );
			dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
			                                 GTK_BUTTONS_CLOSE, tmp_err );
			gtk_dialog_run(GTK_DIALOG (dialog));
			gtk_widget_destroy(dialog);
			g_free(selected_filename);
			gtk_widget_destroy(file_chooser);
			return;
		}

		str[filesize] = '\0';

		gtk_text_buffer_set_text(textbuffer,str,-1);

		delete [] str;

		setfilename(selected_filename);
		g_free(selected_filename);
	}

	gtk_widget_destroy(file_chooser);
}
void file_save(void)
{
	GtkTextIter start, end;
	char *text;
	uint32 bufferlen;
	FILE *fp;

	if(filename == 0)
	{
		file_save_as();
		return;
	}
	if( filename[0] == '\0'  ||  filename[0] == '[' )
	{
		file_save_as();
		return;
	}

	gtk_text_buffer_get_bounds(textbuffer, &start, &end);
	text = gtk_text_iter_get_text(&start, &end);

	fp = fopen(filename,"wb");
	if(fp == 0)
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
		dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run(GTK_DIALOG(dialog));
		g_free(text);
		return;
	}

	bufferlen = strlen(text);
	if( fwrite(text,1,bufferlen,fp) != bufferlen )
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't open/read from file (%s)"), filename );
		dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run(GTK_DIALOG(dialog));
		g_free(text);
		return;
	}
	g_free(text);

	if(fclose(fp) != 0)
	{
		GtkWidget *dialog;
		char tmp_err[ERROR_OBJ_MSG_LEN];
		snprintf( tmp_err, ERROR_OBJ_MSG_LEN, _("Couldn't close file (%s)"), filename );
		THROW_ERROR( ErrorType_File_IO, tmp_err );
		dialog = gtk_message_dialog_new( GTK_WINDOW(editorwin), GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tmp_err );
		gtk_dialog_run(GTK_DIALOG(dialog));
		return;
	}

	modified = false;
	update_title();
}
void file_save_as_umpfile(GtkWidget *widget, gpointer user_data)
{
	GtkWidget *file_selector = (GtkWidget *)user_data;
	const gchar *selected_filename;

	selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));

	{
		bool tmp = modified;
		setfilename(selected_filename);
		modified = tmp;
	}

	file_save();
}
void file_save_as(void)
{
	GtkWidget *file_chooser;
	GtkFileFilter *filter;

	file_chooser = gtk_file_chooser_dialog_new( dialog_title_save_as, GTK_WINDOW(editorwin),
	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                            GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL );

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.ump");
	gtk_file_filter_set_name(filter, filter_ump_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_filter_set_name(filter, filter_all_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		bool tmp = modified;
		char *filename;

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		setfilename(filename);
		modified = tmp;

		file_save();

		g_free(filename);
	}

	gtk_widget_destroy(file_chooser);
}
void edit_cut(void)
{
	gtk_text_buffer_cut_clipboard(textbuffer, gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), TRUE);
}
void edit_copy(void)
{
	gtk_text_buffer_copy_clipboard(textbuffer, gtk_clipboard_get(GDK_SELECTION_CLIPBOARD) );
}
void edit_paste(void)
{
	gtk_text_buffer_paste_clipboard(textbuffer, gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), NULL, TRUE);
}
void edit_delete(void)
{
	gtk_text_buffer_delete_selection(textbuffer,TRUE,TRUE);
}
void edit_sel_all(void)
{
	GtkTextIter start, end;

	gtk_text_buffer_get_bounds(textbuffer, &start, &end);

	gtk_text_buffer_move_mark_by_name(textbuffer,"selection_bound",&start);
	gtk_text_buffer_move_mark_by_name(textbuffer,"insert",&end);
}

void text_changed(GtkTextBuffer *textbuf, gpointer user_data)
{
	modified = true;

	update_title();
}


gboolean skip_white_spaces(gunichar ch, gpointer user_data)
{
	if( ch == gunichar(' ')  ||  ch == gunichar('\t') )
		return FALSE;
	else
		return TRUE;
}

void editor_status_update_buffer( GtkTextBuffer *buffer )
{
	static char str[50];
	static char tmp_row[20];
	static char tmp_col[20];
	GtkTextIter iter, iter_start;
	gint row, col;

	gtk_text_buffer_get_iter_at_mark( buffer, &iter, 
	gtk_text_buffer_get_mark( buffer, "insert" ) );

	row = gtk_text_iter_get_line( &iter );
	gtk_text_buffer_get_iter_at_line( buffer, &iter_start, row );
	col = gtk_text_iter_get_offset( &iter ) - gtk_text_iter_get_offset( &iter_start );

	uint64_to_char( uint64(row+1), tmp_row );
	uint64_to_char( uint64(col+1), tmp_col );
	snprintf( str, 50, " %s: %s  %s: %s ", _("Row"), tmp_row, _("Column"),  tmp_col );

	gtk_label_set_text( GTK_LABEL( editor_status ), str );
}

void editor_status_update( GtkTextView *widget )
{
	editor_status_update_buffer( gtk_text_view_get_buffer( widget ) );
}

void text_insert(GtkTextBuffer *textbuf, GtkTextIter *iter, gchar *str, gint length)
{
	GtkTextIter start_of_prev_line = *iter, end_of_indentation;

	if( str[0] == '\n'  &&  length == 1  &&  editor_autoindent )
	{
		if( gtk_text_iter_backward_line( &start_of_prev_line ) == TRUE )
		{
			end_of_indentation = start_of_prev_line;
			if( gtk_text_iter_forward_find_char( &end_of_indentation, skip_white_spaces,
			      NULL, 0) == TRUE )
			{
				gchar *text;

				text = gtk_text_iter_get_text( &start_of_prev_line, &end_of_indentation );

				if( *text == ' '  ||  *text == '\t' )
					gtk_text_buffer_insert_at_cursor( textbuf, text, strlen( text ) );
			}
		}
	}
	editor_status_update_buffer( textbuf );
}

GtkWidget* editorwin_build_menu(void)
{
	GtkActionGroup *actions;
	GtkUIManager *ui_manager;
	GError *error = NULL;
	static const gchar *menu =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='Open'/>"
"      <separator/>"
"      <menuitem action='Save'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='EditMenu'>"
"      <menuitem action='Cut'/>"
"      <menuitem action='Copy'/>"
"      <menuitem action='Paste'/>"
"      <menuitem action='Delete'/>"
"      <separator/>"
"      <menuitem action='SelAll'/>"
"      <separator/>"
"      <menuitem action='Preferences'/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Simple'/>"
"      <menuitem action='Advanced'/>"
"      <menuitem action='Graphtool'/>"
"      <menuitem action='MatrixEdit'/>"
"    </menu>"
"    <menuitem action='About'/>"
"  </menubar>"
"</ui>";
	GtkActionEntry entries[] = {
{ "FileMenu",    NULL,                  menu_file },
{ "New",         GTK_STOCK_NEW,         menu_file_new,        "<ctrl>N",        NULL, G_CALLBACK(file_new) },
{ "Open",        GTK_STOCK_OPEN,        menu_file_open,       "<ctrl>O",        NULL, G_CALLBACK(file_open) },
{ "Save",        GTK_STOCK_SAVE,        menu_file_save,       "<ctrl>S",        NULL, G_CALLBACK(file_save) },
{ "SaveAs",      GTK_STOCK_SAVE_AS,     menu_file_save_as,    "<shift><ctrl>S", NULL, G_CALLBACK(file_save_as) },
{ "Close",       GTK_STOCK_CLOSE,       menu_file_close,      "<ctrl>W",        NULL, G_CALLBACK(editor_close) },
{ "Quit",        GTK_STOCK_QUIT,        menu_file_quit,       "<ctrl>Q",        NULL, G_CALLBACK(UMP_gtk_quit) },
{ "EditMenu",    NULL,                  edit_edit },
{ "Cut",         GTK_STOCK_CUT,         edit_edit_cut,        "<ctrl>X",        NULL, G_CALLBACK(edit_cut) },
{ "Copy",        GTK_STOCK_COPY,        edit_edit_copy,       "<ctrl>C",        NULL, G_CALLBACK(edit_copy) },
{ "Paste",       GTK_STOCK_PASTE,       edit_edit_paste,      "<ctrl>V",        NULL, G_CALLBACK(edit_paste) },
{ "Delete",      NULL,                  edit_edit_delete,     NULL,             NULL, G_CALLBACK(edit_delete) },
{ "SelAll",      NULL,                  edit_edit_select_all, "<ctrl>A",        NULL, G_CALLBACK(edit_sel_all) },
{ "Preferences", GTK_STOCK_PREFERENCES, menu_pref,            NULL,             NULL, G_CALLBACK(prefwin_show_editor) },
{ "ToolsMenu",   NULL,                  menu_tools },
{ "Simple",      NULL,                  menu_tools_simple,    NULL,             NULL, G_CALLBACK(simplewin_show) },
{ "Advanced",    NULL,                  menu_tools_advanced,  NULL,             NULL, G_CALLBACK(advancedwin_show) },
{ "Graphtool",   NULL,                  menu_tools_graphtool, NULL,             NULL, G_CALLBACK(graphwin_show) },
{ "MatrixEdit",  NULL,                  menu_tools_matrixedit,NULL,             NULL, G_CALLBACK(matrix_editor_show) },
{ "About",       NULL,                  menu_about,           NULL,             NULL, G_CALLBACK(aboutwin_show_editor) }
};
	guint n_entries = G_N_ELEMENTS(entries);


	actions = gtk_action_group_new("Actions");
	gtk_action_group_add_actions(actions, entries, n_entries, NULL);
	ui_manager = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui_manager, actions, 0);
	gtk_window_add_accel_group( GTK_WINDOW(editorwin), gtk_ui_manager_get_accel_group(ui_manager) );
	if(!gtk_ui_manager_add_ui_from_string(ui_manager, menu, -1, &error))
	{
		g_message("building menus failed: %s", error->message);
		g_error_free(error);
		return 0;
	}
	gtk_ui_manager_ensure_update(ui_manager);

	return gtk_ui_manager_get_widget(ui_manager, "/MenuBar");
}

GtkWindow* editor_showwin(void)
{
	if(editorwin == 0)
	{
		GtkWidget *textview,*scrolled;

		editorwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);

		setfilename( _("[Unsaved]") );
		g_signal_connect_swapped(G_OBJECT(editorwin), "delete-event", G_CALLBACK(editor_close), NULL);
		gtk_widget_set_size_request(GTK_WIDGET(editorwin), 150,100);

		editorwin_vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(editorwin), editorwin_vbox);

		if((editorwin_menu = editorwin_build_menu()) == 0)
		{
			gtk_widget_destroy( editorwin );
			editorwin = 0;
			return 0;
		}
		gtk_box_pack_start(GTK_BOX(editorwin_vbox), editorwin_menu, FALSE, FALSE, 0);


		textbuffer = gtk_text_buffer_new(NULL);
		g_signal_connect_swapped(G_OBJECT(textbuffer), "changed", G_CALLBACK(text_changed), NULL);
		g_signal_connect_after(G_OBJECT(textbuffer), "insert-text", G_CALLBACK(text_insert), NULL);
		g_signal_connect_after(G_OBJECT(textbuffer), "delete-range", G_CALLBACK(editor_status_update_buffer), NULL);

		textview = gtk_text_view_new_with_buffer(textbuffer);
		g_signal_connect_after( G_OBJECT(textview), "move-cursor",
		G_CALLBACK(editor_status_update), NULL);

		{
			PangoFontDescription *font_desc;
			font_desc = pango_font_description_from_string("Monospace 10");
			gtk_widget_modify_font(textview, font_desc);
			pango_font_description_free(font_desc);
		}


		modified = false;
	
		scrolled = gtk_scrolled_window_new( NULL, NULL);
		gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC,
										GTK_POLICY_AUTOMATIC );
	
		gtk_container_add(GTK_CONTAINER(scrolled), textview);
	
		gtk_box_pack_start(GTK_BOX(editorwin_vbox), scrolled, TRUE, TRUE, 0);
	
		{
			GtkWidget *tbox, *tbar;
			tbox = gtk_hbox_new( FALSE, 0 );
	
			editor_status = gtk_label_new( PRG_NAME );
			gtk_misc_set_alignment( GTK_MISC(editor_status), 0.0f, 0.5f );
			gtk_box_pack_start( GTK_BOX(tbox), editor_status, FALSE, FALSE, 0 );
	
			tbar = gtk_statusbar_new();
			gtk_box_pack_start( GTK_BOX(tbox), tbar, TRUE, TRUE, 0 );
	
			gtk_box_pack_start( GTK_BOX(editorwin_vbox), tbox, FALSE, FALSE, 0 );
		}
	
	
	
		gtk_window_resize(GTK_WINDOW(editorwin),500,400);
		gtk_widget_show_all(editorwin);
		open_windows |= EDITOR_WIN;
	}
	else
		gtk_window_present(GTK_WINDOW(editorwin));

	return GTK_WINDOW( editorwin );
}

void editor_close(void)
{
	if(editorwin != 0)
	{
		check_save();

		gtk_widget_destroy(editorwin);
		editorwin = 0;

		delete [] filename;
		filename = 0;
		namelen = 0;

		open_windows &= ~EDITOR_WIN;
		if(open_windows == 0)
			UMP_gtk_quit();
	}
}

void editorwin_translate(void)
{
	if(editorwin != 0)
	{
		GtkWidget *new_menu;

		if(( new_menu = editorwin_build_menu()) == 0 )
			return;

		gtk_widget_destroy( editorwin_menu );
		editorwin_menu = new_menu;

		gtk_box_pack_start(GTK_BOX(editorwin_vbox), editorwin_menu, FALSE, FALSE, 0);
		gtk_box_reorder_child( GTK_BOX(editorwin_vbox), editorwin_menu, 0 );

		update_title();

		editor_status_update_buffer( textbuffer );
	}
}
