
/*
 *  Ump - Unnamed Math Program
 *  Copyright (c) 2004-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See main.cpp
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <sys/time.h>
#include "ump_picturewin.h"
#include "main.h"
#include "vector.h"
#include "picture2gtk.h"
#include "math2.h"
#include "vartypes.h"

GtkWidget *picturewin = 0, *picwin_status, *picwin_combo_trace, *picwin_combo_action,
          *picwin_label_trace, *picturewin_vbox, *picturewin_menu;
guint         pic_statusid;
picture2gtk   picbild;
math::Picture picbild2;
Format        stdformat;
Matrix        trace_matrix;

int32 SNAP_SIZE = 5;
struct SnapInfo
{
	enum SnapType { Snap_Root, Snap_Minimum, Snap_Maximum, Snap_Intersection };
	SnapType snap_type;
	int32 x, intersect_with;
	floatx fx, fy;
	bool calculated;
};
Vector<SnapInfo> snap_info[10];

timeval action_combo_changed;

graph_settings act_settings;

int32  old_tx, old_ty;
int    old_best_match_i = -1;
guchar *picwin_rowbuffer = 0, *picwin_colbuffer = 0;
int    picwin_rowbuffer_len = 0, picwin_colbuffer_len = 0;

bool picwin_first_click_done = false;

enum LeftClickAction { LeftClickAction_Zoom, LeftClickAction_FindRoot, LeftClickAction_FindMaximumMinimum,
                       LeftClickAction_FindIntersection, LeftClickAction_CalcDerivate, LeftClickAction_CalcIntegral,
                       LeftClickAction_CalcLength };

bool picturewin_action_dont_show_dialog = false;

graph_settings::graph_settings() throw(error_obj)
{
	show_code = false;
	use_antialiasing = true;

	width = 400;
	height = 400;
	xmin = "-10";
	xmax = "10";
	ymin = "-10";
	ymax = "10";
	bgcolor = picture_h::color::WHITE;

	grid_on = true;
	grid_color = picture_h::color::LIGHT_GRAY;
	grid_scale1 = "2";
	grid_scale2 = "2";
	grid_type = "Rect";

	axes_on = true;
	axes_color = picture_h::color::BLACK;
	scalex = "1";
	scaley = "1";
	scale_length = 3;

	plot_mode = "Connected";
	show_active_page_only = true;
	active_page = 0;

	polar_start = "0";
	polar_stop = "2pi";
	polar_steps = "40";
	par_start = "-10";
	par_stop = "10";
	par_steps = "40";

	for( int i=0; i<10; i++ )
	{
		func[i].show_this = false;
		polar[i].show_this = false;
		func[i].entry = "";
		polar[i].entry = "";
		if( i<5 )
		{
			par[i].show_this = false;
			par[i].entry = "";
			par[i].entry2 = "";
		}
	}
	func[0].color = picture_h::color::BLUE;
	func[1].color = picture_h::color::RED;
	func[2].color = picture_h::color::BLACK;
	func[3].color = picture_h::color::DARK_GREEN;
	func[4].color = picture_h::color::PINK;
	func[5].color = picture_h::color::ORANGE;
	func[6].color = ((picture_h::colorrgb) { 159, 0, 255 });
	func[7].color = ((picture_h::colorrgb) { 60, 117, 117 });
	func[8].color = ((picture_h::colorrgb) { 0, 178, 208 });
	func[9].color = ((picture_h::colorrgb) { 156, 103, 131 });

	polar[0].color = picture_h::color::BLUE;
	polar[1].color = picture_h::color::RED;
	polar[2].color = picture_h::color::BLACK;
	polar[3].color = picture_h::color::DARK_GREEN;
	polar[4].color = picture_h::color::PINK;
	polar[5].color = picture_h::color::ORANGE;
	polar[6].color = ((picture_h::colorrgb) { 159, 0, 255 });
	polar[7].color = ((picture_h::colorrgb) { 60, 117, 117 });
	polar[8].color = ((picture_h::colorrgb) { 0, 178, 208 });
	polar[9].color = ((picture_h::colorrgb) { 156, 103, 131 });

	par[0].color = picture_h::color::BLUE;
	par[1].color = picture_h::color::RED;
	par[2].color = picture_h::color::BLACK;
	par[3].color = picture_h::color::DARK_GREEN;
	par[4].color = picture_h::color::PINK;
}
		       
		       
void picwin_mousedown( const picture2gtk *src, int32 x, int32 y, int32 button );
void find_root( int function, floatx guess, Complex &x, Complex &y ) throw(error_obj);
bool find_min_or_max( int function, floatx guess, Complex &x, Complex &y ) throw(error_obj);  // true if minimum,  false if maximum
void find_intersection( int f1, int f2, floatx guess, Complex &x, Complex &y ) throw(error_obj);

void calculate_snap( SnapInfo *snap, int function ) throw(error_obj)
{
	Complex x, y;
	error_obj remove_this_snap_point;

	switch( snap->snap_type )
	{
	case SnapInfo::Snap_Root:
		find_root( function, snap->fx, x, y );
		break;

	case SnapInfo::Snap_Maximum:
		if( find_min_or_max( function, snap->fx, x, y ) )
			snap->snap_type = SnapInfo::Snap_Minimum;
		break;

	case SnapInfo::Snap_Minimum:
		if( find_min_or_max( function, snap->fx, x, y ) == false )
			snap->snap_type = SnapInfo::Snap_Maximum;
		break;

	case SnapInfo::Snap_Intersection:
		find_intersection( function, snap->intersect_with, snap->fx, x, y );
		break;

	default:
		printf( "Internal error: unhandled snap_type: %ld\n", snap->snap_type );
		throw remove_this_snap_point;
	}
	if( fabsx( floatx(x.real) - snap->fx ) > ( 1.01*(picbild2.get_right() - picbild2.get_left()) / floatx(picbild2.pic.get_width()) ) )
		throw remove_this_snap_point; // there was an extreme point found but not anywhere near here
	snap->fx = floatx( x.real );
	snap->fy = floatx( y.real );
	snap->calculated = true;
}

int conv_combobox_index_to_function_index(int index)
{
	if(index < 2)
		return -1;

	index -= 2;
	for( int i=0; i<10; i++ )
	{
		if( act_settings.func[i].show_this )
		{
			if( index == 0 )
				return i;
			index--;
		}
	}

	return -1;
}
int conv_function_index_to_combobox_index(int index)
{
	int ret = 1;

	if( index < 0  ||  index >= 10 )
		return -1;

	if( !act_settings.func[index].show_this )
		return -1;

	for( int i=0; i<index; i++ )
	{
		if( act_settings.func[i].show_this )
			ret++;
	}
	return ret+2;
}

floatx get_x_value( int32 x, int function, SnapInfo *ptr = 0 )
{
	int min_dist = SNAP_SIZE, index = -1;

	if( function == -1 )
		return floatx(trace_matrix.get_complex( x+1, 1 ).real);

	for( int i=0; i<snap_info[function].get_size(); i++ )
	{
		if( abs(snap_info[function][i]->x -x) < min_dist )
		{
			if( snap_info[function][i]->calculated == false )
			{
				try{ calculate_snap( snap_info[function][i], function ); }
				catch( ... ) { snap_info[function].remove( i ); i--; continue; } // this snap wasn't calculatable
			}                                                                        // let's remove it
			min_dist = abs(snap_info[function][i]->x -x);
			index = i;
		}
	}
	if( index != -1 )
	{
		if( ptr != 0 )
			*ptr = *snap_info[function][index];
		return snap_info[function][index]->fx;
	}
	return floatx(trace_matrix.get_complex( x+1, 1 ).real);
}
floatx get_y_value( int32 x, int function, SnapInfo *ptr = 0 )
{
	int min_dist = SNAP_SIZE, index = -1;

	if( function == -1 )
		THROW_ERROR( ErrorType_General, "Intern error: Illegal value of function in get_y_value" );

	for( int i=0; i<snap_info[function].get_size(); i++ )
	{
		if( abs(snap_info[function][i]->x -x) < min_dist )
		{
			if( snap_info[function][i]->calculated == false )
			{
				try{ calculate_snap( snap_info[function][i], function ); }
				catch( ... ) { snap_info[function].remove( i ); i--; continue; } // this snap wasn't calculatable
			}                                                                        // let's remove it
			min_dist = abs(snap_info[function][i]->x -x);
			index = i;
		}
	}
	if( index != -1 )
	{
		if( ptr != 0 )
			*ptr = *snap_info[function][index];
		return snap_info[function][index]->fy;
	}
	return floatx(trace_matrix.get_complex( x+1, function + 2 ).real);
}

void find_root( int function, floatx guess, Complex &x, Complex &y ) throw(error_obj)
{
	math::CodeLine code;
	math::Variable tmpvar;
	utf8_string str;
	char char_str[1000];

	snprintf( char_str, 1000, "x = nSolve( y%ld, x, ", function );
	str = char_str;
	floatx_to_string( guess, str, stdformat);
	str.append( ", " );
	floatx_to_string( (picbild2.get_right() - picbild2.get_left()) / (floatx(picbild2.pic.get_width()) * 5.0), str, stdformat );
	str.append( " )\n" );
	if( act_settings.show_code )
		(*TEXTVIEW_FUNCTION)( str.c_str() );
	code.set_code_line( str.c_str() );
	code.calc( 0 );

	x = *math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex();

	code.set_code_line( act_settings.func[function].entry.c_str() );
	code.calc( &tmpvar );
	y = *tmpvar.get_complex();
}

bool find_min_or_max( int function, floatx guess, Complex &x, Complex &y ) throw(error_obj)
{
	math::CodeLine code;
	math::Variable tmpvar;
	utf8_string str;
	char char_str[500];

	snprintf( char_str, 500, "ans = \"nDeriv( y%ld, x, x, ", function );
	str = char_str;
	floatx_to_string( (picbild2.get_right() - picbild2.get_left()) / (floatx(picbild2.pic.get_width()) * 5.0), str, stdformat );
	str.append( " )\"\n" );
	if( act_settings.show_code )
		(*TEXTVIEW_FUNCTION)( str.c_str() );
	code.set_code_line( str.c_str() );
	code.calc( 0 );

	str = "x = nSolve( ans, x, ";
	floatx_to_string( guess, str, stdformat );
	str.append( ", " );
	floatx_to_string( (picbild2.get_right() - picbild2.get_left()) / (floatx(picbild2.pic.get_width()) * 5.0), str, stdformat);
	str.append( " )\n" );
	if( act_settings.show_code )
		(*TEXTVIEW_FUNCTION)( str.c_str() );
	code.set_code_line( str.c_str() );
	code.calc( 0 );

// saves x,y for the solution
	x = *math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex();
	code.set_code_line( act_settings.func[function].entry.c_str() );
	code.calc( &tmpvar );
	y = *tmpvar.get_complex();

	str = "x = nDeriv( ans, x, x , ";
	floatx_to_string( (picbild2.get_right() - picbild2.get_left()) / (floatx(picbild2.pic.get_width()) * 5.0), str, stdformat );
	str.append( " )\n" );
	if( act_settings.show_code )
		(*TEXTVIEW_FUNCTION)( str.c_str() );
	code.set_code_line( str.c_str() );
	code.calc( 0 );

	return ( floatx(math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex()->real) > 0.0);
}

void find_intersection( int f1, int f2, floatx guess, Complex &x, Complex &y ) throw(error_obj)
{
	char tmp_str[50];
	math::Variable tmpvar;
	math::CodeLine code;
	utf8_string str;

	snprintf( tmp_str, 50, "x = nSolve( y%ld + \" - (\" + y%ld + \")\", x, ", f1, f2 );
	str = tmp_str;
	floatx_to_string( guess, str, stdformat);
	str.append( ", " );
	floatx_to_string( (picbild2.get_right() - picbild2.get_left()) / (floatx(picbild2.pic.get_width() * 5.0)), str, stdformat);
	str.append( " )\n" );
	if( act_settings.show_code )
		(*TEXTVIEW_FUNCTION)( str.c_str() );
	code.set_code_line( str.c_str() );
	code.calc( 0 );

	x = *math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex();
	code.set_code_line( act_settings.func[f1].entry.c_str() );
	code.calc( &tmpvar );
	y = *tmpvar.get_complex();
}

int picwin_reload_combos(int combo_action_entries)
{
	if( combo_action_entries != 0 )
	{
		for( int i=0; i<7; i++ )
			gtk_combo_box_remove_text( GTK_COMBO_BOX(picwin_combo_action), 0 );
	}

	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Zoom") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Find root") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Find maximum/minimum") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Find intersection") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Calculate derivate") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Calculate integral") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_action), _("Calculate length") );

	gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), -1 );


	for( ; combo_action_entries > 0; combo_action_entries-- ) // removing all entries
		gtk_combo_box_remove_text( GTK_COMBO_BOX(picwin_combo_trace), 0 );

	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_trace), _("Off") );
	gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_trace), _("Auto") );

	combo_action_entries = 2;
	if( act_settings.active_page == 0  ||  !act_settings.show_active_page_only )
	{
		char tmp_str[100];
		const char *trans = _("Locked on y%ld");

		for( int i=0; i<10; i++ )
		{
			if( act_settings.func[i].show_this )
			{
				snprintf( tmp_str, 100, trans, i );

				gtk_combo_box_append_text( GTK_COMBO_BOX(picwin_combo_trace), tmp_str );
				combo_action_entries++;
			}
		}
	}

	gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), 0 );

	return combo_action_entries;
}

void place_marker(float x, float y, picture_h::colorrgb color)
{
	picbild2.pic.line( int32(x)-3, int32(y), int32(x)+3, int32(y), color );
	picbild2.pic.line( int32(x), int32(y)-3, int32(x), int32(y)+3, color );

	picbild.update_from( picbild2.pic );
}

void picwin_intersection_show(int32 x)
{
	GtkWidget *dialog, *label, *align, *vbox;
	char tmp_str[50];
	utf8_string str;
	int second_function,nr;
	GtkWidget *radios[10];
	Complex tmp_value_x, tmp_value_y;

	nr = 0;
	for( int i=0; i<10; i++ )
	{
		if( act_settings.func[i].show_this )
			nr++;
	}
	if( nr < 2 ) // at least 2 functions has to be drawn
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
		                                 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
		             _("At least 2 functions must be drawn\nto be able to find an intersection") );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Find intersection") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);

		return;
	}
	else if( nr > 2 ) // if it's more than 2, dialog must be shown and user should choose which...
	{
		dialog = gtk_dialog_new_with_buttons( _("Find intersection"), GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
		                                      GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );

		align = gtk_alignment_new( 0.0, 0.0, 1.0, 1.0 );
		gtk_container_set_border_width( GTK_CONTAINER(align), 5 );
		gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), align, FALSE, FALSE, 0 );

		vbox = gtk_vbox_new( FALSE, 3 );
		gtk_container_add( GTK_CONTAINER(align), vbox );

		label = gtk_label_new( _("Find intersection between") );
		gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5 );
		gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0) ;


		snprintf( tmp_str, 50, "y%ld = \"", old_best_match_i );
		str = tmp_str;
		str.append( act_settings.func[old_best_match_i].entry );
		str.append( "\"" );
		label = gtk_label_new( str.c_str() );
		gtk_misc_set_alignment( GTK_MISC(label), 0.05, 0.5 );
		gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0 );

		label = gtk_label_new( _("and") );
		gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5 );
		gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0 );

		nr = 0;
		for(int i=0;i<10;i++)
		{
			if( act_settings.func[i].show_this  &&  i != old_best_match_i )
			{
				snprintf( tmp_str, 50, "y%ld = \"", i );
				str = tmp_str;
				str.append( act_settings.func[i].entry );
				str.append( "\"" );

				label = gtk_label_new( str.c_str() );

				if(nr == 0)
				{
					radios[0] = gtk_radio_button_new( NULL );
					gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radios[0]), TRUE );
				}
				else
					radios[nr] = gtk_radio_button_new_from_widget( GTK_RADIO_BUTTON(radios[0]) );

				gtk_container_add( GTK_CONTAINER(radios[nr]), label );

				gtk_box_pack_start( GTK_BOX(vbox), radios[nr], FALSE, FALSE, 0 );

				nr++;
			}
		}

		gtk_widget_show_all( dialog );

		switch( gtk_dialog_run( GTK_DIALOG(dialog) ) )
		{
		case GTK_RESPONSE_OK:

			second_function = 0;
			for(int i=0;i<10;i++)
			{
				if( act_settings.func[i].show_this  &&  i != old_best_match_i )
				{
					if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radios[second_function]) ) ==
						TRUE )
					{
						second_function = i;
						break;
					}
					second_function++;
				}
			}
			gtk_widget_destroy(dialog);
			break;

		default:
			gtk_widget_destroy(dialog);
			return;
		}
	}
	else // if(nr == 2)
	{
		for( int i=0; i<10; i++ )
		{
			if( act_settings.func[i].show_this  &&  i != old_best_match_i )
			{
				second_function = i;
				break;
			}
		}
	}

	try
	{
		find_intersection( old_best_match_i, second_function, get_x_value( x, old_best_match_i ), tmp_value_x, tmp_value_y );

		str = _("Intersection found at");
		str.append( "\nx = " );
		tmp_value_x.append_to_string( str, format );

		str.append( "\ny = " );
		tmp_value_y.append_to_string( str, format );
		if(act_settings.show_code)
		{
			(*TEXTVIEW_FUNCTION)( str.c_str() );
			(*TEXTVIEW_FUNCTION)("\n");
		}
		place_marker( picbild2.get_float_x_pixel( floatx(tmp_value_x.real) ),
		              picbild2.get_float_y_pixel( floatx(tmp_value_y.real)),
		              act_settings.func[old_best_match_i].color );


		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
		                                 GTK_BUTTONS_OK, str.c_str() );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Find intersection") );
		gtk_dialog_run( GTK_DIALOG(dialog) );
		gtk_widget_destroy(dialog);
	}
	catch(error_obj error)
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE, error.msg );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Find intersection") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		catch(error_obj error)
		{
			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
			                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg);
			gtk_window_set_title( GTK_WINDOW(dialog), _("Find intersection") );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}
}

void picwin_calc_derivate( int32 x )
{
	utf8_string str, str2;
	math::CodeLine function;
	GtkWidget *dialog;

	char char_str[1000];

	try
	{
		str = "";
		floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat );
		str2 = "";
		floatx_to_string( ((floatx(trace_matrix.get_complex( 2, 1 ).real) -
		                    floatx(trace_matrix.get_complex( 1, 1 ).real))/2.0), str2, stdformat);
		snprintf( char_str, 1000, "x = nDeriv( y%ld, x, %s, %s )", old_best_match_i, str.c_str(), str2.c_str() );

		function.set_code_line( char_str );
		function.calc( 0 );

		str2 = "";
		math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex()->append_to_string( str2, format );

		snprintf( char_str, 1000, _("Derivate of y%ld at X=%s is\n%s"), old_best_match_i, str.c_str(), str2.c_str() );

		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
		                                 GTK_BUTTONS_OK, char_str );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate derivate") );
		gtk_dialog_run( GTK_DIALOG(dialog) );
		gtk_widget_destroy(dialog);
	}
	catch(error_obj error)
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE, error.msg );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate derivate") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		catch(error_obj error)
		{
			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
			                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg);
			gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate deriavte") );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}
}

void picwin_calc_integral( int32 x )
{
	static int32 first_x;
	static int first_old_best_match_i;
	utf8_string str, str2, str3;
	math::CodeLine function;
	GtkWidget *dialog;
	Complex tmp_value_x;
	int tmp_old_best_match_i;

	char char_str[1000];

	try
	{
		if( picwin_first_click_done )
		{
			picwin_first_click_done = false;

			if( first_old_best_match_i == old_best_match_i )
			{
				snprintf( char_str, 1000, "x = nInt( y%ld, x, ", old_best_match_i );
				str = char_str;
				floatx_to_string( get_x_value( first_x, old_best_match_i ), str, stdformat );
				str.append( ", " );
				floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat );
				snprintf( char_str, 1000, ", %ld )", abs(first_x-x)*10 );
				str.append( char_str );

				function.set_code_line( str.c_str() );
				function.calc( 0 );

				tmp_value_x = *math::global_varlist.get_pointer(
				                       math::global_varlist.get_id("x") )->get_complex();

				snprintf( char_str, 1000, "plot_picture = shade( plot_picture, y%ld, \"0\", x, ", old_best_match_i );
				str = char_str;
				floatx_to_string( get_x_value( first_x, old_best_match_i ), str, stdformat );
				str.append( ", " );
				floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat );
				snprintf( char_str, 1000, ", 0, [[ %f, %f, %f ]] )",
				          act_settings.func[old_best_match_i].color.red/255.0f,
				          act_settings.func[old_best_match_i].color.green/255.0f,
				          act_settings.func[old_best_match_i].color.blue/255.0f );
				str.append( char_str );
				function.set_code_line( str.c_str() );
				function.calc( 0 );
			}
			else
			{
				snprintf( char_str, 1000, "x = nInt( y%ld + \" - ( \" + y%ld + \" )\", x, ", first_old_best_match_i,
				          old_best_match_i );
				str = char_str;
				floatx_to_string( get_x_value( first_x, first_old_best_match_i ), str, stdformat );
				str.append( ", " );
				floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat );
				snprintf( char_str, 1000, ", %ld )", abs(first_x-x)*10 );
				str.append( char_str );

				function.set_code_line( str.c_str() );
				function.calc( 0 );

				tmp_value_x = *math::global_varlist.get_pointer(
				                       math::global_varlist.get_id("x") )->get_complex();

				snprintf( char_str, 1000, "plot_picture = shade( plot_picture, y%ld, y%ld, x, ", first_old_best_match_i,
				          old_best_match_i );
				str = char_str;
				floatx_to_string( get_x_value( first_x, first_old_best_match_i ), str, stdformat );
				str.append( ", " );
				floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat);
				snprintf( char_str, 1000, ", 0, [[ %f, %f, %f ]] )",
				          act_settings.func[first_old_best_match_i].color.red/255.0f,
				          act_settings.func[first_old_best_match_i].color.green/255.0f,
				          act_settings.func[first_old_best_match_i].color.blue/255.0f );
				str.append( char_str );
				function.set_code_line( str.c_str() );
				function.calc( 0 );
			}
			{
				int tmp = gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_trace) );

				tmp_old_best_match_i = old_best_match_i;     // saving this because it will be reseted to -1 when
				function.set_code_line("show plot_picture"); // redrawing the picture
				function.calc( 0 );

				gtk_widget_set_sensitive( picwin_combo_trace, TRUE );
				gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), tmp ); // auto-trace
				gtk_widget_set_sensitive( picwin_combo_action, TRUE );
				gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), LeftClickAction_CalcIntegral );

				picbild.mousedown_func = picwin_mousedown;
			}

			str = "";  str2 = "";  str3 = "";
			floatx_to_string( get_x_value( first_x, first_old_best_match_i ), str, stdformat );
			floatx_to_string( get_x_value( x, tmp_old_best_match_i ), str2, stdformat );
			tmp_value_x.append_to_string( str3, format );

			snprintf( char_str, 1000, _("Integral from %s to %s is\n%s"), str.c_str(), str2.c_str(), str3.c_str() );

			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
			                                 GTK_BUTTONS_OK, char_str );
			gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate integral") );
			gtk_dialog_run( GTK_DIALOG(dialog) );
			gtk_widget_destroy(dialog);
		}
		else
		{
			first_x = x;
			first_old_best_match_i = old_best_match_i;
			picwin_first_click_done = true;

			gettimeofday( &action_combo_changed, 0 );
			gtk_label_set_text( GTK_LABEL(picwin_status), _("Select end point") );
		}
	}
	catch(error_obj error)
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE, error.msg );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate integral") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		catch(error_obj error)
		{
			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
			                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg);
			gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate integral") );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}
}


void picwin_calc_length( int32 x )
{
	static int32 first_x;
	static int first_old_best_match_i;
	utf8_string str, str2, str3;
	math::CodeLine function;
	GtkWidget *dialog;
	Complex tmp_value_x;

	char char_str[1000];

	try
	{
		if( picwin_first_click_done )
		{
			picwin_first_click_done = false;

			if( first_old_best_match_i == old_best_match_i )
			{
				snprintf( char_str, 1000, "x = nInt( \"sqrt( 1.0 + nDeriv( y%ld, x, x, ", old_best_match_i );
				str = char_str;
				floatx_to_string( ((floatx(trace_matrix.get_complex( 2, 1 ).real) -
				                    floatx(trace_matrix.get_complex( 1, 1 ).real))/10.0), str, stdformat);
				str.append( " ) ^2 )\", x, " );
				floatx_to_string( get_x_value( first_x, old_best_match_i ), str, stdformat);
				str.append( ", " );
				floatx_to_string( get_x_value( x, old_best_match_i ), str, stdformat);
				snprintf( char_str, 1000, ", %ld )", abs(first_x-x)*10 );
				str.append( char_str );

				function.set_code_line( str.c_str() );
				function.calc( 0 );

				tmp_value_x = *math::global_varlist.get_pointer( math::global_varlist.get_id("x") )->get_complex();
			}
			else
			{
				error_obj error;
				snprintf( error.msg, ERROR_OBJ_MSG_LEN, _("Start and end point must both\nbe at the same function.") );
				throw error;
			}

			str = "";  str2 = "";  str3 = "";
			floatx_to_string( get_x_value( first_x, old_best_match_i ), str, stdformat);
			floatx_to_string( get_x_value( x, old_best_match_i ), str2, stdformat);
			tmp_value_x.append_to_string( str3, format );

			snprintf( char_str, 1000, _("The length of y%ld from %s to %s is\n%s"), old_best_match_i, str.c_str(),
			                                                                        str2.c_str(), str3.c_str() );

			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
							GTK_BUTTONS_OK, char_str );
			gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate length") );
			gtk_dialog_run( GTK_DIALOG(dialog) );
			gtk_widget_destroy(dialog);
		}
		else
		{
			first_x = x;
			first_old_best_match_i = old_best_match_i;
			picwin_first_click_done = true;

			gettimeofday( &action_combo_changed, 0 );
			gtk_label_set_text( GTK_LABEL(picwin_status), _("Select end point") );
		}
	}
	catch(error_obj error)
	{
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE, error.msg );
		gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate length") );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		catch(error_obj error)
		{
			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
			                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, error.msg);
			gtk_window_set_title( GTK_WINDOW(dialog), _("Calculate length") );
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}
}

void picwin_mousedown( const picture2gtk *src, int32 x, int32 y, int32 button )
{
	utf8_string str;
	floatx xcenter, ycenter, width, height;
	math::Variable tmpvar;
	char char_str[1000];

	if( old_best_match_i != -1 )
	{
		x = old_tx;
		y = old_ty;
	}

	switch(button)
	{
	case 1: // left button
		if( GTK_WIDGET_IS_SENSITIVE(picwin_combo_trace) == TRUE )
		{
			switch(gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_action) ))
			{
			case LeftClickAction_Zoom:

				xcenter = picbild2.get_x_value( x );
				ycenter = picbild2.get_y_value( y );
				width = picbild2.get_right() - picbild2.get_left();
				height = picbild2.get_top() - picbild2.get_bottom();
				width /= 4.0;
				height /= 4.0;

				try
				{
					floatx_to_string( xcenter-width, str, stdformat);
					act_settings.xmin = str.c_str();

					str = "";
					floatx_to_string( xcenter+width, str, stdformat);
					act_settings.xmax = str.c_str();

					str = "";
					floatx_to_string( ycenter-height, str, stdformat);
					act_settings.ymin = str.c_str();

					str = "";
					floatx_to_string( ycenter+height, str, stdformat);
					act_settings.ymax = str.c_str();

					{
						gint tmpint;
						tmpint = gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace));
						picturewin_show( (graph_settings *)(0) );
						gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), tmpint );
					}
				}
				catch(...) { }
				break;

			case LeftClickAction_FindRoot:
				{
					GtkWidget *dialog;
					Complex tmp_value_x,tmp_value_y;

					if( old_best_match_i == -1 )
						return;

					try
					{
						find_root( old_best_match_i, get_x_value( x, old_best_match_i ), tmp_value_x, tmp_value_y );

						str = "x = ";
						tmp_value_x.append_to_string( str, format );
						if( act_settings.show_code )
						{
							(*TEXTVIEW_FUNCTION)( str.c_str() );
							(*TEXTVIEW_FUNCTION)( "\n" );
						}
						place_marker( picbild2.get_float_x_pixel( floatx(tmp_value_x.real) ),
						              picbild2.get_float_y_pixel( floatx(tmp_value_y.real)),
						              act_settings.func[old_best_match_i].color );

						dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
						                                 GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
						                                 str.c_str() );
						gtk_window_set_title( GTK_WINDOW(dialog), _("Find root") );
						gtk_dialog_run( GTK_DIALOG(dialog) );
						gtk_widget_destroy( dialog );
					}
					catch(error_obj error)
					{
						dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
						                                 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
						                                 error.msg );
						gtk_window_set_title( GTK_WINDOW(dialog), _("Find root") );
						gtk_dialog_run( GTK_DIALOG(dialog) );
						gtk_widget_destroy( dialog );
					}
					catch(...)
					{
						try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
						catch(error_obj error)
						{
							dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin),
							                                 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
							                                 GTK_BUTTONS_CLOSE, error.msg );
							gtk_window_set_title( GTK_WINDOW(dialog), _("Find root") );
							gtk_dialog_run( GTK_DIALOG(dialog) );
							gtk_widget_destroy( dialog );
						}
					}
				}
				break;  // LeftClickAction_FindRoot

			case LeftClickAction_FindMaximumMinimum:
				{
					GtkWidget *dialog;
					Complex tmp_value_x,tmp_value_y;

					if( old_best_match_i == -1 )
						return;

					try
					{
						if( find_min_or_max( old_best_match_i, get_x_value( x, old_best_match_i ),
						                               tmp_value_x, tmp_value_y ) )
							str = _("Local minimum at");
						else
							str = _("Local maximum at");

						str.append( "\nx = " );
						tmp_value_x.append_to_string( str, format );

						str.append( "\ny = " );
						tmp_value_y.append_to_string( str, format );
						if(act_settings.show_code)
						{
							(*TEXTVIEW_FUNCTION)( str.c_str() );
							(*TEXTVIEW_FUNCTION)( "\n" );
						}
						place_marker( picbild2.get_float_x_pixel( floatx(tmp_value_x.real) ),
						              picbild2.get_float_y_pixel( floatx(tmp_value_y.real)),
						              act_settings.func[old_best_match_i].color );

						dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
						                                 GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
						                                 str.c_str() );
						gtk_window_set_title( GTK_WINDOW(dialog), _("Find maximum/minimum") );
						gtk_dialog_run( GTK_DIALOG(dialog) );
						gtk_widget_destroy(dialog);
					}
					catch(error_obj error)
					{
						dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin), GTK_DIALOG_MODAL,
						                                 GTK_MESSAGE_ERROR,
						                                 GTK_BUTTONS_CLOSE, error.msg );
						gtk_window_set_title( GTK_WINDOW(dialog), _("Find maximum/minimum") );
						gtk_dialog_run( GTK_DIALOG(dialog) );
						gtk_widget_destroy(dialog);
					}
					catch(...)
					{
						try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
						catch(error_obj error)
						{
							dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin),
							                                 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
							                                 GTK_BUTTONS_CLOSE, error.msg );
							gtk_window_set_title( GTK_WINDOW(dialog), _("Find maximum/minimum") );
							gtk_dialog_run( GTK_DIALOG(dialog) );
							gtk_widget_destroy(dialog);
						}
					}
				}
				break; // LeftClickAction_FindMaximumMinimum

			case LeftClickAction_FindIntersection:
				if( old_best_match_i == -1 )
					return;
				picwin_intersection_show( x );
				break;

			case LeftClickAction_CalcDerivate:
				if( old_best_match_i == -1 )
					return;
				picwin_calc_derivate( x );
				break;

			case LeftClickAction_CalcIntegral:
				if( old_best_match_i == -1 )
					return;
				picwin_calc_integral( x );
				break;

			case LeftClickAction_CalcLength:
				if( old_best_match_i == -1 )
					return;
				picwin_calc_length( x );
				break;
			}
		}
		break;

	case 2: // middle button
		if(GTK_WIDGET_IS_SENSITIVE(picwin_combo_trace) == TRUE)
		{
			switch( gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace)) )
			{
			case 0: // off
				gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), 1 );
				break;

			case 1: // auto
				if(old_best_match_i != -1)
					gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace),
					                   conv_function_index_to_combobox_index(old_best_match_i) );
				break;


			default: // locked
				gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), 1 );
			}
		}
		break;

	case 3: // right button
		if(GTK_WIDGET_IS_SENSITIVE(picwin_combo_trace) == TRUE)
		{
			xcenter = picbild2.get_x_value( x );
			ycenter = picbild2.get_y_value( y );
			width = picbild2.get_right() - picbild2.get_left();
			height = picbild2.get_top() - picbild2.get_bottom();

			try
			{
				str = "";
				floatx_to_string( xcenter-width, str, stdformat);
				act_settings.xmin = str.c_str();

				str = "";
				floatx_to_string( xcenter+width, str, stdformat);
				act_settings.xmax = str.c_str();

				str = "";
				floatx_to_string( ycenter-height, str, stdformat);
				act_settings.ymin = str.c_str();

				str = "";
				floatx_to_string( ycenter+height, str, stdformat);
				act_settings.ymax = str.c_str();

				{
					gint tmpint;
					tmpint = gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace));
					picturewin_show( (graph_settings *)(0) );
					gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), tmpint );
				}
			}
			catch(...) { }
		}
		break;
	}
}




void picwin_hline( guchar *buffer, int32 width, int32 height, int32 row, picture_h::colorrgb &color)
{
	if( row < 0  ||  row >= height  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	buffer += width*3*row;
	for(;width != 0;width--)
	{
		*buffer++ = guchar(color.red);
		*buffer++ = guchar(color.green);
		*buffer++ = guchar(color.blue);
	}
}
void picwin_vline( guchar *buffer, int32 width, int32 height, int32 col, picture_h::colorrgb &color)
{
	if( col < 0  ||  col >= width  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	int rowstride = 3*width - 2;
	buffer += 3*col;
	for(;height != 0;height--, buffer += rowstride)
	{
		*buffer++ = guchar(color.red);
		*buffer++ = guchar(color.green);
		*buffer = guchar(color.blue);
	}
}

void picwin_get_row( const guchar *buffer, int32 width, int32 height, int32 row, guchar *dest)
{
	if( row < 0  ||  row >= height  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	register int32 tmp = width*3;
	buffer += tmp*row;

	memcpy(dest, buffer, tmp);
}

void picwin_set_row( guchar *buffer, int32 width, int32 height, int32 row, const guchar *src)
{
	if( row < 0  ||  row >= height  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	register int32 tmp = width*3;
	buffer += tmp*row;

	memcpy(buffer, src, tmp);
}

void picwin_get_col( const guchar *buffer, int32 width, int32 height, int32 col, guchar *dest)
{
	if( col < 0  ||  col >= width  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	int rowstride = 3*width - 2;
	buffer += 3*col;
	for(;height != 0;height--, buffer += rowstride)
	{
		*dest++ = *buffer++;
		*dest++ = *buffer++;
		*dest++ = *buffer;
	}
}

void picwin_set_col( guchar *buffer, int32 width, int32 height, int32 col, const guchar *src)
{
	if( col < 0  ||  col >= width  ||  width <= 0  ||  height <= 0  ||  buffer == 0 )
		return;

	int rowstride = 3*width - 2;
	buffer += 3*col;
	for(;height != 0;height--, buffer += rowstride)
	{
		*buffer++ = *src++;
		*buffer++ = *src++;
		*buffer = *src++;
	}
}

void picwin_remove_old_trace_mark( void )
{
	if( old_best_match_i != -1 )
	{
		if( picwin_rowbuffer_len < picbild2.pic.get_width() )
		{
			delete [] picwin_rowbuffer;
			picwin_rowbuffer = 0;
			picwin_rowbuffer_len = 0;

			try{
				picwin_rowbuffer = new guchar [ picbild2.pic.get_width()*3 ];
				picwin_rowbuffer_len = picbild2.pic.get_width();
			}catch( ... ) {  return;  }
		}
		if( picwin_colbuffer_len < picbild2.pic.get_height() )
		{
			delete [] picwin_colbuffer;
			picwin_colbuffer = 0;
			picwin_colbuffer_len = 0;

			try{
				picwin_colbuffer = new guchar [ picbild2.pic.get_height()*3 ];
				picwin_colbuffer_len = picbild2.pic.get_height();
			}catch( ... ) {  return;  }
		}

		picwin_set_col( picbild.get_intern_buffer(), picbild2.pic.get_width(),
		                picbild2.pic.get_height(), old_tx, picwin_colbuffer );
		picwin_set_row( picbild.get_intern_buffer(), picbild2.pic.get_width(),
		                picbild2.pic.get_height(), old_ty, picwin_rowbuffer );

		picbild.force_expose_line( old_ty );
		picbild.force_expose_col( old_tx );

		old_best_match_i = -1;
	}
}

void picwin_mousemove( const picture2gtk *src, int32 x, int32 y )
{
	const uint32 limit_tiden = 3000000;
	uint32 tiden;
	{
		timeval tmp;
		gettimeofday( &tmp ,0 );

		tiden = 1000000*(tmp.tv_sec - action_combo_changed.tv_sec) + tmp.tv_usec - action_combo_changed.tv_usec;
	}

	picwin_remove_old_trace_mark();

	if( GTK_WIDGET_IS_SENSITIVE(picwin_combo_trace) == TRUE  &&
	    gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_trace) ) == 0 ) // no trace
		gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), 0 ); // zoom

	if( x < 0  ||  x >= picbild2.pic.get_width()  ||  y < 0  ||  y >= picbild2.pic.get_height() )
	{
		if( gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_action) ) < 1 )
			gtk_label_set_text( GTK_LABEL(picwin_status), PRG_NAME );
	}
	else
	{
		Complex tmp, best_match;
		floatx y_value;
		int i, best_match_i;
		static utf8_string str;
		Format tmp_format;

		char char_str[300];

		tmp_format = format;
		tmp_format.show_trailingzeros = true;

		if( gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace)) != 0 )
		{
			if( act_settings.active_page != 0  &&  act_settings.show_active_page_only )
				goto picwin_no_trace;

			if( gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace)) == 1 )
			{
				y_value = picbild2.get_y_value( y );
				best_match_i = -1;
				for(i=0;i<10;i++) // this loop finds first (if any) non error trace
				{
					if(act_settings.func[i].show_this)
					{
						best_match_i = i;
						best_match = trace_matrix.get_complex( x+1, i+2 );
						if( floatx(best_match.imaginary) == 0.0 )
							break;
					}
				}
				if(best_match_i == -1)
					goto picwin_no_trace;
				for( i++; i<10; i++ )
				{
					if(act_settings.func[i].show_this)
					{
						tmp = trace_matrix.get_complex( x+1, i+2 );
						if(floatx(tmp.imaginary) != 0.0) // skip this
							continue;
						if( fabsx(floatx(tmp.real) - y_value) < fabsx(floatx(best_match.real) - y_value) )
						{
							best_match_i = i;
							best_match = tmp;
						}
					}
				}
			}
			else // this is a locked trace
			{
				best_match_i = conv_combobox_index_to_function_index(
				                    gtk_combo_box_get_active(GTK_COMBO_BOX(picwin_combo_trace)) );
				best_match = trace_matrix.get_complex( x+1, best_match_i+2 );
			}

			if(floatx(best_match.imaginary) != 0.0)
			{
				if( gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_action) ) < 1  ||  tiden > limit_tiden )
				{
					str = "X=";
					floatx_to_string( get_x_value( x, best_match_i ), str, tmp_format );
					str.append( "  Y=" );
					str.append( _("Error") );
					gtk_label_set_text( GTK_LABEL(picwin_status), str.c_str() );
				}
			}
			else
			{
				if( gtk_combo_box_get_active( GTK_COMBO_BOX(picwin_combo_action) ) < 1  ||  tiden > limit_tiden )
				{
					SnapInfo snappen;

					snprintf( char_str, 300, "y%ld( ", best_match_i );
					str = char_str;

					snappen.calculated = false;
					floatx_to_string( get_x_value( x, best_match_i ), str, tmp_format );
					str.append( " ) = " );
					floatx_to_string( get_y_value( x, best_match_i, &snappen ), str, tmp_format );
					if( snappen.calculated )
					{
						str.append( " (" );
						switch( snappen.snap_type )
						{
						case SnapInfo::Snap_Root:
							str.append( _("Root") );
							break;
						case SnapInfo::Snap_Maximum:
							str.append( _("Maximum") );
							break;
						case SnapInfo::Snap_Minimum:
							str.append( _("Minimum") );
							break;
						case SnapInfo::Snap_Intersection:
							snprintf( char_str, 300, _("Intersection with y%ld"),
							          snappen.intersect_with );
							str.append( char_str );
							break;
						}
						str.append( ")" );
					}
					gtk_label_set_text( GTK_LABEL(picwin_status), str.c_str() );
				}

				try
				{
					int32 tx,ty;

					if( picwin_rowbuffer_len < picbild2.pic.get_width() )
					{
						delete [] picwin_rowbuffer;
						picwin_rowbuffer = 0;
						picwin_rowbuffer_len = 0;

						picwin_rowbuffer = new guchar [ picbild2.pic.get_width()*3 ];
						picwin_rowbuffer_len = picbild2.pic.get_width();
					}
					if( picwin_colbuffer_len < picbild2.pic.get_height() )
					{
						delete [] picwin_colbuffer;
						picwin_colbuffer = 0;
						picwin_colbuffer_len = 0;

						picwin_colbuffer = new guchar [ picbild2.pic.get_height()*3 ];
						picwin_colbuffer_len = picbild2.pic.get_height();
					}

					tx = picbild2.get_x_pixel( get_x_value( x, best_match_i ) );

					if( tx != old_tx  ||  best_match_i != old_best_match_i )
					{
						ty = picbild2.get_y_pixel( get_y_value( x, best_match_i ) );

						if( old_best_match_i != -1 )
						{
							picwin_set_col( picbild.get_intern_buffer(), picbild2.pic.get_width(),
							                picbild2.pic.get_height(), old_tx, picwin_colbuffer );
							picwin_set_row( picbild.get_intern_buffer(), picbild2.pic.get_width(),
							                picbild2.pic.get_height(), old_ty, picwin_rowbuffer );

							picbild.force_expose_line( old_ty );
							picbild.force_expose_col( old_tx );
						}
						picwin_get_row( picbild.get_intern_buffer(), picbild2.pic.get_width(),
						                picbild2.pic.get_height(), ty, picwin_rowbuffer );
						picwin_get_col( picbild.get_intern_buffer(), picbild2.pic.get_width(),
						                picbild2.pic.get_height(), tx, picwin_colbuffer );

						picwin_hline( picbild.get_intern_buffer(), picbild2.pic.get_width(),
						              picbild2.pic.get_height(), ty,
						              act_settings.func[best_match_i].color );
						picwin_vline( picbild.get_intern_buffer(), picbild2.pic.get_width(),
						              picbild2.pic.get_height(), tx,
						              act_settings.func[best_match_i].color );

						picbild.force_expose_line( ty );
						picbild.force_expose_col( tx );

						old_tx = tx;
						old_ty = ty;
						old_best_match_i = best_match_i;
					}
				}
				catch(error_obj error)
				{
					(*TEXTVIEW_FUNCTION)("\n\nError while updating trace: ");
					(*TEXTVIEW_FUNCTION)(error.msg);
					(*TEXTVIEW_FUNCTION)("\n\n");
				}
				catch(...)
				{
					try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
					catch(error_obj error)
					{
						(*TEXTVIEW_FUNCTION)("\n\nError while updating trace: ");
						(*TEXTVIEW_FUNCTION)(error.msg);
						(*TEXTVIEW_FUNCTION)("\n\n");
					}
				}
			}
		}
		else
		{
picwin_no_trace:
			bool trailingzeros = stdformat.show_trailingzeros;
			stdformat.show_trailingzeros = true;
			str = "X=";
			floatx_to_string( picbild2.get_x_value( x ), str, stdformat);
			str.append( "  Y=" );
			floatx_to_string( picbild2.get_y_value( y ), str, stdformat);
			gtk_label_set_text( GTK_LABEL(picwin_status), str.c_str() );
			stdformat.show_trailingzeros = trailingzeros;

			if( old_best_match_i != -1 )
			{
				if( picwin_rowbuffer_len < picbild2.pic.get_width() )
				{
					delete [] picwin_rowbuffer;
					picwin_rowbuffer = 0;
					picwin_rowbuffer_len = 0;

					picwin_rowbuffer = new guchar [ picbild2.pic.get_width()*3 ];
					picwin_rowbuffer_len = picbild2.pic.get_width();
				}
				if( picwin_colbuffer_len < picbild2.pic.get_height() )
				{
					delete [] picwin_colbuffer;
					picwin_colbuffer = 0;
					picwin_colbuffer_len = 0;

					picwin_colbuffer = new guchar [ picbild2.pic.get_height()*3 ];
					picwin_colbuffer_len = picbild2.pic.get_height();
				}

				picwin_set_col( picbild.get_intern_buffer(), picbild2.pic.get_width(),
				                picbild2.pic.get_height(), old_tx, picwin_colbuffer );
				picwin_set_row( picbild.get_intern_buffer(), picbild2.pic.get_width(),
				                picbild2.pic.get_height(), old_ty, picwin_rowbuffer );

				picbild.force_expose_line( old_ty );
				picbild.force_expose_col( old_tx );

				old_best_match_i = -1;
			}
		}
	}
}

void picwin_mousenop(const picture2gtk *src,int32 x,int32 y,int32 button) {  }

void picturewin_save_as(void)
{
	GtkWidget *file_chooser, *tmp, *tmp2, *hbox, *picwin_save_format;
	GtkFileFilter *filter;

	file_chooser = gtk_file_chooser_dialog_new( dialog_title_save_as,
	                                            GTK_WINDOW(picturewin),
	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                            GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
	                                            NULL );


	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.pcx");
	gtk_file_filter_add_pattern(filter, "*.ppm");
	gtk_file_filter_set_name(filter, filter_picture_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.pcx");
	gtk_file_filter_set_name(filter, filter_pcx_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.ppm");
	gtk_file_filter_set_name(filter, filter_ppm_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_filter_set_name(filter, filter_all_files);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(file_chooser), filter);

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show( hbox );

	picwin_save_format = gtk_option_menu_new();
	gtk_widget_show( picwin_save_format );

	tmp = gtk_menu_new();
	gtk_widget_show( tmp );

	tmp2 = gtk_menu_item_new_with_label( _("PCX") );
	gtk_widget_show( tmp2 );
	gtk_menu_shell_append(GTK_MENU_SHELL(tmp),tmp2);

	tmp2 = gtk_menu_item_new_with_label( _("PPM (binary)") );
	gtk_widget_show( tmp2 );
	gtk_menu_shell_append(GTK_MENU_SHELL(tmp),tmp2);

	tmp2 = gtk_menu_item_new_with_label( _("PPM (text/ascii)") );
	gtk_widget_show( tmp2 );
	gtk_menu_shell_append(GTK_MENU_SHELL(tmp),tmp2);

	gtk_option_menu_set_menu(GTK_OPTION_MENU(picwin_save_format), tmp);

	gtk_box_pack_end(GTK_BOX(hbox), picwin_save_format, FALSE, FALSE, 0);


	tmp = gtk_label_new( _("File type:") );
	gtk_widget_show( tmp );
	gtk_box_pack_end(GTK_BOX(hbox), tmp, FALSE, FALSE, 0);

	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(file_chooser), hbox);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		gint type;
		char *filename;

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		type = gtk_option_menu_get_history(GTK_OPTION_MENU(picwin_save_format));

		try
		{
			switch(type)
			{
			case 0: // PCX
				picbild2.pic.save_pcx(filename);
				break;
			case 1: // PPM (binary)
				picbild2.pic.save_ppm(filename);
				break;
			case 2: // PPM (ascii)
				picbild2.pic.save_ppm(filename,1);
				break;
			default:
				THROW_ERROR( ErrorType_General, _("Unknown format") );
			}
		}
		catch(error_obj error)
		{
			GtkWidget *dialog;

			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT,
			                                 GTK_MESSAGE_ERROR,
			                                 GTK_BUTTONS_CLOSE,
			                                 error.msg );
			gtk_dialog_run( GTK_DIALOG(dialog) );
			gtk_widget_destroy(dialog);
		}
		g_free(filename);
	}
	gtk_widget_destroy(file_chooser);
}

void find_all_snap_points( void )
{
	SnapInfo tmp_snap;
	Complex tmp0[10], tmp1[10], tmp2[10];
	floatx f0, f1, f2;
	bool active[10];

	tmp_snap.calculated = false;

	for( int32 x=2; x<act_settings.width; x++ )
	{
/*
struct SnapInfo
{
	enum SnapType { Snap_Root, Snap_Minimum, Snap_Maximum, Snap_Intersection };
	SnapType snap_type;
	int32 x, intersect_with;
	floatx fx, fy;
	bool calculated;
};
*/
		for( int i=0; i<10; i++ )
		{
			if( act_settings.func[i].show_this == false )
				continue;
			tmp0[i] = trace_matrix.get_complex( x-1, i+2 );
			tmp1[i] = trace_matrix.get_complex( x,   i+2 );
			tmp2[i] = trace_matrix.get_complex( x+1, i+2 );
		}

		for( int i=0; i<10; i++ )
		{
			if( act_settings.func[i].show_this == false )
				continue;
			for( int a=i; a<10; a++ )
			{
				if( act_settings.func[a].show_this == false )
				{
					active[a] = false;
					continue;
				}
				active[a] = bool( floatx(tmp0[a].imaginary) == 0.0  &&  floatx(tmp1[a].imaginary) == 0.0  &&
				                  floatx(tmp2[a].imaginary) == 0.0 );
			}
			if( active[i] )
			{
				f0 = floatx(tmp0[i].real);
				f1 = floatx(tmp1[i].real);
				f2 = floatx(tmp2[i].real);
				if( f1 * f2 < 0.0 ) // they have different signs, here might a root be found
				{
					tmp_snap.snap_type = SnapInfo::Snap_Root;
					tmp_snap.x = x;
					tmp_snap.fx = floatx( trace_matrix.get_complex( x, 1 ).real );
					snap_info[i].append( tmp_snap );
				}
				if( f0 <= f1  &&  f2 < f1 )
				{
					tmp_snap.snap_type = SnapInfo::Snap_Maximum;
					tmp_snap.x = x;
					tmp_snap.fx = floatx( trace_matrix.get_complex( x, 1 ).real );
					snap_info[i].append( tmp_snap );
				}
				if( f0 >= f1  &&  f2 > f1 )
				{
					tmp_snap.snap_type = SnapInfo::Snap_Minimum;
					tmp_snap.x = x;
					tmp_snap.fx = floatx( trace_matrix.get_complex( x, 1 ).real );
					snap_info[i].append( tmp_snap );
				}
				for( int a=i; a<10; a++ )
				{
					if( active[a] == false )
						continue;
					if( (f1 < floatx(tmp1[a].real)) ^ (f2 < floatx(tmp2[a].real)) )
					{
						tmp_snap.snap_type = SnapInfo::Snap_Intersection;
						tmp_snap.x = x;
						tmp_snap.fx = floatx( trace_matrix.get_complex( x, 1 ).real );
						tmp_snap.intersect_with = a;
						snap_info[i].append( tmp_snap );
						tmp_snap.intersect_with = i;
						snap_info[a].append( tmp_snap );
					}
				}
			}
		}
	}
}


void picturewin_show( const graph_settings *settings )
{
	enum IM_HERE { IM_HERE_picture, IM_HERE_funcion, IM_HERE_polar, IM_HERE_par };
	math::CodeLine function;
	utf8_string str;
	picture_h::colorrgb color;
	const Matrix *matr;
	IM_HERE Im_here = IM_HERE_picture;
	int i;
	char char_str[1000];

	old_best_match_i = -1;

	try
	{
		for( i=0; i<10; i++ )
			snap_info[i].clear();

		if(settings != 0)
			act_settings = *settings;

		snprintf( char_str, 1000, "plot_picture = picture( %ld, %ld, ", act_settings.width, act_settings.height );
		str = char_str;
		str.append( act_settings.xmin ).append( ", " );
		str.append( act_settings.xmax ).append( ", " );
		str.append( act_settings.ymin ).append( ", " );
		str.append( act_settings.ymax );
		snprintf( char_str, 1000, ", [[ %f, %f, %f ]] )\n", act_settings.bgcolor.red/255.0f,
		          act_settings.bgcolor.green/255.0f, act_settings.bgcolor.blue/255.0f );
		str.append( char_str );
		if(act_settings.show_code)
			(*TEXTVIEW_FUNCTION)( str.c_str() );
		function.set_code_line( str.c_str() );
		function.calc( 0 );  // resizing the picture

		if( act_settings.grid_on )
		{
			snprintf( char_str, 1000, "plot_picture.grid( \"%s\", %s, %s, %s, [[ %f, %f, %f ]] )\n",
			          act_settings.grid_type.c_str(), act_settings.grid_scale1.c_str(),
			          act_settings.grid_scale2.c_str(), (act_settings.use_antialiasing) ? "true" : "false",
			          act_settings.grid_color.red/255.0f, act_settings.grid_color.green/255.0f,
			          act_settings.grid_color.blue/255.0f );
			if(act_settings.show_code)
				(*TEXTVIEW_FUNCTION)( char_str );
			function.set_code_line( char_str );
			function.calc( 0 );  // adding grid
		}

		if( act_settings.axes_on )
		{
			snprintf( char_str, 1000, "plot_picture.axes( %s, %s, %ld, [[ %f, %f, %f ]] )\n",
			          act_settings.scalex.c_str(), act_settings.scaley.c_str(),
			          act_settings.scale_length, act_settings.axes_color.red/255.0f,
			          act_settings.axes_color.green/255.0f, act_settings.axes_color.blue/255.0f );
			if(act_settings.show_code)
				(*TEXTVIEW_FUNCTION)( char_str );
			function.set_code_line( char_str );
			function.calc( 0 );  // adding axes
		}

		if(act_settings.active_page == 0  ||  act_settings.show_active_page_only == false) // function-page
		{
			Im_here = IM_HERE_funcion;
			for( i=0; i<10; i++ ) // setting up all functions
			{
				snprintf( char_str, 1000, "y%ld = \"", i );
				str = char_str;
				str.append( act_settings.func[i].entry ).append( "\"\n" );
				if( act_settings.show_code )
					(*TEXTVIEW_FUNCTION)( str.c_str() );
				function.set_code_line( str.c_str() );
				function.calc( 0 );
			}
			str = "plot_x_matrix = table( \"x\", x, dec( ";
			str.append( act_settings.xmin ).append( " ), dec( " );
			str.append( act_settings.xmax );
			snprintf( char_str, 1000, " ), %ld )\n", act_settings.width );
			str.append( char_str );
			if( act_settings.show_code )
				(*TEXTVIEW_FUNCTION)( str.c_str() );
			function.set_code_line( str.c_str() );
			function.calc( 0 ); // calculating the x list

			trace_matrix.set_size( act_settings.width, 11 );
			trace_matrix.fill( Complex( 0.0, 1.0 ) );

			matr = math::global_varlist.get_pointer( math::global_varlist.get_id("plot_x_matrix") )->get_matrix();
			for( int i2=0; i2<act_settings.width; i2++ )
				trace_matrix.set_complex(i2+1,1, matr->get_complex(i2+1,1) );

			for( i=0; i<10; i++ )
			{
				color = act_settings.func[i].color;
				if( act_settings.func[i].show_this )
				{
					snprintf( char_str, 1000, "plot_y_matrix = table( y%ld, x, dec( ", i );
					str = char_str;
					str.append( act_settings.xmin ).append( " ), dec( " );
					str.append( act_settings.xmax );
					snprintf( char_str, 1000, " ), %ld, i )\n", act_settings.width );
					str.append( char_str );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // calculating the y list

					matr = math::global_varlist.get_pointer(
					                  math::global_varlist.get_id("plot_y_matrix") )->get_matrix();
					for( int i2=0; i2<act_settings.width; i2++ )
						trace_matrix.set_complex(i2+1,i+2, matr->get_complex(i2+1,1) );

					str = "plot_picture.plot( plot_x_matrix, plot_y_matrix, \"";
					str.append( act_settings.plot_mode );
					snprintf( char_str, 1000, "\", \"Dot\", 0, false, %s, [[ %f, %f, %f ]] )\n",
					          (act_settings.use_antialiasing) ? "true" : "false",
					          color.red/255.0f, color.green/255.0f, color.blue/255.0f );
					str.append( char_str );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // drawing function
				}
			}
			find_all_snap_points();
		}
		if( act_settings.active_page == 1  ||  act_settings.show_active_page_only == false ) // polar-page
		{
			Im_here = IM_HERE_polar;
			bool connected = true;
			if( act_settings.plot_mode == "Dots" )
				connected = false;
			for( i=0; i<10; i++ ) // setting up all functions
			{
				snprintf( char_str, 1000, "r%ld = \"", i );
				str = char_str;
				str.append( act_settings.polar[i].entry ).append( "\"\n" );
				if( act_settings.show_code )
					(*TEXTVIEW_FUNCTION)( str.c_str() );
				function.set_code_line( str.c_str() );
				function.calc( 0 );
			}
			for( i=0; i<10; i++ )
			{
				color = act_settings.polar[i].color;
				if( act_settings.polar[i].show_this )
				{
					snprintf( char_str, 1000, "plot_x_matrix = table( \"r%ld(x)cos x\", x, ", i );
					str = char_str;
					str.append( act_settings.polar_start ).append( ", " );
					str.append( act_settings.polar_stop ).append( ", " );
					str.append( act_settings.polar_steps ).append( ", i )\n" );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // calculating the x list

					snprintf( char_str, 1000, "plot_y_matrix = table( \"r%ld(x)sin x\", x, ", i );
					str = char_str;
					str.append( act_settings.polar_start ).append( ", " );
					str.append( act_settings.polar_stop ).append( ", " );
					str.append( act_settings.polar_steps ).append( ", i )\n" );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // calculating the x list

					snprintf( char_str, 1000, "plot_picture.plot( plot_x_matrix, plot_y_matrix, \"%s\", "
					                          "\"Dot\", 0, false, %s, [[ %f, %f, %f ]] )\n",
					          connected ? "Connected" : "Dots",
					          act_settings.use_antialiasing ? "true" : "false",
					          color.red/255.0f, color.green/255.0f, color.blue/255.0f );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( char_str );
					function.set_code_line( char_str );
					function.calc( 0 );  // drawing function
				}
			}
		}
		if( act_settings.active_page == 2  ||  act_settings.show_active_page_only == false ) // parameter-page
		{
			Im_here = IM_HERE_par;
			bool connected = (act_settings.plot_mode == "Dots") ? false : true;

			for( i=0; i<5; i++ ) // setting up all functions
			{
				snprintf( char_str, 1000, "xf%ld = \"", i );
				str = char_str;
				str.append( act_settings.par[i].entry ).append( "\"\n" );
				if( act_settings.show_code )
					(*TEXTVIEW_FUNCTION)( str.c_str() );
				function.set_code_line( str.c_str() );
				function.calc( 0 );

				snprintf( char_str, 1000, "yf%ld = \"", i );
				str = char_str;
				str.append( act_settings.par[i].entry2 ).append( "\"\n" );
				if( act_settings.show_code )
					(*TEXTVIEW_FUNCTION)( str.c_str() );
				function.set_code_line( str.c_str() );
				function.calc( 0 );
			}
			for( i=0; i<5; i++ )
			{
				color = act_settings.par[i].color;
				if( act_settings.par[i].show_this )
				{
					snprintf( char_str, 1000, "plot_x_matrix = table( xf%ld, x, ", i );
					str = char_str;
					str.append( act_settings.par_start ).append( ", " );
					str.append( act_settings.par_stop ).append( ", " );
					str.append( act_settings.par_steps ).append( ", i )\n" );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // calculating the x list

					snprintf( char_str, 1000, "plot_y_matrix = table( yf%ld, x, ", i );
					str = char_str;
					str.append( act_settings.par_start ).append( ", " );
					str.append( act_settings.par_stop ).append( ", " );
					str.append( act_settings.par_steps ).append( ", i )\n" );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( str.c_str() );
					function.set_code_line( str.c_str() );
					function.calc( 0 );  // calculating the y list

					snprintf( char_str, 1000, "plot_picture.plot( plot_x_matrix, plot_y_matrix, \"%s\""
					                          ", \"Dot\", 0, false, %s, [[ %f, %f, %f ]] )\n",
					          connected ? "Connected" : "Dots",
					          act_settings.use_antialiasing ? "true" : "false",
					          color.red/255.0f, color.green/255.0f, color.blue/255.0f );
					if( act_settings.show_code )
						(*TEXTVIEW_FUNCTION)( char_str );
					function.set_code_line( char_str );
					function.calc( 0 );  // drawing parameter function
				}
			}
		}
		if( act_settings.show_code )
			(*TEXTVIEW_FUNCTION)("show plot_picture\n\n");
		function.set_code_line("show plot_picture");
		function.calc( 0 );

		gtk_widget_set_sensitive( picwin_combo_trace, TRUE );
		gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), 1 ); // auto-trace
		gtk_widget_set_sensitive( picwin_combo_action, TRUE );
		gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), 0 ); // zoom
		old_best_match_i = -1;

		picbild.mousedown_func = picwin_mousedown;
	}
	catch(error_obj error)
	{
		GtkWidget *dialog;
		utf8_string str;
		char tkn[2];
		tkn[1] = '\0';

		switch( Im_here )
		{
		case IM_HERE_picture:
			str = "When creating plot-picture: ";
			break;
		case IM_HERE_funcion:
			str = "When plotting y";
			tkn[0] = char(i)+'0';
			str.append( tkn );
			str.append( ": " );
			break;
		case IM_HERE_polar:
			str = "When plotting r";
			tkn[0] = char(i)+'0';
			str.append( tkn );
			str.append( ": " );
			break;
		case IM_HERE_par:
			str = "When plotting xf";
			tkn[0] = char(i)+'0';
			str.append( tkn );
			str.append( " yf" );
			str.append( tkn );
			str.append( ": " );
			break;
		}

		str.append( error.msg );
		dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin),
		                                 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE, str.c_str() );
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
	}
	catch(...)
	{
		try{ THROW_ERROR( ErrorType_Memory, _("Couldn't get memory.") ); }
		catch(error_obj error)
		{
			GtkWidget *dialog;

			dialog = gtk_message_dialog_new( GTK_WINDOW(picturewin),
			                                 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
			                                 GTK_BUTTONS_CLOSE, error.msg );
			gtk_dialog_run( GTK_DIALOG(dialog) );
			gtk_widget_destroy(dialog);
		}
	}
}

void picwin_combo_action_changed( GtkComboBox *widget, gpointer user_data )
{
	gettimeofday( &action_combo_changed, 0 );

	switch( gtk_combo_box_get_active( widget ) )
	{
	case LeftClickAction_Zoom:
		gtk_label_set_text( GTK_LABEL(picwin_status), PRG_NAME );
		break;

	case LeftClickAction_FindRoot:
	case LeftClickAction_FindMaximumMinimum:
	case LeftClickAction_FindIntersection:
		gtk_label_set_text( GTK_LABEL(picwin_status), _("Make a guess") );
		break;

	case LeftClickAction_CalcDerivate:
		gtk_label_set_text( GTK_LABEL(picwin_status), _("Select point") );
		break;
	case LeftClickAction_CalcIntegral:
	case LeftClickAction_CalcLength:
		gtk_label_set_text( GTK_LABEL(picwin_status), _("Select start point") );
		break;
	}
	picwin_first_click_done = false;
}

GtkWidget* picturewin_build_menu(void)
{
	GtkActionGroup *actions;
	GtkUIManager *ui_manager;
	GError *error = NULL;

	static const gchar *menu =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"  </menubar>"
"</ui>";
	GtkActionEntry entries[] = {
{ "FileMenu", NULL,              menu_file },
{ "SaveAs",   GTK_STOCK_SAVE_AS, menu_file_save_as, "<shift><ctrl>S", NULL, G_CALLBACK(picturewin_save_as) },
{ "Close",    GTK_STOCK_CLOSE,   menu_file_close,   "<ctrl>W",        NULL, G_CALLBACK(picturewin_close) },
{ "Quit",     GTK_STOCK_QUIT,    menu_file_quit,    "<ctrl>Q",        NULL, G_CALLBACK(UMP_gtk_quit) }
};
	guint n_entries = G_N_ELEMENTS(entries);

	actions = gtk_action_group_new("Actions");
	gtk_action_group_add_actions(actions, entries, n_entries, NULL);
	ui_manager = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui_manager, actions, 0);
	gtk_window_add_accel_group( GTK_WINDOW(picturewin), gtk_ui_manager_get_accel_group(ui_manager) );
	if (!gtk_ui_manager_add_ui_from_string(ui_manager, menu, -1, &error))
	{
		g_message("building menus failed: %s", error->message);
		g_error_free(error);
		return 0;
	}
	gtk_ui_manager_ensure_update(ui_manager);


	return gtk_ui_manager_get_widget(ui_manager, "/MenuBar");
}

void picturewin_show( const char *title, const math::Picture &src )
{
	static int combobox_trace_last_nrofentries;
	GtkWidget *drawarea,*scrolled;

	old_best_match_i = -1;

	if(picturewin == 0)
	{
		picturewin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(picturewin), title);
		g_signal_connect_swapped( GTK_OBJECT(picturewin), "delete-event",
		                          G_CALLBACK(picturewin_close), NULL);

		picturewin_vbox = gtk_vbox_new(FALSE,0);

		if((picturewin_menu = picturewin_build_menu()) == 0)
		{
			gtk_widget_destroy( picturewin );
			picturewin = 0;
			return;
		}
		gtk_box_pack_start(GTK_BOX(picturewin_vbox), picturewin_menu, FALSE, FALSE, 0);

		{
			GtkWidget *tbox;
			utf8_string str;

			tbox = gtk_hbox_new(FALSE,2);

			picwin_label_trace = gtk_label_new( _("Trace action") );
			gtk_box_pack_start(GTK_BOX(tbox), picwin_label_trace, FALSE, FALSE, 2 );

			picwin_combo_trace = gtk_combo_box_new_text();
			gtk_box_pack_start(GTK_BOX(tbox), picwin_combo_trace, FALSE, FALSE, 2 );
			gtk_label_set_mnemonic_widget( GTK_LABEL(picwin_label_trace), picwin_combo_trace );

			gtk_widget_set_sensitive(picwin_combo_trace, FALSE);


			picwin_combo_action = gtk_combo_box_new_text();
			gtk_box_pack_end(GTK_BOX(tbox), picwin_combo_action, FALSE, FALSE, 2 );

			combobox_trace_last_nrofentries = picwin_reload_combos( 0 );

			gtk_widget_set_sensitive(picwin_combo_action, FALSE);
			gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), -1 );
			g_signal_connect( GTK_OBJECT(picwin_combo_action), "changed",
			                  G_CALLBACK(picwin_combo_action_changed), gpointer(NULL) );

			gtk_box_pack_start(GTK_BOX(picturewin_vbox), tbox, FALSE, FALSE, 2 );
		}

		drawarea = gtk_drawing_area_new();
		picbild.connect_to_drawing_area( drawarea );

		scrolled = gtk_scrolled_window_new( NULL, NULL );
		gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC,
		                                GTK_POLICY_AUTOMATIC );

		gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW(scrolled), drawarea );

		gtk_box_pack_start( GTK_BOX(picturewin_vbox), scrolled, TRUE, TRUE, 0 );

		{
			GtkWidget *tbox, *tbar;
			tbox = gtk_hbox_new( FALSE, 0 );

			picwin_status = gtk_label_new( PRG_NAME );
			gtk_misc_set_alignment( GTK_MISC(picwin_status), 0.0f, 0.5f );
			gtk_box_pack_start( GTK_BOX(tbox), picwin_status, FALSE, FALSE, 0 );

			tbar = gtk_statusbar_new();
			gtk_box_pack_start( GTK_BOX(tbox), tbar, TRUE, TRUE, 0 );

			gtk_box_pack_start( GTK_BOX(picturewin_vbox), tbox, FALSE, FALSE, 0 );
		}

		gtk_container_add( GTK_CONTAINER(picturewin), picturewin_vbox );

		gtk_window_resize( GTK_WINDOW(picturewin), src.pic.get_width() + 10,
		                   src.pic.get_height() + 90 );

		gtk_widget_show_all(picturewin);
	}
	else
	{
		gtk_window_set_title( GTK_WINDOW(picturewin), title );
		gtk_window_resize( GTK_WINDOW(picturewin), src.pic.get_width() + 10,
		                   src.pic.get_height() + 90 );

		gtk_window_present( GTK_WINDOW(picturewin) );

		gtk_widget_set_sensitive(picwin_combo_trace, FALSE);
		gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_trace), 0 ); // trace off

		gtk_widget_set_sensitive(picwin_combo_action, FALSE);
		gtk_combo_box_set_active( GTK_COMBO_BOX(picwin_combo_action), -1 ); // no action

		combobox_trace_last_nrofentries = picwin_reload_combos( combobox_trace_last_nrofentries );
	}

	picbild2 = src;
	picbild.update_from( src.pic );

	picbild.mousemove_func = picwin_mousemove;
	picbild.mousedown_func = picwin_mousenop;
	picbild.mouseup_func = picwin_mousenop;
}


void picturewin_translate(void)
{
	if(picturewin != 0)
	{
		GtkWidget *new_menu;

		gtk_label_set( GTK_LABEL(picwin_label_trace), _("Trace action") );


		if(( new_menu = picturewin_build_menu()) == 0 )
			return;

		gtk_widget_destroy( picturewin_menu );
		picturewin_menu = new_menu;

		gtk_box_pack_start( GTK_BOX(picturewin_vbox), picturewin_menu, FALSE, FALSE, 0 );
		gtk_box_reorder_child( GTK_BOX(picturewin_vbox), picturewin_menu, 0 );

		picwin_reload_combos( 12 );
	}
}

void picturewin_close(void)
{
	if(picturewin != 0)
	{
		gtk_widget_destroy(picturewin);
		picturewin = 0;
	}
}
