
/*
 *  utf8_string - Implements a string-class which handles utf8 coded strings
 *  Copyright (c) 2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */


#ifndef UTF8_STRING_H_
#define UTF8_STRING_H_

#include "vartypes.h"


#define UTF8_STRING_H_VERSION "v1"
#define UTF8_STRING_H_DATE    "2006-07 - 2006-10"


class utf8_string
{
private:
	char *str;
	uint32 size;

	// these sizes are not in characters but in bytes

	// set_size and enlarge_to uses bytes...
	// the string is erased and size is set to new_size
	void set_size( uint32 new_size ) throw(error_obj);
	// the string is preserved and size is set to the biggest of size and new_size
	void enlarge_to( uint32 new_size ) throw(error_obj);

public:
	utf8_string();
	utf8_string( const utf8_string &src ) throw(error_obj);
	utf8_string( const char *src ) throw(error_obj);
	~utf8_string();

	utf8_string & clear(void) throw();

	// if pos is greater than the string's length nothing happens
	inline utf8_string & crop( uint32 pos ) { return remove( pos, uint32_max ); }

	// returns length in characters (not in bytes)
	uint32 get_length(void) const;

	utf8_string & insert( const utf8_string &src, uint32 pos ) throw(error_obj);
	inline utf8_string & prepend( const utf8_string &src ) throw(error_obj) { return insert( src, 0 ); }
	utf8_string & append( const utf8_string &src ) throw(error_obj);
	utf8_string & remove( uint32 pos, uint32 len = 1 );

	inline utf8_string & replace( uint32 pos, uint32 len, const utf8_string &src ) throw(error_obj)
		{
			remove( pos, len );
			return insert( src, pos );
		}

	void operator=( const utf8_string &src ) throw(error_obj);

	void assign( const utf8_string &src, uint32 pos, uint32 length ) throw(error_obj);

	bool operator==( const utf8_string &src ) const;
	inline bool operator!=( const utf8_string &src ) const { return !(*this == src); }

	bool operator==( const char *src ) const;
	inline bool operator!=( const char *src ) const { return !(*this == src); }


// just think of that the index in a C-string might not be the same as in utf8_string
	inline const char * c_str(void) const { return str; }
	const char * c_str_from( uint32 index ) const;

	// the character at pos will be tested for equality with all characters in test
	bool test_character( uint32 pos, const char *test ) const;

	// if the character at pos isn't a digit then an error is thrown else the digit is returned
	int32 get_digit( uint32 pos ) const throw(error_obj);

	utf8_string substr( uint32 start, uint32 length ) const throw(error_obj);
	
	void remove_escape_sequences(void);
};


#endif // UTF8_STRING_H_
