
/*
 *  variable typedef's and error messages
 *  Copyright (c) 2002-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*
News
----

v9  2006-07-27 2006-10-05
--

	Extracted keyfile_io to it's own file
	Removed all Throw_... functions
	Added function uint64_to_char

v8  2006-06-01
--

	Implemented keyfile_io::operator=
	Rewrote keyfile_io::write to use fopen instead of std::ofstream

v7  2006-02-25
--

	keyfile_io now supports lines with upto 5000 bytes (up from 500)

v6  2005-09-25 - 2005-11-06
--

	Added support for the escape sequence \"
	Moved ump_float from math2.h into this file
	Renamed error_lang -> translation
	Renamed ump_float -> floatx (x symbolice that the precision is choosed at compile time)

v1.1.4  2005-07-30
------

	Added an implementation of trunc using floor optional by -DDONT_HAVE_TRUNC

v1.1.3  2005-01-10
------

	added more standard functions for translatable error message
	added THROW_MACRO_FIX

v1.1.2  2004-10-17 - 2004-12-23
------

	added int64_max, int64_min, uin64_max, uint64_min
	small fixes
	added more standard functions for translatable error message
	fixed some compiler warnings

v1.1.1  2004-07-14 - 2004-09-01
------

	added more standard functions for translatable error message
	removed the Thrower field in error_obj
	added support to replace escape sequences in keyfile_io
	  known escape sequences are \\ \n \r \t
	added keyfile_io::write

v1.1.0  2004-06-30 - 2004-07-03
------

	merged keyfile_io.h and vartypes.h
	added some standard functions to ease up the work with translated error message

*/

#ifndef VARTYPES_H_
#define VARTYPES_H_

#define VARTYPES_H_VERSION "v9"
#define VARTYPES_H_DATE    "2002-12 - 2006-08"

#include <math.h>


typedef unsigned char uint8;
typedef   signed char  int8;
const uint8 uint8_max = 0xff;
const uint8 uint8_min = 0x00;
const  int8  int8_max = 0x7f;
const  int8  int8_min = 0x80;

typedef unsigned short uint16;
typedef   signed short  int16;
const uint16 uint16_max = 0xffff;
const uint16 uint16_min = 0x0000;
const  int16  int16_max = 0x7fff;
const  int16  int16_min = 0x8000;

typedef unsigned int uint32;
typedef   signed int  int32;
const uint32 uint32_max = 0xffffffff;
const uint32 uint32_min = 0x00000000;
const  int32  int32_max = 0x7fffffff;
const  int32  int32_min = 0x80000000;

typedef unsigned long long int uint64;
typedef   signed long long int  int64;
const uint64 uint64_max = uint64(0) - 1;
const uint64 uint64_min = 0;
const  int64  int64_max = int64(uint64_max >> 1);
const  int64  int64_min = int64_max + 1;


// str must be big enough, and if val doesn't contain enough
// digits it will be zero padded from the left
void uint64_to_char( uint64 val, char *str, uint32 min_digits=0 );


#ifdef DONT_HAVE_TRUNC
double trunc(double value);
#endif
#ifdef DONT_HAVE_LOG2
double log2(double value);
#endif


#ifdef USE_FLOAT
	typedef float floatx;

	inline floatx sinx(floatx val) { return sinf(val); }
	inline floatx cosx(floatx val) { return cosf(val); }
	inline floatx tanx(floatx val) { return tanf(val); }
	inline floatx asinx(floatx val) { return asinf(val); }
	inline floatx acosx(floatx val) { return acosf(val); }
	inline floatx atanx(floatx val) { return atanf(val); }
	inline floatx sinhx(floatx val) { return sinhf(val); }
	inline floatx coshx(floatx val) { return coshf(val); }
	inline floatx tanhx(floatx val) { return tanhf(val); }
	inline floatx asinhx(floatx val) { return asinhf(val); }
	inline floatx acoshx(floatx val) { return acoshf(val); }
	inline floatx atanhx(floatx val) { return atanhf(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2f(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypotf(v1,v2); }
	inline floatx truncx(floatx val) { return truncf(val); }
	inline floatx fabsx(floatx val) { return fabsf(val); }
	inline floatx logx(floatx val) { return logf(val); }
	inline floatx log2x(floatx val) { return log2f(val); }
	inline floatx log10x(floatx val) { return log10f(val); }
	inline floatx expx(floatx val) { return expf(val); }
	inline floatx sqrtx(floatx val) { return sqrtf(val); }
	inline floatx powx(floatx v1,floatx v2) { return powf(v1,v2); }
#endif
#ifdef USE_DOUBLE
	typedef double floatx;

	inline floatx sinx(floatx val) { return sin(val); }
	inline floatx cosx(floatx val) { return cos(val); }
	inline floatx tanx(floatx val) { return tan(val); }
	inline floatx asinx(floatx val) { return asin(val); }
	inline floatx acosx(floatx val) { return acos(val); }
	inline floatx atanx(floatx val) { return atan(val); }
	inline floatx sinhx(floatx val) { return sinh(val); }
	inline floatx coshx(floatx val) { return cosh(val); }
	inline floatx tanhx(floatx val) { return tanh(val); }
	inline floatx asinhx(floatx val) { return asinh(val); }
	inline floatx acoshx(floatx val) { return acosh(val); }
	inline floatx atanhx(floatx val) { return atanh(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypot(v1,v2); }
	inline floatx truncx(floatx val) { return trunc(val); }
	inline floatx fabsx(floatx val) { return fabs(val); }
	inline floatx logx(floatx val) { return log(val); }
	inline floatx log2x(floatx val) { return log2(val); }
	inline floatx log10x(floatx val) { return log10(val); }
	inline floatx expx(floatx val) { return exp(val); }
	inline floatx sqrtx(floatx val) { return sqrt(val); }
	inline floatx powx(floatx v1,floatx v2) { return pow(v1,v2); }
#endif
#ifdef USE_LONG_DOUBLE
	typedef long double floatx;

	inline floatx sinx(floatx val) { return sinl(val); }
	inline floatx cosx(floatx val) { return cosl(val); }
	inline floatx tanx(floatx val) { return tanl(val); }
	inline floatx asinx(floatx val) { return asinl(val); }
	inline floatx acosx(floatx val) { return acosl(val); }
	inline floatx atanx(floatx val) { return atanl(val); }
	inline floatx sinhx(floatx val) { return sinhl(val); }
	inline floatx coshx(floatx val) { return coshl(val); }
	inline floatx tanhx(floatx val) { return tanhl(val); }
	inline floatx asinhx(floatx val) { return asinhl(val); }
	inline floatx acoshx(floatx val) { return acoshl(val); }
	inline floatx atanhx(floatx val) { return atanhl(val); }
	inline floatx atan2x(floatx v1,floatx v2) { return atan2l(v1,v2); }
	inline floatx hypotx(floatx v1,floatx v2) { return hypotl(v1,v2); }
	inline floatx truncx(floatx val) { return truncl(val); }
	inline floatx fabsx(floatx val) { return fabsl(val); }
	inline floatx logx(floatx val) { return logl(val); }
	inline floatx log2x(floatx val) { return log2l(val); }
	inline floatx log10x(floatx val) { return log10l(val); }
	inline floatx expx(floatx val) { return expl(val); }
	inline floatx sqrtx(floatx val) { return sqrtl(val); }
	inline floatx powx(floatx v1,floatx v2) { return powl(v1,v2); }
#endif

extern const floatx PI, E;

enum ErrorType { ErrorType_None,
                 ErrorType_Internal,
                 ErrorType_General,
                 ErrorType_Memory,
                 ErrorType_File_IO,
                 ErrorType_Domain,
                 ErrorType_Overflow,
                 ErrorType_Divide_by_zero,
                 ErrorType_Variable_not_found,
                 ErrorType_Name_isnt_unique,
                 ErrorType_UmpCode_General = 1000,
                 ErrorType_UmpCode_Domain,
                 ErrorType_UmpCode_Divide_by_zero };

const uint32 ERROR_OBJ_MSG_LEN = 200;
struct error_obj
{
	ErrorType type;
	char msg[ERROR_OBJ_MSG_LEN];
};



#endif // VARTYPES_H_
